package com.caimei365.order.components;

import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.PayOrderMapper;
import com.caimei365.order.model.dto.VipSkuPriceDto;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.utils.MathUtil;
import com.caimei365.order.utils.ImageUtil;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ArrayUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/29
 */
@Slf4j
@Service
public class ProductService {
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private PayOrderMapper payOrderMapper;

    private final List<Integer> provinceList = Arrays.asList(5, 26, 29, 31);

    /**
     * 设置商品图片及税费
     *
     * @param cartItemVo 购物车商品
     * @return taxFlag 是否需要设置税费
     */
    public boolean setCartItemImgAndTax(CartItemVo cartItemVo) {
        // 图片路径
        String image = ImageUtil.getImageUrl("product", cartItemVo.getImage(), domain);
        cartItemVo.setImage(image);
        // 是否添加税费，不含税商品 开票需添加税费
        boolean taxFlag = (Integer.valueOf(0).equals(cartItemVo.getIncludedTax()) && (Integer.valueOf(1).equals(cartItemVo.getInvoiceType()) || Integer.valueOf(2).equals(cartItemVo.getInvoiceType())));
        if (taxFlag) {
            BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(cartItemVo.getPrice(), cartItemVo.getTaxRate()), 100, 2);
            BigDecimal originalTax = MathUtil.div(MathUtil.mul(cartItemVo.getOriginalPrice(), cartItemVo.getTaxRate()), 100, 2);
            cartItemVo.setPrice(MathUtil.add(cartItemVo.getPrice(), cartItemTax).doubleValue());
            cartItemVo.setOriginalPrice(MathUtil.add(cartItemVo.getOriginalPrice(), originalTax).doubleValue());
        }
        return taxFlag;
    }

    /**
     * 设置购物车阶梯价
     *
     * @param cartItemVo
     * @param taxFlag
     */
    public void setCartLadderPrices(CartItemVo cartItemVo, boolean taxFlag) {
        // 阶梯价列表
        List<LadderPriceVo> ladderPrices = baseMapper.getLadderPriceList(cartItemVo.getSkuId());
        if (!CollectionUtils.isEmpty(ladderPrices)) {
            IntStream.range(0, ladderPrices.size()).forEach(i -> {
                boolean isThisLadder;
                // 添加税费
                if (taxFlag) {
                    BigDecimal addedValueTax = MathUtil.div(MathUtil.mul(ladderPrices.get(i).getBuyPrice(), cartItemVo.getTaxRate()), 100, 2);
                    ladderPrices.get(i).setBuyPrice(MathUtil.add(ladderPrices.get(i).getBuyPrice(), addedValueTax).doubleValue());
                }
                if (i == ladderPrices.size() - 1) {
                    ladderPrices.get(i).setMaxNum(0);
                    ladderPrices.get(i).setNumRange("≥" + ladderPrices.get(i).getBuyNum());
                    isThisLadder = (cartItemVo.getNumber() >= ladderPrices.get(i).getBuyNum());
                } else {
                    ladderPrices.get(i).setMaxNum(ladderPrices.get(i + 1).getBuyNum());
                    String buyNumRangeShow = ladderPrices.get(i).getBuyNum() + "~" + (ladderPrices.get(i + 1).getBuyNum() - 1);
                    ladderPrices.get(i).setNumRange(buyNumRangeShow);
                    isThisLadder = (cartItemVo.getNumber() >= ladderPrices.get(i).getBuyNum() && cartItemVo.getNumber() <= ladderPrices.get(i).getMaxNum());
                }
                if (isThisLadder) {
                    cartItemVo.setPrice(ladderPrices.get(i).getBuyPrice());
                    cartItemVo.setOriginalPrice(ladderPrices.get(i).getBuyPrice());
                }
            });
            cartItemVo.setMin(ladderPrices.get(0).getBuyNum());
            cartItemVo.setLadderPrices(ladderPrices);
        } else {
            cartItemVo.setLadderFlag(0);
        }
    }

    /**
     * 当前促销活动的价格计算列表
     *
     * @param promotions 促销活动
     * @param cartItemVo 当前购物车商品
     * @param taxFlag    计算税费标志
     * @return 价格计算列表
     */
    public List<PromotionPriceVo> getPromotionProducts(PromotionsVo promotions, CartItemVo cartItemVo, boolean taxFlag) {
        List<PromotionPriceVo> promotionPriceList = new ArrayList<>();
        PromotionPriceVo promotionPrice = new PromotionPriceVo();
        promotionPrice.setProductId(cartItemVo.getProductId());
        //购买数量
        promotionPrice.setNumber(cartItemVo.getNumber());
        // 设置价格
        if (promotions.getType() == 1 && promotions.getMode() == 1) {
            // 查询当前sku的优惠价
            Double touchPrice = baseMapper.getTouchPriceBySku(cartItemVo.getSkuId(), promotions.getId());
            promotions.setTouchPrice(touchPrice);
            // 单品优惠价添加税费
            if (taxFlag) {
                BigDecimal addedValueTax = MathUtil.div(MathUtil.mul(promotions.getTouchPrice(), cartItemVo.getTaxRate()), BigDecimal.valueOf(100), 2);
                promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(), addedValueTax).doubleValue());
            }
            promotionPrice.setPrice(promotions.getTouchPrice());
            // 划线价/原价
            promotionPrice.setOriginalPrice(cartItemVo.getOriginalPrice());
            // 设置商品活动价
            cartItemVo.setPrice(promotions.getTouchPrice());
        } else {
            promotionPrice.setPrice(cartItemVo.getPrice());
        }
        promotionPriceList.add(promotionPrice);
        return promotionPriceList;
    }

    /**
     * 把当前促销更新购物车总促销
     *
     * @param totalPromotions    总促销列表
     * @param promotionsIds      总促销Id集合
     * @param promotions         当前促销
     * @param promotionPriceList 当前促销商品价格计算列表
     */
    public void updateTotalPromotions(List<PromotionsVo> totalPromotions, List<Integer> promotionsIds, PromotionsVo promotions, List<PromotionPriceVo> promotionPriceList, Double floor) {
        if (promotionsIds.contains(promotions.getId())) {
            // 列表已有该促销活动
            totalPromotions.forEach(item -> {
                if (item.getId().equals(promotions.getId())) {
                    promotionPriceList.addAll(item.getProductList());
                    item.setProductList(promotionPriceList);
                }
            });
        } else {
            // 新的促销活动
            promotionsIds.add(promotions.getId());
            if (promotions.getMode() == 3) {
                // 获取赠品
                List<CartItemVo> giftList = baseMapper.getPromotionGifts(promotions.getId());
                // 符合条件赠品翻倍
                if (floor != null && promotions.getDiscount() != null && floor > 1 && promotions.getDiscount() == 1) {
                    Integer in = floor.intValue();
                    giftList.forEach(g -> g.setNumber(MathUtil.mul(g.getNumber(), in).intValue()));
                }
                promotions.setGiftList(giftList);
            }
            promotions.setProductList(promotionPriceList);
            // 添加到总促销
            totalPromotions.add(promotions);
        }
    }

    /**
     * 是否订单充值商品或定金商品
     *
     * @param productId 商品id
     * @return Boolean
     */
    public boolean isRechargeProduct(Integer productId) {
        int[] products = {6060, 6061, 6062, 6063, 6064, 6065, 6066, 6067, 6068, 6069};
        return ArrayUtils.contains(products, productId);
    }

    /**
     * 计算运费
     *
     * @param userId    用户ID
     * @param cityId    地区市Id
     * @param num       购买数量
     * @param amount    购买商品金额
     * @param skuIdList 商品Id列表
     */
    public Map<String, Object> computePostage(Integer userId, Integer cityId, Integer num, Double amount, List<String> skuIdList) {
        // 用户组织Id
        Integer organizeId = baseMapper.getOrganizeId(userId);
        // 返回数据初始化
        Map<String, Object> postageMap = new HashMap<>(5);
        // 运费标志：0包邮 1到付 2使用运费模板
        int postageFlag = 0;
        // 运费
        AtomicDouble postage = new AtomicDouble(0);
        // 可用采美豆
        Integer userBeans = baseMapper.getUserBeans(userId);
        postageMap.put("userBeans", userBeans);
        // 超级会员标识
        Integer superVipUserId = baseMapper.getSvipUserIdByUserId(userId);
        if (null != superVipUserId) {
            // 超级VIP包邮
            postageMap.put("postageFlag", 0);
            postageMap.put("postage", 0.00d);
            return postageMap;
        }
        // 获取商品运费 （商品数据库 0包邮 2默认（遵循运费模板） 2到付 ）
        List<ProductPostageVo> postageFlagList = null;
        if (0 == organizeId) {
            postageFlagList = baseMapper.getPostageFlagList(skuIdList);
        } else {
            postageFlagList = baseMapper.getPostageFlagOrganizeList(skuIdList);
        }
        List<Integer> commodityTypeList = null;
        if (null != postageFlagList && postageFlagList.size() > 0) {
            commodityTypeList = postageFlagList.stream().map(ProductPostageVo::getCommodityType).collect(Collectors.toList());
            for (ProductPostageVo postageVo : postageFlagList) {
                if (null != postageVo) {
                    // 设置了运费到付
                    if ((null != postageVo.getPostageFlag() && 2 == postageVo.getPostageFlag())) {
                        // 到付
                        postageFlag = 2;
                    } else if (null != postageVo.getPostageFlag() && 0 == postageVo.getPostageFlag()) {
                        // 包邮
                        postageFlag = 0;
                    } else {
                        // 使用运费模板
                        postageFlag = 1;
                        if (null == postageVo.getFreightTemplateId()) {
                            // 商品运费规则异常
                            postageMap.put("postageFlag", 999);
                            return postageMap;
                        } else {
                            Integer freightTemplate = baseMapper.getFreightTemplate(postageVo.getFreightTemplateId());
                            if (null == freightTemplate) {
                                // 商品运费规则异常
                                postageMap.put("postageFlag", 999);
                                return postageMap;
                            }
                            FreightRuleVo freightRule = baseMapper.getFreightRule(postageVo.getFreightTemplateId(), cityId);
                            if (null != freightRule) {
                                postage.set(MathUtil.add(postage.get(), freightRule.getFreightAmount()).doubleValue());
                                // 含有包邮的规则
                                if (1 == freightRule.getFreeType()) {
                                    if (0 == freightRule.getFreeCondition()) {
                                        // 满足数量 包邮
                                        if (num >= freightRule.getProductNum()) {
                                            postage.set(MathUtil.sub(postage.get(), 0).doubleValue());
                                            postageFlag = 0;
                                        }
                                    } else {
                                        // 满足金额 包邮
                                        if (amount >= freightRule.getAllAmount()) {
                                            postage.set(MathUtil.sub(postage.get(), 0).doubleValue());
                                            postageFlag = 0;
                                        }
                                    }
                                }
                            } else {
                                // 未选择地区包邮
                                postageFlag = 0;
                            }
                        }
                    }
                }
            }
        }
        // 商品包邮
        if (0 == postageFlag) {
            // 包邮
            postage.set(0d);
        }
        // 偏远地区 获取地址的 省地址
        Integer province = baseMapper.getProvince(cityId);
        // 是否首单（统计订单数）
        Integer count = null;
        if (0 == organizeId) {
            // 采美
            count = baseMapper.countUserOrder(userId);
        } else {
            // 组织 -- 联合丽格无首单优惠
            count = -1;
        }
        // 首单包邮条件：首单 且 不包含仪器 且不包含偏远地区
        boolean freeFlag = ((null != count && count == 0) && (null != commodityTypeList && !commodityTypeList.contains(2))
                && (null != province && !provinceList.contains(province)));
        if (freeFlag) {
            // 包邮
            postageFlag = 0;
            postage.set(0d);

        }
        postageMap.put("postageFlag", postageFlag);
        postageMap.put("postage", postage.get());
        return postageMap;
    }

    /**
     * 更新复购价格库
     *
     * @param order        主订单
     * @param orderProduct 订单商品
     * @param secondFlag   是否二手商品
     * @param note         日志备注
     */
    public void savePurchasePrice(OrderPo order, OrderProductPo orderProduct, boolean secondFlag, String note) {
        // 非二手订单, 非活动商品，非阶梯价商品，非运费商品，非赠品 才可以写入复购价
        boolean purchaseFlag = ((!secondFlag) && Integer.valueOf(0).equals(orderProduct.getActProduct()) && orderProduct.getShopId() != 998 && orderProduct.getPrice() > 0);
        boolean historyFlag = false;
        // 不是二手订单才可以写入价格库
        if (purchaseFlag) {
            Date date = new Date();
            String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
            // 机构价大于成本价   ---成本价已经在整理订单商品时计算好（固定或比例）
            boolean costFlag = MathUtil.compare(orderProduct.getPrice(), orderProduct.getCostPrice()) > 0;
            //  查询当前商品复购价信息
            PurchasePricePo purchase = baseMapper.getPurchasePricePo(order.getUserId(), orderProduct.getSkuId());
            // 已有复购价
            if (null != purchase) {
                // 复购价大于机构价（降价）
                boolean priceFlag1 = MathUtil.compare(purchase.getCurrentPrice(), orderProduct.getPrice()) > 0;
                // 成本价大于复购价（亏本）
                boolean priceFlag2 = MathUtil.compare(orderProduct.getCostPrice(), purchase.getCurrentPrice()) > 0;
                if (priceFlag2 || (priceFlag1 && costFlag)) {
                    // 更新价格库复购价
                    purchase.setCurrentPrice(orderProduct.getPrice());
                    purchase.setTaxRate(orderProduct.getTaxRate());
                    purchase.setUpdateTime(curDateStr);
                    purchase.setOrderId(orderProduct.getOrderId());
                    // 更新复购价格库
                    baseMapper.updatePurchasePrice(purchase);
                    historyFlag = true;
                    log.info(note + ">>>>>>>>>>>更新复购价格库(update[repeat_purchase_price])ProductId：" + orderProduct.getProductId() + "，orderId：" + order.getOrderId());
                }
            } else {
                // 没有复购价
                if (costFlag) {
                    // 新增复购价到价格库
                    purchase = new PurchasePricePo();
                    purchase.setUserId(order.getUserId());
                    purchase.setClubId(order.getClubId());
                    purchase.setOrderId(order.getOrderId());
                    purchase.setProductId(orderProduct.getProductId());
                    purchase.setShopId(orderProduct.getShopId());
                    purchase.setShopName(orderProduct.getShopName());
                    purchase.setCurrentPrice(orderProduct.getPrice());
                    purchase.setTaxRate(orderProduct.getTaxRate());
                    purchase.setCreateTime(curDateStr);
                    purchase.setUpdateTime(curDateStr);
                    purchase.setDelFlag(0);
                    purchase.setSkuId(orderProduct.getSkuId());
                    // 新增复购价格库
                    baseMapper.insertPurchasePrice(purchase);
                    historyFlag = true;
                    log.info(note + ">>>>>>>>>>>新增复购价格库(insert[repeat_purchase_price])skuId：" + orderProduct.getSkuId() + "，orderId：" + order.getOrderId());
                }
            }
            if (historyFlag) {
                // 历史复购价记录
                PurchaseHistoryPo purchaseHistory = new PurchaseHistoryPo();
                purchaseHistory.setUserId(order.getUserId());
                purchaseHistory.setClubId(order.getClubId());
                purchaseHistory.setOrderId(order.getOrderId());
                purchaseHistory.setProductId(orderProduct.getProductId());
                purchaseHistory.setCurrentCostPrice(orderProduct.getCostPrice());
                purchaseHistory.setPrice(orderProduct.getPrice());
                purchaseHistory.setCreateTime(curDateStr);
                purchaseHistory.setSkuId(orderProduct.getSkuId());
                purchaseHistory.setDelFlag(0);
                baseMapper.insertPurchaseHistory(purchaseHistory);
                log.info(note + ">>>>>>>>>>>新增历史复购价记录(insert[repeat_purchase_price_history])skuId：" + orderProduct.getSkuId() + "，orderId：" + order.getOrderId());
            }
        }
    }

    /**
     * 保存余额抵扣到收款记录
     *
     * @param balancePayFee 余额抵扣
     * @param shopOrderList 主订单Id
     * @param note          日志备注
     */
    public void saveDiscernReceipt(Double balancePayFee, List<OrderShopPo> shopOrderList, String note) {
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        ShopOrderVo shopOrderVo = new ShopOrderVo();
        AtomicReference<Double> ba = new AtomicReference<>(balancePayFee);
        shopOrderList.forEach(shopOrder -> {
            double shopOrderPay = 0d;
            if (MathUtil.compare(balancePayFee, shopOrder.getNeedPayAmount()) >= 0) {
                shopOrder.setReceiptStatus(3);
                shopOrderPay = shopOrder.getNeedPayAmount();
            } else {
                shopOrder.setReceiptStatus(2);
                shopOrderPay = ba.get();
            }
            ba.updateAndGet(v -> ba.get() - shopOrder.getNeedPayAmount());
            // 识别款项表
            DiscernReceiptPo discernReceipt = new DiscernReceiptPo();
            discernReceipt.setPayWay(3);
            discernReceipt.setPayType(16);
            discernReceipt.setReceiptType(1);
            discernReceipt.setReceiptStatus(3);
            discernReceipt.setReceiptAmount(shopOrderPay);
            discernReceipt.setConfirmType(4);
            discernReceipt.setReceiptDate(curDateStr);
            discernReceipt.setConfirmDate(curDateStr);
            discernReceipt.setReviewDate(curDateStr);
            discernReceipt.setUpdateDate(curDateStr);
            discernReceipt.setDelFlag(0);
            // 保存 收款记录
            baseMapper.insertDiscernReceipt(discernReceipt);
            log.info(note + ">>>>>>>>>>>保存识别款项(insert[cm_discern_receipt])id：" + discernReceipt.getId() + "，shopOrderId：" + shopOrder.getShopOrderId());
            // 收款项和订单关系表
            OrderReceiptRelationPo relation = new OrderReceiptRelationPo();
            relation.setReceiptId(discernReceipt.getId());
            relation.setOrderId(shopOrder.getOrderId());
            relation.setShopOrderId(shopOrder.getShopOrderId());
            relation.setAssociateAmount(shopOrderPay);
            relation.setRelationType(2);
            relation.setDelFlag(0);
            // 保存 收款项和订单关系
            baseMapper.insertOrderReceiptRelation(relation);
            //保存余额到余额收支记录
            BalanceRecordPo balanceRecord = new BalanceRecordPo();
            balanceRecord.setUserId(shopOrder.getUserId());
            balanceRecord.setType(2);
            balanceRecord.setBalanceType(1);
            balanceRecord.setAddDate(new Date());
            balanceRecord.setAmount(shopOrderPay);
            balanceRecord.setOrderId(shopOrder.getOrderId());
            balanceRecord.setShopOrderId(shopOrder.getShopOrderId());
            balanceRecord.setRemark("机构自主订单");
            balanceRecord.setDelFlag(0);
            // 保存 余额收支记录
            baseMapper.insertBalanceRecord(balanceRecord);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增用户余额收支记录(insert[cm_user_balance_record])shopOrderId：" + shopOrder.getShopOrderId());
            log.info(note + ">>>>>>>>>>>收款项和订单关系(insert[cm_receipt_order_relation])id：" + relation.getId() + "，shopOrderId：" + shopOrder.getShopOrderId());
            // 修改子订单状态
            shopOrderVo.setShopOrderId(shopOrder.getShopOrderId());
            // 订单状态
            shopOrderVo.setReceiptStatus(shopOrder.getReceiptStatus());
            // 余额抵扣金额
            shopOrderVo.setAccountAmount(shopOrderPay);
            // 用户已付金额
            shopOrderVo.setReceiptAmount(shopOrderPay);
            // 余额抵扣状态
            shopOrderVo.setUseBalanceFlag(1);
            payOrderMapper.updateShopOrderBalancePay(shopOrderVo);

        });

    }

    /**
     * 保存余额抵扣到收款记录
     * @param balancePayFee
     * @param shopOrder
     * @param note
     */
    public void saveShopOrderDiscernReceipt(Double balancePayFee, ShopOrderVo shopOrder, String note) {
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        AtomicReference<Double> ba = new AtomicReference<>(balancePayFee);
        double shopOrderPay = 0d;
        if (MathUtil.compare(balancePayFee, shopOrder.getNeedPayAmount()) >= 0) {
            shopOrder.setReceiptStatus(3);
            shopOrderPay = shopOrder.getNeedPayAmount();
        } else {
            shopOrder.setReceiptStatus(2);
            shopOrderPay = ba.get();
        }
        ba.updateAndGet(v -> ba.get() - shopOrder.getNeedPayAmount());
        // 识别款项表
        DiscernReceiptPo discernReceipt = new DiscernReceiptPo();
        discernReceipt.setPayWay(3);
        discernReceipt.setPayType(16);
        discernReceipt.setReceiptType(1);
        discernReceipt.setReceiptStatus(3);
        discernReceipt.setReceiptAmount(shopOrderPay);
        discernReceipt.setConfirmType(4);
        discernReceipt.setReceiptDate(curDateStr);
        discernReceipt.setConfirmDate(curDateStr);
        discernReceipt.setReviewDate(curDateStr);
        discernReceipt.setUpdateDate(curDateStr);
        discernReceipt.setDelFlag(0);
        // 保存 收款记录
        baseMapper.insertDiscernReceipt(discernReceipt);
        log.info(note + ">>>>>>>>>>>保存识别款项(insert[cm_discern_receipt])id：" + discernReceipt.getId() + "，shopOrderId：" + shopOrder.getShopOrderId());
        // 收款项和订单关系表
        OrderReceiptRelationPo relation = new OrderReceiptRelationPo();
        relation.setReceiptId(discernReceipt.getId());
        relation.setOrderId(shopOrder.getOrderId());
        relation.setShopOrderId(shopOrder.getShopOrderId());
        relation.setAssociateAmount(shopOrderPay);
        relation.setRelationType(2);
        relation.setDelFlag(0);
        // 保存 收款项和订单关系
        baseMapper.insertOrderReceiptRelation(relation);
        //保存余额到余额收支记录
        BalanceRecordPo balanceRecord = new BalanceRecordPo();
        balanceRecord.setUserId(shopOrder.getUserId());
        balanceRecord.setType(2);
        balanceRecord.setBalanceType(1);
        balanceRecord.setAddDate(new Date());
        balanceRecord.setAmount(shopOrderPay);
        balanceRecord.setOrderId(shopOrder.getOrderId());
        balanceRecord.setShopOrderId(shopOrder.getShopOrderId());
        balanceRecord.setRemark("协销订单");
        balanceRecord.setDelFlag(0);
        // 保存 余额收支记录
        baseMapper.insertBalanceRecord(balanceRecord);
        log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增用户余额收支记录(insert[cm_user_balance_record])shopOrderId：" + shopOrder.getShopOrderId());
        log.info(note + ">>>>>>>>>>>收款项和订单关系(insert[cm_receipt_order_relation])id：" + relation.getId() + "，shopOrderId：" + shopOrder.getShopOrderId());

        // 修改子订单状态
        // 子订单余额抵扣金额
        shopOrder.setAccountAmount(MathUtil.add(shopOrder.getAccountAmount(),shopOrderPay).doubleValue());
        // 用户已付金额
        shopOrder.setReceiptAmount(MathUtil.add(shopOrder.getReceiptAmount(),shopOrderPay).doubleValue());
        // 余额抵扣状态
        shopOrder.setUseBalanceFlag(1);
        payOrderMapper.updateShopOrderBalancePay(shopOrder);

    }

    /**
     * 更新用户采美豆
     *
     * @param beansHistory 采美豆使用记录
     * @param userBeans    用户原有采美豆数量
     * @param note         日志备注
     */
    public void updateUserBeans(UserBeansHistoryPo beansHistory, Integer userBeans, String note) {
        // 保存 采美豆使用记录
        baseMapper.insertBeansHistory(beansHistory);
        baseMapper.updateUserBeans(beansHistory.getUserId(), userBeans);
        log.info(note + ">>>>>更新用户采美豆(update[user(userBeans:" + userBeans + ")])，userId:" + beansHistory.getUserId());
    }

    /**
     * 设置超级会员优惠商品价格
     *
     * @param cartItemVo   当前购物车商品
     * @param taxFlag      计算税费标志
     * @param svipUserFlag
     */
    public void setSvipProductPrice(CartItemVo cartItemVo, boolean taxFlag, boolean svipUserFlag) {
        //超级会员价按sku取
        VipSkuPriceDto vipSku = baseMapper.getSvipSku(cartItemVo.getSkuId());
        Double discountPrice = 0d;
        if (1 == vipSku.getPriceType()) {
            // 折扣价
            cartItemVo.setSvipPriceTag(MathUtil.div(vipSku.getDiscount(), 10, 1) + "折");
            // 设置折扣价（原价格已计算税费，无需重复计算）
            discountPrice = MathUtil.div(MathUtil.mul(cartItemVo.getPrice(), vipSku.getDiscount()), 100, 2).doubleValue();
        } else if (2 == vipSku.getPriceType()) {
            // 直接优惠价
            discountPrice = vipSku.getDiscountPrice();
            if (taxFlag) {
                //添加税费
                BigDecimal thisTaxFee = MathUtil.div(MathUtil.mul(discountPrice, cartItemVo.getTaxRate()), 100, 2);
                discountPrice = MathUtil.add(discountPrice, thisTaxFee).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            }
            cartItemVo.setSvipPriceTag("￥" + discountPrice);
        }
        if (svipUserFlag) {
            // 超级会员设置优惠价
            cartItemVo.setPrice(discountPrice);
            // 设置优惠金额=（商品原价-折扣价）*商品数量
            cartItemVo.setSvipTotalReducedPrice(MathUtil.mul(MathUtil.sub(cartItemVo.getOriginalPrice(), discountPrice), cartItemVo.getNumber()).doubleValue());
        }
    }

    public void saveShopOrderDiscernReceipt(Double balancePayFee, ShopOrderVo order) {
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        // 识别款项表
        DiscernReceiptPo discernReceipt = new DiscernReceiptPo();
        discernReceipt.setPayWay(3);
        discernReceipt.setPayType(16);
        discernReceipt.setReceiptType(1);
        discernReceipt.setReceiptStatus(3);
        discernReceipt.setReceiptAmount(balancePayFee);
        discernReceipt.setConfirmType(4);
        discernReceipt.setReceiptDate(curDateStr);
        discernReceipt.setConfirmDate(curDateStr);
        discernReceipt.setReviewDate(curDateStr);
        discernReceipt.setUpdateDate(curDateStr);
        discernReceipt.setDelFlag(0);
        // 保存 收款记录
        baseMapper.insertDiscernReceipt(discernReceipt);
        log.info(">>>>>>>>>>>保存识别款项(insert[cm_discern_receipt])id：" + discernReceipt.getId() + "，shopOrderId：" + order.getShopOrderId());
        // 收款项和订单关系表
        OrderReceiptRelationPo relation = new OrderReceiptRelationPo();
        relation.setReceiptId(discernReceipt.getId());
        relation.setOrderId(order.getOrderId());
        relation.setShopOrderId(order.getShopOrderId());
        relation.setAssociateAmount(balancePayFee);
        relation.setRelationType(2);
        relation.setDelFlag(0);
        // 保存 收款项和订单关系
        baseMapper.insertOrderReceiptRelation(relation);
        log.info(">>>>>>>>>>>收款项和订单关系(insert[cm_receipt_order_relation])id：" + relation.getId() + "，shopOrderId：" + order.getShopOrderId());
        //保存余额到余额收支记录
        BalanceRecordPo balanceRecord = new BalanceRecordPo();
        balanceRecord.setUserId(order.getUserId());
        balanceRecord.setType(2);
        balanceRecord.setBalanceType(1);
        balanceRecord.setAddDate(new Date());
        balanceRecord.setAmount(balancePayFee);
        balanceRecord.setOrderId(order.getOrderId());
        balanceRecord.setShopOrderId(order.getShopOrderId());
        balanceRecord.setRemark("订单支付,余额抵扣");
        balanceRecord.setDelFlag(0);
        // 保存 余额收支记录
        baseMapper.insertBalanceRecord(balanceRecord);
        log.info("【订单支付,余额抵扣】>>>>>>>>>>>>>>>>>>>>>>>>>>新增用户余额收支记录(insert[cm_user_balance_record])shopOrderId：" + order.getShopOrderId());
    }
}
