package com.caimei365.order.components;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONType;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.utils.RequestUtil;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang.StringUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Component;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.security.AlgorithmParameters;
import java.security.Security;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 微信 服务工具类
 *
 * @author : Charles
 * @date : 2021/3/9
 */
@Slf4j
@Component
public class WeChatService {

    @Value("${wx.redirect-uri}")
    private String redirectUri;
    @Value("${wx.app-id}")
    private String appId;
    @Value("${wx.app-secret}")
    private String appSecret;
    @Value("${wx.mini-app-id}")
    private String miniAppId;
    @Value("${wx.mini-app-secret}")
    private String miniAppSecret;
    @Value("${wx.receipt-app-id}")
    private String receiptAppId;
    @Value("${wx.receipt-app-secret}")
    private String receiptAppSecret;
    @Value("${wx.crm-app-id}")
    private String crmAppId;
    @Value("${wx.crm-app-secret}")
    private String crmAppSecret;
    @Value("${wx.hehe-app-id}")
    private String heHeAppId;
    @Value("${wx.hehe-app-secret}")
    private String heHeAppSecret;
    @Value("${wx.mcare-app-id}")
    private String mcareAppId;
    @Value("${wx.mcare-app-secret}")
    private String mcareAppSecret;

    /**
     * 收款通知模板Id
     */
    @Value("${wx.message-template.receipt}")
    private String messageTemplateReceipt;

    private RedisService redisService;
    @Autowired
    public void setRedisService(RedisService redisService) {
        this.redisService = redisService;
    }

    public static class Keys {
        public static final String OPEN_ID = "openid";
        public static final String UNION_ID = "unionid";
        public static final String SESSION_KEY = "session_key";
    }

    static {
        // BouncyCastle是一个开源的加解密解决方案
        Security.addProvider(new BouncyCastleProvider());
    }

    /**
     * AES解密
     *
     * @param data           密文，被加密的数据
     * @param key            秘钥
     * @param iv             偏移量
     * @param encodingFormat 解密后的结果需要进行的编码
     * @return 解密结果
     */
    public static String decrypt(String data, String key, String iv, String encodingFormat) throws Exception {
        // 被加密的数据
        byte[] dataByte = Base64.decodeBase64(data);
        // 加密秘钥
        byte[] keyByte = Base64.decodeBase64(key);
        // 偏移量
        byte[] ivByte = Base64.decodeBase64(iv);
        try {
            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS7Padding");
            SecretKeySpec spec = new SecretKeySpec(keyByte, "AES");
            AlgorithmParameters parameters = AlgorithmParameters.getInstance("AES");
            parameters.init(new IvParameterSpec(ivByte));
            // 初始化
            cipher.init(Cipher.DECRYPT_MODE, spec, parameters);
            byte[] resultByte = cipher.doFinal(dataByte);
            if (null != resultByte && resultByte.length > 0) {
                return new String(resultByte, encodingFormat);
            }
            return null;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 小程序微信授权登录,获取openid
     *
     * @param code    微信凭证
     * @param headers HttpHeaders
     * @param mode    1:采美小程序,2:收款工具小程序appId
     * @return HashMap
     */
    public ResponseJson<Map<String, Object>> getInfoMapByApplets(String code, HttpHeaders headers, Integer mode) {
        log.info("Start get SessionKey");
        Map<String, Object> returnMap = new HashMap<>(4);
        // 获取当前微信小程序的环境
        String referer = headers.getFirst("Referer");
        log.info("referer-is----:" + referer);
        returnMap.put("referer", referer);
        String requestUrl = "https://api.weixin.qq.com/sns/jscode2session";
        Map<String, String> requestUrlParam = new HashMap<String, String>(4);
        if (mode == 1) {
            // 小程序appId
            requestUrlParam.put("appid", miniAppId);
            log.info("采美小程序appId: ---" + miniAppId);
            // 小程序appSecret
            requestUrlParam.put("secret", miniAppSecret);
        } else if (mode == 2) {
            // 收款工具小程序appId
            requestUrlParam.put("appid", receiptAppId);
            log.info("收款工具小程序appId: ---" + receiptAppId);
            // 收款工具小程序appSecret
            requestUrlParam.put("secret", receiptAppSecret);
        } else if (mode == 3) {
            // 联合丽格小程序appId
            requestUrlParam.put("appid", mcareAppId);
            log.info("联合丽格小程序appId: ---" + mcareAppId);
            // 联合丽格小程序appSecret
            requestUrlParam.put("secret", mcareAppSecret);
        } else {
            // 呵呵商城小程序appId
            requestUrlParam.put("appid", heHeAppId);
            log.info("呵呵商城appId: ---" + heHeAppId);
            // 呵呵商城小程序appSecret
            requestUrlParam.put("secret", heHeAppSecret);
        }
        // 小程序端返回的code
        requestUrlParam.put("js_code", code);
        // 默认参数
        requestUrlParam.put("grant_type", "authorization_code");
        // 发送post请求读取调用微信接口获取openid用户唯一标识
        String infos;
        try {
            infos = RequestUtil.sendPost(requestUrl, requestUrlParam);
        } catch (Exception e) {
            return ResponseJson.error("服务器内部异常", returnMap);
        }
        // 解析相应内容（转换成json对象）
        JSONObject jsonObject = JSON.parseObject(infos);
        String openId = jsonObject.getString(Keys.OPEN_ID);
        String unionId = jsonObject.getString(Keys.UNION_ID);
        String sessionKey = jsonObject.getString(Keys.SESSION_KEY);
        String errCode = jsonObject.getString("errcode");
        String errMsg = jsonObject.getString("errmsg");
        log.info("openId----->" + openId + ", unionId------>" + unionId);
        returnMap.put(Keys.OPEN_ID, openId);
        returnMap.put(Keys.UNION_ID, unionId);
        returnMap.put(Keys.SESSION_KEY, sessionKey);
        boolean errFlag = StringUtils.isNotEmpty(errCode) && ("-1".equals(errCode) || "40029".equals(errCode) || "45011".equals(errCode));
        if (errFlag) {
            return ResponseJson.error(-1, errMsg, returnMap);
        }
        return ResponseJson.success(returnMap);
    }

    /**
     * 微信公众号授权链接(www)
     *
     * @param redirectUri 用于微信授权的中间页面
     * @param mode        授权方式：1静默授权，其他手动同意授权
     * @return newRedirectUri
     */
    public String getAuthorizationLink(String redirectUri, Integer mode) {
        String link = "https://open.weixin.qq.com/connect/oauth2/authorize?appid=APPID&redirect_uri=REDIRECT_URI&response_type=code&scope=snsapi_userinfo&state=STATE#wechat_redirect";
        // 应用唯一标识
        link = link.replace("APPID", crmAppId);
        // 中间页连接
        link = link.replace("REDIRECT_URI", redirectUri);
        if (mode == 1) {
            // 静默授权
            link = link.replace("snsapi_userinfo", "snsapi_base");
        }
        log.info("》》》》授权消息"+link);
        return link;
    }

    /**
     * 网页授权登录,通过code获取openid
     *
     * @param code   微信code
     * @param source 来源
     * @return
     */
    public Map<String, Object> getInfoMapByWeb(String code, String source) throws Exception {
        String link = "https://api.weixin.qq.com/sns/oauth2/access_token?appid=APPID&secret=SECRET&code=CODE&grant_type=authorization_code";
        if ("pc".equals(source)) {
            // 应用唯一标识
            link = link.replace("APPID", appId);
            // 应用密钥AppSecret，在微信开放平台提交应用审核通过后获得
            link = link.replace("SECRET", appSecret);
        } else {
            //微信公众号
            link = link.replace("APPID", crmAppId);
            link = link.replace("SECRET", crmAppSecret);
        }
        // 获取的code参数
        link = link.replace("CODE", code);
        // 发送授权链接，得到微信用户信息
        String result = RequestUtil.sendGet(link);
        log.info("用户信息获取成功"+result);
        Map<String, Object> map = JSONObject.parseObject(result, Map.class);
        return map;
    }

    /**
     * 微信公众号获取access_token
     *
     * @return access_token
     */
    public String getAccessToken() throws Exception {
//        String access_token = String.valueOf(redisService.get("access_token:"+crmAppId));
//        // 过期后会得到"null"值，所以需判断字符串"null"
//        if (access_token != null && access_token.length() != 0 && !"null".equals(access_token)) {
//            return access_token;
//        }
        String link = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=APPID&secret=APPSECRET";
        link = link.replace("APPID", crmAppId);
        link = link.replace("APPSECRET", crmAppSecret);
        String result = RequestUtil.sendGet(link);
        log.info("微信公众号获取access_token>>>" + result);
        Map<String, Object> map = JSONObject.parseObject(result, Map.class);
        String access_token = (String) map.get("access_token");
//        // 将token存入redis, access_token的有效期目前为2个小时(redis存1.5小时)
//        redisService.set("access_token:"+crmAppId, access_token, 5400L);
        return access_token;
    }

    /**
     * 微信公众号获取用户信息
     *
     * @return
     */
    public Map<String, Object> getUserInfo(String accessToken, String openId) throws Exception {
        String requestUrl = "https://api.weixin.qq.com/cgi-bin/user/info?access_token=" + accessToken + "&openid=" + openId + "&lang=zh_CN";
        String userInfo = RequestUtil.sendGet(requestUrl);
        log.info("微信公众号授权用户数据>>>>>>>>>>>" + userInfo);
        Map<String, Object> map = JSONObject.parseObject(userInfo, Map.class);
        return map;

    }

    /**
     * 网页授权登录,获取用户信息（UnionID机制）
     */
    public Map<String, Object> getUserInfoByWeb(String access_token, String openId) throws Exception {
        String requestUrl = "https://api.weixin.qq.com/sns/userinfo?access_token=" + access_token + "&openid=" + openId;
        String userInfo = RequestUtil.sendGet(requestUrl);
        log.info("获取用户信息"+userInfo);
        Map<String, Object> map = JSONObject.parseObject(userInfo, Map.class);
        return map;
    }


    /**
     * 推送收款微信模板消息(公众号)
     *
     * 模板内容:
     * {{first.DATA}}
     * 交易金额：{{keyword1.DATA}}
     * 交易时间：{{keyword2.DATA}}
     * {{remark.DATA}}
     * 在发送时，需要将内容中的参数（{{.DATA}}内为参数）赋值替换为需要的信息
     * @param accessToken 微信公众号 access_token
     * @param openid 被推送人微信公众号的openid
     * @param title 标题
     * @param money 金额
     * @param time 日期时间
     * @param remarkText 备注文本：收款类型+确认时间
     * @param pagePath 小程序跳转链接
     */
    public void sendTemplateMsg(String accessToken, String openid, String title, Double money, String time, String remarkText, String pagePath) {
        JSONObject first = new JSONObject();
        first.put("value", title);
        first.put("color", "#000000");
        JSONObject keyword1 = new JSONObject();
        keyword1.put("value", money.toString());
        keyword1.put("color", "#FF0000");
        JSONObject keyword2 = new JSONObject();
        keyword2.put("value", time);
        keyword2.put("color", "#c4c400");
        JSONObject remark = new JSONObject();
        remark.put("value", remarkText);
        remark.put("color", "#c4c400");

        JSONObject data = new JSONObject();
        data.put("first", first);
        data.put("keyword1", keyword1);
        data.put("keyword2", keyword2);
        data.put("remark", remark);

        JSONObject miniProgram = new JSONObject();
        miniProgram.put("appid", receiptAppId);
        miniProgram.put("pagepath", pagePath);

        JSONObject json = new JSONObject();
        json.put("touser", openid);
        json.put("template_id",messageTemplateReceipt);
        json.put("url","https://www.caimei365.com/");
        json.put("miniprogram", miniProgram);
        json.put("data", data);
        // json 字符串
        String jsonString = json.toJSONString();
        log.info(">>>>>>>>推送微信模板消息：" + jsonString);
        try {
            // https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=ACCESS_TOKEN
            String requestUrl = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token="+accessToken;
            // 发送请求
            String result = RequestUtil.httpRequest(requestUrl, "POST", jsonString);
            log.info(">>>>>>>>推送结果：" + result);
        } catch (Exception e) {
            log.error("推送微信模板消息失败：", e);
        }
    }

    /**
     * 下单消息推送
     * @param accessToken 微信公众号
     * @param openid 公众号的openid
     * @param title 标题
     * @param name 商品名称
     * @param orderno 订单编号
     * @param money 金额
     * @param remarkText 备注
     * @param pagePath 跳转链接
     */
    public void sendTemplateMsgs(String accessToken, String openid, String title, String name,String orderno, Double money, String remarkText, String pagePath) {
        JSONObject first = new JSONObject();
        first.put("value", title);
        JSONObject keyword1 = new JSONObject();
        keyword1.put("value", name);
        JSONObject keyword2 = new JSONObject();
        keyword2.put("value", orderno);
        JSONObject keyword3 = new JSONObject();
        keyword3.put("value", money.toString());
        JSONObject remark = new JSONObject();
        remark.put("value", remarkText);

        JSONObject data = new JSONObject();
        data.put("first", first);
        data.put("keyword1", keyword1);
        data.put("keyword2", keyword2);
        data.put("keyword3", keyword3);
        data.put("remark", remark);

       JSONObject miniProgram = new JSONObject();
       miniProgram.put("appid", miniAppId);
       miniProgram.put("pagepath", pagePath);

        JSONObject json = new JSONObject(new LinkedHashMap());
        json.put("touser",openid);
        json.put("template_id","lLPssEY0tx6k5jT-8tDQgEpInf5quA3_M9RsI9aw3IM");
        json.put("url","https://www.caimei365.com/");
        json.put("miniprogram", miniProgram);
        json.put("data", data);
        // json 字符串
        String jsonString = json.toJSONString();
        log.info(">>>>>>>>推送微信模板消息：" + jsonString);
        try {
            // https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=ACCESS_TOKEN
//            String requestUrl =  "https://api.weixin.qq.com/cgi-bin/message/wxopen/template/send?access_token="+accessToken;
            String requestUrl = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token="+accessToken;
            // 发送请求
            String result = RequestUtil.httpRequest(requestUrl, "POST", jsonString);
            log.info(">>>>>>>>推送结果：" + result);
        } catch (Exception e) {
            log.error("推送微信模板消息失败：", e);
        }
    }

    /**
     * 发货消息推送
     * @param accessToken
     * @param openid
     * @param title 标题
     * @param orderno 订单编号
     * @param time 发货时间
     * @param company 快递名称
     * @param companyno 快递编号
     * @param remarkText 备注
     * @param pagePath 跳转链接
     */
    public void sendTemplateMsgy(String accessToken, String openid, String title, String orderno,String time, String company, String companyno,String remarkText, String pagePath) {
        JSONObject first = new JSONObject();
        first.put("value", title);
        first.put("color", "#000000");
        JSONObject keyword1 = new JSONObject();
        keyword1.put("value", orderno);
        keyword1.put("color", "#FF0000");
        JSONObject keyword2 = new JSONObject();
        keyword2.put("value", time);
        keyword2.put("color", "#c4c400");
        JSONObject keyword3 = new JSONObject();
        keyword3.put("value", company);
        keyword3.put("color", "#c4c400");
        JSONObject keyword4 = new JSONObject();
        keyword4.put("value", companyno);
        keyword4.put("color", "#c4c400");
        JSONObject remark = new JSONObject();
        remark.put("value", remarkText);
        remark.put("color", "#c4c400");

        JSONObject data = new JSONObject();
        data.put("first", first);
        data.put("keyword1", keyword1);
        data.put("keyword2", keyword2);
        data.put("keyword3", keyword3);
        data.put("keyword4", keyword4);
        data.put("remark", remark);

        JSONObject miniProgram = new JSONObject();
        miniProgram.put("appid", receiptAppId);
        miniProgram.put("pagepath", pagePath);

        JSONObject json = new JSONObject();
        json.put("touser", openid);
        json.put("template_id", "_Zc6WiUp3hM4yBgT30HkTyA-SmcCN-sjnGpIa6n3rKI");
        json.put("url", "https://www.caimei365.com/");
        json.put("miniprogram", miniProgram);
        json.put("data", data);
        // json 字符串
        String jsonString = json.toJSONString();
        log.info(">>>>>>>>推送微信模板消息：" + jsonString);
        try {
            // https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=ACCESS_TOKEN
              String requestUrl =  "https://api.weixin.qq.com/cgi-bin/message/wxopen/template/send?access_token=ACCESS_TOKEN";
//            String requestUrl = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token="+accessToken;
            // 发送请求
            String result = RequestUtil.httpRequest(requestUrl, "POST", jsonString);
            log.info(">>>>>>>>推送结果：" + result);
        } catch (Exception e) {
            log.error("推送微信模板消息失败：", e);
        }
    }

    /**
     * 购买成功消息推送
     * @param accessToken
     * @param openid
     * @param title 标题
     * @param paytime 付款时间
     * @param name 商品名称
     * @param orderno 商品编号
     * @param money 订单金额
     * @param remarkText 备注
     * @param pagePath 跳转链接
     */
    public void sendTemplateMsgz(String accessToken, String openid, String title,String paytime, String name,String orderno, Double money, String remarkText, String pagePath) {
        JSONObject first = new JSONObject();
        first.put("value", title);
        JSONObject keyword1 = new JSONObject();
        keyword1.put("value", name);
        JSONObject keyword2 = new JSONObject();
        keyword2.put("value", orderno);
        JSONObject keyword3 = new JSONObject();
        keyword3.put("value", money);
        JSONObject keyword4 = new JSONObject();
        keyword4.put("value", paytime);
        JSONObject remark = new JSONObject();
        remark.put("value", remarkText);

        JSONObject data = new JSONObject();
        data.put("first", first);
        data.put("keyword1", keyword1);
        data.put("keyword2", keyword2);
        data.put("keyword3", keyword3);
        data.put("keyword4", keyword4);
        data.put("remark", remark);

        JSONObject miniProgram = new JSONObject();
        miniProgram.put("appid", miniAppId);
        miniProgram.put("pagepath", pagePath);

        JSONObject json = new JSONObject();
        json.put("touser", openid);
        json.put("template_id", "KV6huLTZInkwTRyPkvaUElD3xs4TgEBK4e0xuDiSAmU");
        json.put("url", "https://www.caimei365.com/");
        json.put("miniprogram", miniProgram);
        json.put("data", data);
        // json 字符串
        String jsonString = json.toJSONString();
        log.info(">>>>>>>>推送微信模板消息：" + jsonString);
        log.info(">>>>>>>>推送微信模板json消息：" + json);
        try {
            // https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=ACCESS_TOKEN
            String requestUrl = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token="+accessToken;
            // 发送请求
            String result = RequestUtil.httpRequest(requestUrl, "POST", jsonString);
            log.info(">>>>>>>>推送结果：" + result);
        } catch (Exception e) {
            log.error("推送微信模板消息失败：", e);
        }
    }

    /**
     * 续费会员消息推送
     * @param accessToken 微信公众号
     * @param openid 公众号的openid
     * @param title 标题
     * @param name 商品名称
     * @param remarkText 备注
     * @param pagePath 跳转链接
     */
    public void sendTempl(String accessToken, String openid, String title, String name,String type,String date,String remarkText, String pagePath) {
        JSONObject first = new JSONObject();
        first.put("value", title);
        JSONObject keyword1 = new JSONObject();
        keyword1.put("value", name);
        JSONObject keyword2 = new JSONObject();
        keyword2.put("value", date);
        JSONObject keyword3 = new JSONObject();
        keyword2.put("value", "会员剩余天数：30天");
        JSONObject remark = new JSONObject();
        remark.put("value", remarkText);

        JSONObject data = new JSONObject();
        data.put("first", first);
        data.put("keyword1", keyword1);
        data.put("keyword2", keyword2);
        data.put("keyword2", keyword3);
        data.put("remark", remark);

        JSONObject miniProgram = new JSONObject();
        miniProgram.put("appid", miniAppId);
        miniProgram.put("pagepath", pagePath);

        JSONObject json = new JSONObject(new LinkedHashMap());
        json.put("touser",openid);
        json.put("template_id","QHsM0AhlgGaX6kJ6vFm1wAKIkNjnZdWPjFGOKopLbsM");
        json.put("url","https://www.caimei365.com/");
        json.put("miniprogram", miniProgram);
        json.put("data", data);
        // json 字符串
        String jsonString = json.toJSONString();
        log.info(">>>>>>>>推送微信模板消息：" + jsonString);
        try {
            // https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=ACCESS_TOKEN
//            String requestUrl =  "https://api.weixin.qq.com/cgi-bin/message/wxopen/template/send?access_token="+accessToken;
            String requestUrl = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token="+accessToken;
            // 发送请求
            String result = RequestUtil.httpRequest(requestUrl, "POST", jsonString);
            log.info(">>>>>>>>推送结果：" + result);
        } catch (Exception e) {
            log.error("推送微信模板消息失败：", e);
        }
    }


    /**
     * 续费会员消息推送
     */
    public void sendTemplss(String accessToken, String openid,String Time, String pagePath) {
        JSONObject first = new JSONObject();
        first.put("value", "恭喜您成功续费采美超级会员");
        JSONObject keyword1 = new JSONObject();
        keyword1.put("value", "超级会员");
        JSONObject keyword2 = new JSONObject();
        keyword2.put("value", Time);
        JSONObject remark = new JSONObject();
        remark.put("value", "如在使用中有任何疑问，请在小程序内联系在线客服");

        JSONObject data = new JSONObject();
        data.put("first", first);
        data.put("keyword1", keyword1);
        data.put("keyword2", keyword2);
        data.put("remark", remark);

        JSONObject miniProgram = new JSONObject();
        miniProgram.put("appid", miniAppId);
        miniProgram.put("pagepath", pagePath);

        JSONObject json = new JSONObject(new LinkedHashMap());
        json.put("touser",openid);
        json.put("template_id","QHsM0AhlgGaX6kJ6vFm1wAKIkNjnZdWPjFGOKopLbsM");
        json.put("url","https://www.caimei365.com/");
        json.put("miniprogram", miniProgram);
        json.put("data", data);
        // json 字符串
        String jsonString = json.toJSONString();
        log.info(">>>>>>>>推送微信模板消息：" + jsonString);
        try {
            // https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=ACCESS_TOKEN
//            String requestUrl =  "https://api.weixin.qq.com/cgi-bin/message/wxopen/template/send?access_token="+accessToken;
            String requestUrl = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token="+accessToken;
            // 发送请求
            String result = RequestUtil.httpRequest(requestUrl, "POST", jsonString);
            log.info(">>>>>>>>推送结果：" + result);
        } catch (Exception e) {
            log.error("推送微信模板消息失败：", e);
        }
    }


}
