package com.caimei365.order.controller;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.CartDto;
import com.caimei365.order.model.dto.InvoiceDto;
import com.caimei365.order.model.dto.InvoiceOrganizeDto;
import com.caimei365.order.model.vo.InvoiceVo;
import com.caimei365.order.service.CartClubService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 机构购物车API
 *
 * @author : Charles
 * @date : 2021/6/25
 */
@Api(tags = "机构购物车API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/club")
public class CartClubApi {

    private final CartClubService cartClubService;

    @ApiOperation("购物车重选商品sku")
    @GetMapping("/check/sku")
    public ResponseJson checkOldCart(Integer oldSkuId, Integer userId, Integer newSkuId, Integer count) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == count) {
            return ResponseJson.error("购买数量不能为空！", null);
        }
        if (null == oldSkuId || null == newSkuId) {
            return ResponseJson.error("skuId不能为空！", null);
        }
        return cartClubService.checkSkuId(userId, oldSkuId, newSkuId, count);
    }

    /**
     * 购物车列表详细数据
     */
    @ApiOperation("购物车列表详细数据（旧：/shoppingCart/list）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "source", value = "来源 : 1 网站 ; 2 小程序")
    })
    @GetMapping("/cart/list")
    public ResponseJson<Map<String, Object>> getShoppingCartList(Integer userId, Integer source) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return cartClubService.getShoppingCartList(userId, source);
    }

    /**
     * 网站顶部购物车数据
     */
    @ApiOperation("网站顶部购物车数据（旧：/shoppingCart/header/cart）")
    @ApiImplicitParam(required = true, name = "userId", value = "用户Id")
    @GetMapping("/cart/head")
    public ResponseJson<Map<String, Object>> getShoppingCartHead(Integer userId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return cartClubService.getShoppingCartHead(userId);
    }

    /**
     * 获取购物车商品种类
     */
    @ApiOperation("获取购物车数量（旧：/home/modules内）")
    @ApiImplicitParam(required = true, name = "userId", value = "用户Id")
    @GetMapping("/cart/count")
    public ResponseJson<Integer> getShoppingCartCount(Integer userId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return cartClubService.getUserCartCount(userId);
    }

    /**
     * 获取购物车商品种类 -- 组织
     */
    @ApiOperation("获取购物车数量（旧：/home/modules内）")
    @ApiImplicitParam(required = true, name = "userId", value = "用户Id")
    @GetMapping("/cart/organizeCount")
    public ResponseJson<Integer> getShoppingCartOrganizeCount(Integer userId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return cartClubService.getShoppingCartOrganizeCount(userId);
    }

    /**
     * 添加购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                skuId        skuId
     *                productCount 商品数量
     *                }
     */
    @ApiOperation("添加购物车（旧：/shoppingCart/addCart）")
    @PostMapping("/cart/add")
    public ResponseJson<Integer> addShoppingCart(CartDto cartDto) {
        if (null == cartDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == cartDto.getProductId()) {
            return ResponseJson.error("商品Id不能为空！", null);
        }
        if (null == cartDto.getSkuId()) {
            return ResponseJson.error("skuId不能为空！", null);
        }
        if (null == cartDto.getProductCount()) {
            return ResponseJson.error("商品数量不能为空！", null);
        }
        return cartClubService.addShoppingCart(cartDto);
    }

    /**
     * 批量添加购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productInfo  商品及数量信息:[   // 商品id，数量
     *                {"productId": 2789, "skuId":123, "productCount": 1},
     *                {"productId": 2790, "skuId":123, "productCount": 1}
     *                ]
     *                }
     */
    @ApiOperation("批量添加购物车（旧：/shoppingCart/addCarts）")
    @PostMapping("/cart/add/bulk")
    public ResponseJson<Integer> addShoppingCartBulk(CartDto cartDto) {
        if (null == cartDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == cartDto.getProductInfo()) {
            return ResponseJson.error("商品及数量信息不能为空！", null);
        }
        return cartClubService.addShoppingCartBulk(cartDto);
    }


    /**
     * 更新购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     *                }
     */
    @ApiOperation("更新购物车（旧：/shoppingCart/update）")
    @PostMapping("/cart/update")
    public ResponseJson<Integer> updateShoppingCart(CartDto cartDto) {
        if (null == cartDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == cartDto.getSkuId()) {
            return ResponseJson.error("skuId不能为空！", null);
        }
        if (null == cartDto.getProductCount()) {
            return ResponseJson.error("商品数量不能为空！", null);
        }
        return cartClubService.updateShoppingCart(cartDto);
    }

    /**
     * 删除购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                skuIds       skuIds,逗号隔开
     *                }
     */
    @ApiOperation("删除购物车（旧：/shoppingCart/delete）")
    @PostMapping("/cart/delete")
    public ResponseJson<Integer> deleteShoppingCart(CartDto cartDto) {
        if (null == cartDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (StringUtils.isEmpty(cartDto.getSkuIds())) {
            return ResponseJson.error("skuId集合不能为空！", null);
        }
        return cartClubService.deleteShoppingCart(cartDto);
    }

    /**
     * 购物车结算
     */
    @ApiOperation("购物车结算（旧：/order/confirm）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "cityId", value = "地址"),
            @ApiImplicitParam(required = false, name = "skuIds", value = "skuIds,逗号隔开"),
            @ApiImplicitParam(required = false, name = "source", value = "来源 : 1 网站 ; 2 小程序")
    })
    @GetMapping("/cart/settlement")
    public ResponseJson<Map<String, Object>> settlementShoppingCart(Integer userId, Integer cityId, String skuIds, Integer source) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (StringUtils.isEmpty(skuIds)) {
            return ResponseJson.error("skuId集合不能为空！", null);
        }
        return cartClubService.settlementShoppingCart(userId, cityId, skuIds, source);
    }

    /**
     * 立即购买
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     *                skuId
     *                source       来源 : 1 网站 ; 2 小程序
     *                }
     */
    @ApiOperation("立即购买（旧：/order/confirm）")
    @GetMapping("/product/settlement")
    public ResponseJson<Map<String, Object>> buyNowProduct(CartDto cartDto) {
        if (null == cartDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == cartDto.getProductId()) {
            return ResponseJson.error("商品Id不能为空！", null);
        }
        if (null == cartDto.getSkuId()) {
            return ResponseJson.error("skuId不能为空！", null);
        }
        if (null == cartDto.getProductCount()) {
            return ResponseJson.error("商品数量不能为空！", null);
        }
        return cartClubService.buyNowProduct(cartDto);
    }

    @ApiOperation("组合商品立即购买")
    @GetMapping("/product/buynow")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "townId", value = "地址"),
            @ApiImplicitParam(required = false, name = "productInfo", value = "组合商品信息：Json字符串格式[{\"id\":4351,\"count\":1,\"skuId\":123},{}]"),
            @ApiImplicitParam(required = false, name = "source", value = "来源 : 1 网站 ; 2 小程序")
    })
    public ResponseJson<Map<String, Object>> MultipleBuyNow(Integer userId, Integer cityId, String productInfo, Integer source) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (StringUtils.isEmpty(productInfo)) {
            return ResponseJson.error("组合商品信息不能为空！", null);
        }
        return cartClubService.MultipleBuyNow(userId, cityId, productInfo, source);
    }

    /**
     * 获取结算商品运费
     */
    @ApiOperation("计算运费（旧：/order/postage）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "skuIds", value = "商品ids,逗号隔开"),
            @ApiImplicitParam(required = false, name = "townId", value = "地区Id")
    })
    @GetMapping("/postage")
    public ResponseJson<Map<String, Object>> orderPostage(Integer userId, String skuIds, Integer cityId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (StringUtils.isEmpty(skuIds)) {
            return ResponseJson.error("skuId集合不能为空！", null);
        }
        if (null == cityId) {
            return ResponseJson.error("地区Id不能为空！", null);
        }
        return cartClubService.getProductsPostage(userId, skuIds, cityId);
    }

    /**
     * 获取用户发票信息
     */
    @ApiOperation("获取用户发票信息（旧：/personalCenter/findInvoice）")
    @ApiImplicitParam(required = false, name = "userId", value = "用户Id")
    @GetMapping("/invoice")
    public ResponseJson<InvoiceVo> getUserInvoice(Integer userId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return cartClubService.getUserInvoice(userId);
    }

    /**
     * 更新用户发票
     *
     * @param invoiceDto {
     *                   userId:用户Id
     *                   id:id
     *                   invoiceTitle:单位名，发票抬头
     *                   corporationTaxNum: 企业税号、纳税人识别号
     *                   registeredAddress:注册地址
     *                   registeredPhone:注册电话
     *                   bankAccountNo:开户银行账户
     *                   openBank:开户银行
     *                   }
     */
    @ApiOperation("更新用户发票（旧：/personalCenter/invoice）")
    @PostMapping("/invoice/update")
    public ResponseJson<Integer> updateUserInvoice(InvoiceDto invoiceDto) {
        if (null == invoiceDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return cartClubService.updateUserInvoice(invoiceDto);
    }

    /**
     * 更新用户发票--组织
     *
     * @param invoiceDto {
     *                   userId:用户Id
     *                   id:id
     *                   invoiceTitle:单位名，发票抬头
     *                   corporationTaxNum: 企业税号、纳税人识别号
     *                   registeredAddress:注册地址
     *                   registeredPhone:注册电话
     *                   bankAccountNo:开户银行账户
     *                   openBank:开户银行
     *                   }
     */
    @ApiOperation("更新用户发票--组织")
    @PostMapping("/invoice/updateOrganize")
    public ResponseJson<Integer> updateOrganizeUserInvoice(InvoiceOrganizeDto invoiceDto) {
        if (null == invoiceDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == invoiceDto.getType()) {
            return ResponseJson.error("发票类型不能为空！", null);
        }
        if (1 == invoiceDto.getType()) {
            if (null == invoiceDto.getHeadingType()) {
                return ResponseJson.error("抬头类型不能为空！", null);
            }
            if (0 == invoiceDto.getHeadingType() && null == invoiceDto.getInvoiceTitle()) {
                return ResponseJson.error("发票抬头不能为空！", null);
            }
            if (1 == invoiceDto.getHeadingType()) {
                if  (null == invoiceDto.getInvoiceTitle()) {
                    return ResponseJson.error("发票抬头不能为空！", null);
                }
                if  (null == invoiceDto.getCorporationTaxNum()) {
                    return ResponseJson.error("企业税号不能为空！", null);
                }
            }

        }
        if (2 == invoiceDto.getType()) {
            if  (null == invoiceDto.getInvoiceTitle()) {
                return ResponseJson.error("发票抬头不能为空！", null);
            }
            if  (null == invoiceDto.getCorporationTaxNum()) {
                return ResponseJson.error("企业税号不能为空！", null);
            }
            if  (null == invoiceDto.getRegisteredAddress()) {
                return ResponseJson.error("注册地址不能为空！", null);
            }
            if  (null == invoiceDto.getRegisteredPhone()) {
                return ResponseJson.error("注册电话不能为空！", null);
            }
            if  (null == invoiceDto.getBankAccountNo()) {
                return ResponseJson.error("开户银行账户不能为空！", null);
            }
            if  (null == invoiceDto.getOpenBank()) {
                return ResponseJson.error("开户银行不能为空！", null);
            }
            // 增值税专用发票 抬头为null
            invoiceDto.setHeadingType(null);
        }
        return cartClubService.updateOrganizeUserInvoice(invoiceDto);
    }

    /**
     * 供应商优惠券
     *
     * @param userId 用户id
     * @param shopId 供应商id
     * @param source 来源 : 1 网站 ; 2 小程序
     * @param status 状态: 1未领取 2已领取
     */
    @ApiOperation("领取供应商优惠券（旧：/shoppingCart/get/coupon）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "shopId", value = "供应商Id"),
            @ApiImplicitParam(required = false, name = "status", value = "状态: 1未领取, 2已领取"),
            @ApiImplicitParam(required = false, name = "source", value = "来源: 1网站, 2小程序")
    })
    @GetMapping("/coupon")
    public ResponseJson<Map<String, Object>> getShopCoupons(Integer userId, Integer shopId, Integer source,
                                                            @RequestParam(name = "status", defaultValue = "1") Integer status) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == shopId) {
            return ResponseJson.error("供应商Id不能为空！", null);
        }
        if (null == source) {
            return ResponseJson.error("来源source不能为空！", null);
        }
        return cartClubService.getShopCoupons(userId, shopId, source, status);
    }
}
