package com.caimei365.order.controller;

import com.caimei365.order.feign.UserFeign;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.OrderProductDto;
import com.caimei365.order.model.dto.RechargeDto;
import com.caimei365.order.model.dto.SubmitDto;
import com.caimei365.order.model.po.CmReportingClub;
import com.caimei365.order.service.CmReportingClubService;
import com.caimei365.order.service.SubmitService;
import com.caimei365.order.utils.MathUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.*;

import java.util.Map;


/**
 * 提交订单API
 *
 * @author : Charles
 * @date : 2021/7/6
 */
@Api(tags = "提交订单API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/submit")
public class OrderSubmitApi {
    private final SubmitService submitService;
    private final UserFeign userFeign;
    private final CmReportingClubService cmReportingClubService;

    /**
     * 生成订单
     *
     * @param submitDto {
     *                  "clubId": 10708,            //机构ID
     *                  "unionId":"",               //微信unionId
     *                  "addressId": 2732,          //地址ID
     *                  "orderSource": 2,           //订单来源：1WWW、2CRM、4客服[适用后台下单]、5外单[适用后台下单]、6小程序[采美,星范]、7呵呵商城小程序、8维沙小程序
     *                  "cartType":3,               //购买类型:（1自主下单(购物车),2自主下单(立即购买)，3协销下单）
     *                  "serviceProviderId": 1378,  //协销ID（小程序忽略）
     *                  "clubCouponId": ""          //优惠券Id
     *                  "orderSeen":               //订单对机构可见度，1可见，2不可见
     *                  "orderInfo": [              //【订单商品】
     *                  { "shopId":1001,                         // 供应商Id
     *                  "splitCode:E123123"       //分账号
     *                  "note":备注,
     *                  "productInfo":[                      // skuId，数量，赠品数,商品类型
     *                  {"skuId": 2789, "productNum": 1, "presentNum":0,"productType":2},
     *                  {"skuId": 2789, "productNum": 1, "presentNum":0,"productType":0}
     *                  ]
     *                  },{多个供应商商品数据结构同上}
     *                  ],
     *                  "payInfo": {                //【订单金额】
     *                  "orderShouldPayFee": 609.11,
     *                  "balancePayFlag": 0,
     *                  "clauseId": "2",
     *                  "postage": "15",
     *                  "postageFlag": 1,
     *                  "userBeans": 100,//抵扣采美豆数量
     *                  "rebateFlag":0,
     *                  "rebateFee":0.00 //普通订单返佣服务费
     *                  },
     *                  "orderInvoice":             //【发票信息】
     *                  {"type": 0 // 不开发票  }
     *                  或： { // 普通发票
     *                  "type": 1,
     *                  "invoiceTitle": "企业抬头",
     *                  "invoiceTitleType": 1,
     *                  "corporationTaxNum": "XXX4156465465",
     *                  "invoiceContent": "明细 "
     *                  }
     *                  或： { // 增值税发票
     *                  "type": 2,
     *                  "invoiceTitle": "单位名称",
     *                  "corporationTaxNum": "NSRSBM97897",
     *                  "registeredAddress": "注册地址",
     *                  "registeredPhone": "15814011616",
     *                  "openBank": "开户银行",
     *                  "bankAccountNo": "987987465465464"
     *                  }
     *                  <p>
     *                  }
     */


    @ApiOperation("提交订单（旧：/order/submit）（/seller/order/submit）")
    @PostMapping("/generate")
    public ResponseJson<Map<String, Object>> generateOrder(SubmitDto submitDto) {
        if (null == submitDto.getOrderSource()) {
            return ResponseJson.error("订单来源不能为空！", null);
        }
        // 1,www来源    2,crm来源    6,小程序来源  ，9分销系统来源
        if (null == submitDto.getOrderSource() || !(1 == submitDto.getOrderSource() || 2 == submitDto.getOrderSource() || 6 == submitDto.getOrderSource() || 9 == submitDto.getOrderSource())) {
            return ResponseJson.error("订单来源异常！", null);
        }
        if (null == submitDto.getCartType()) {
            return ResponseJson.error("购买类型不能为空！", null);
        }
        if (null == submitDto.getClubId()) {
            return ResponseJson.error("机构Id不能为空！", null);
        }
        if (null == submitDto.getAddressId()) {
            return ResponseJson.error("收货地址Id不能为空！", null);
        }
        if (StringUtils.isEmpty(submitDto.getOrderInfo())) {
            return ResponseJson.error("订单商品数据不能为空！", null);
        }
        if (StringUtils.isEmpty(submitDto.getPayInfo())) {
            return ResponseJson.error("订单金额数据不能为空！", null);
        }
        if (StringUtils.isEmpty(submitDto.getOrderInvoice())) {
            return ResponseJson.error("发票信息数据不能为空！", null);
        }
        if (3 == submitDto.getCartType() && null == submitDto.getServiceProviderId()) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        return submitService.generateOrder(submitDto);
    }

    @ApiOperation("分销人员提交订单（旧：/order/submit）（/seller/order/submit）")
    @PostMapping("/distributionGenerate")
    public ResponseJson distributionGenerateOrder(SubmitDto submitDto, @RequestParam String clubReportingId, @RequestHeader HttpHeaders headers) {
        CmReportingClub cmReportingClubById = cmReportingClubService.getCmReportingClubById(clubReportingId);
        if (null==cmReportingClubById) {
            return ResponseJson.error("报备机构记录异常！", null);
        }
        ResponseJson responseJson = userFeign.distributionClubRegister(cmReportingClubById.clubName(), cmReportingClubById.linkMan(), cmReportingClubById.mobile(),submitDto.getServiceProviderId(), headers);
        if (responseJson.getCode() != 0) {
            return responseJson;
        }
        submitDto.setClubId(Integer.valueOf(responseJson.getMsg()));
        submitDto.setClubReportingId(clubReportingId);
        submitDto.setCartType(3);
        submitDto.setOrderSource(9);
        return generateOrder(submitDto);
    }

    /**
     * 生成充值商品订单
     *
     * @param rechargeDto {
     *                    serviceProviderId 协销id
     *                    clubId            机构id
     *                    name              商品名称
     *                    price             价格
     *                    note              备注
     *                    }
     */
    @ApiOperation("提交充值商品订单（旧：/order/submit/rechargeOrder）")
    @PostMapping("/recharge")
    public ResponseJson<Map<String, Object>> generateRechargeOrder(RechargeDto rechargeDto) {
        if (null == rechargeDto.getServiceProviderId()) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        if (null == rechargeDto.getClubId()) {
            return ResponseJson.error("机构Id不能为空！", null);
        }
        if (StringUtils.isEmpty(rechargeDto.getName())) {
            return ResponseJson.error("商品名称不能为空！", null);
        }
        if (MathUtil.compare(rechargeDto.getPrice(), 0) < 1) {
            return ResponseJson.error("商品价格不正确！", null);
        }
        return submitService.generateRechargeOrder(rechargeDto);
    }

    /**
     * 生成充值商品订单
     */
    @ApiOperation("修改订单")
    @PostMapping("/update")
    public ResponseJson updateOrder(@RequestBody OrderProductDto orderProducts) {
        if (null == orderProducts) {
            return ResponseJson.error("订单不能为空！", null);
        }
        return submitService.updateOrder(orderProducts.getOrderProducts());
    }

}
