package com.caimei365.order.controller;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.ReceiptAuthDto;
import com.caimei365.order.model.dto.ReceiptDto;
import com.caimei365.order.model.dto.ReceiptOperateDto;
import com.caimei365.order.model.po.ReceiptPo;
import com.caimei365.order.model.po.cmOfflineCollectionPo;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.ReceiptService;
import com.caimei365.order.service.cmOfflineCollectionService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * 线下收款接口
 *
 * @author : Charles
 * @date : 2022/2/11
 */
@Api(tags = "线下收款API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/receipt")
public class ReceiptApi {
    private final ReceiptService receiptService;

    @Resource
    private cmOfflineCollectionService collectionService;

    /**
     * 线下收款用户微信授权
     *
     * @param receiptAuthDto {
     *                       code          微信授权code
     *                       encryptedData 微信加密数据
     *                       iv            加密算法的初始向量
     *                       }
     */
    @ApiOperation("线下收款用户微信授权(小程序)")
    @PostMapping("/auth")
    public ResponseJson<ReceiptUserVo> receiptAuthorization(ReceiptAuthDto receiptAuthDto, @RequestHeader HttpHeaders headers) {
        String code = receiptAuthDto.getCode();
        String encryptedData = receiptAuthDto.getEncryptedData();
        String iv = receiptAuthDto.getIv();
        if (!StringUtils.hasLength(code)) {
            return ResponseJson.error("没有获取到微信授权code", null);
        }
        return receiptService.receiptAuthorization(code, encryptedData, iv, headers);
    }

    /**
     * 线下收款用户密码登录
     *
     * @param receiptAuthDto {
     *                       code          微信授权code
     *                       encryptedData 微信加密数据
     *                       iv            加密算法的初始向量
     *                       mobile        手机号
     *                       password      密码
     *                       }
     */
    @ApiOperation("线下收款用户密码登录")
    @PostMapping("/password")
    public ResponseJson<ReceiptUserVo> passwordAuthorization(ReceiptAuthDto receiptAuthDto, @RequestHeader HttpHeaders headers) {
        String mobile = receiptAuthDto.getMobile();
        String password = receiptAuthDto.getPassword();
        String code = receiptAuthDto.getCode();
        String encryptedData = receiptAuthDto.getEncryptedData();
        String iv = receiptAuthDto.getIv();
        if (!StringUtils.hasLength(code)) {
            return ResponseJson.error("没有获取到微信授权code", null);
        }
        if (StringUtils.hasLength(mobile) && StringUtils.hasLength(password)) {
            return receiptService.passwordAuthorization(mobile, password, code, encryptedData, iv, headers);
        } else {
            return ResponseJson.error("手机号或密码不能为空", null);
        }
    }

    /**
     * 获取收款类型
     */
    @ApiOperation("获取收款类型")
    @GetMapping("/type")
    public ResponseJson<List<cmOfflineCollectionVo>> getReceiptType() {
        return receiptService.getReceiptType();
    }

    /**
     * 获取后台配置显示银行账户配置信息
     *
     * @return
     */
    @ApiOperation("获取银行账户配置信息")
    @GetMapping("/bankData")
    public ResponseJson<cmOfflineCollectionPo> selectData() {

        return collectionService.seData();
    }

    /**
     * 识别收款短信
     *
     * @param receiptDto {
     *                   smsContent          短信类容
     *                   }
     */
    @ApiOperation("识别收款短信")
    @PostMapping("/read/sms")
    public ResponseJson<ReceiptVo> readSmsMessage(ReceiptDto receiptDto, @RequestHeader HttpHeaders headers) {
        String smsContent = receiptDto.getSmsContent();
        if (StringUtils.hasLength(smsContent)) {
            return receiptService.readSmsMessage(smsContent, headers);
        } else {
            return ResponseJson.error("收款短信不能为空", null);
        }
    }

    /**
     * 保存收款信息
     * crm:/api/user/receivables/save
     */
    @ApiOperation("保存收款信息")
    @PostMapping("/save")
    public ResponseJson<ReceiptPo> saveReceipt(ReceiptDto receiptDto, @RequestHeader HttpHeaders headers) {
        return receiptService.saveReceipt(receiptDto, headers);
    }

    /**
     * 操作收款信息（作废，设为返佣/普通/供应商退款/非订单款）
     * crm:/api/user/receivables/save
     *
     * @param receiptOperateDto {
     *                          id：收款Id
     *                          receiptType: 1订单款，2非订单款，3返佣款 4订单款或者非订单款（因财务阶段无法区分订单非订单）, 5供应商退款
     *                          noOrderReason: 非订单款说明
     *                          delFlag: 作废标记: 0否，其余是
     *                          }
     */
    @ApiOperation("操作收款信息（作废，设为返佣/普通/供应商退款/非订单款）")
    @PostMapping("/operate")
    public ResponseJson<ReceiptPo> operateReceipt(ReceiptOperateDto receiptOperateDto, @RequestHeader HttpHeaders headers) {
        if (null == receiptOperateDto.getId() || receiptOperateDto.getId() <= 0) {
            return ResponseJson.error("收款Id不能为空", null);
        }
        return receiptService.operateReceipt(receiptOperateDto, headers);
    }

    /**
     * 获取收款列表
     * crm:/api/user/receivables/list
     * 返佣款/api/user/rebate/list  或 （receiptType==3：/api/user/receivables/list）
     */
    @ApiOperation("获取收款列表")
    @ApiImplicitParams({@ApiImplicitParam(required = false, name = "receiptType", value = "收款款项类型：1订单款，2非订单款，3返佣款 4订单款或者非订单款（因财务阶段无法区分订单非订单）, 5供应商退款, 6返佣服务费, 7返佣服务费欠款"), @ApiImplicitParam(required = false, name = "receiptStatus", value = "收款状态：1待确认、2已确认(待审核)、3审核通过、4审核未通过、5收款撤销【线上支付成功为审核通过】"), @ApiImplicitParam(required = false, name = "smsContent", value = "收款短信"), @ApiImplicitParam(required = false, name = "startDate", value = "筛选开始时间"), @ApiImplicitParam(required = false, name = "endDate", value = "筛选结束时间"), @ApiImplicitParam(required = false, name = "pageNum", value = "页码"), @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")})
    @GetMapping("/list")
    public ResponseJson<PageInfo<ReceiptVo>> getReceiptList(Integer newReceiptType, Integer receiptType, Integer receiptStatus, String smsContent, String startDate, String endDate, @RequestParam(value = "pageNum", defaultValue = "1") int pageNum, @RequestParam(value = "pageSize", defaultValue = "10") int pageSize, @RequestHeader HttpHeaders headers) {
        if (null == receiptType || receiptType <= 0) {
            return ResponseJson.error("款项类型不能为空", null);
        }
        return receiptService.getReceiptList(newReceiptType, receiptType, receiptStatus, smsContent, startDate, endDate, pageNum, pageSize, headers);
    }

    /**
     * 收款详情（订单/非订单）
     * crm:/api/user/receivables/get
     */
    @ApiOperation("收款详情（订单/非订单）")
    @ApiImplicitParam(required = true, name = "id", value = "款项Id")
    @GetMapping("/detail")
    public ResponseJson<ReceiptVo> getReceiptDetail(Integer id, @RequestHeader HttpHeaders headers) {
        return receiptService.getReceiptDetail(id, headers);
    }

    /**
     * 收款详情的款项类型
     */
    @ApiOperation("收款详情的款项类型")
    @ApiImplicitParam(required = true, name = "id", value = "款项Id")
    @GetMapping("/detail/type")
    public ResponseJson<Map<String, Integer>> getReceiptDetailType(Integer id, @RequestHeader HttpHeaders headers) {
        return receiptService.getReceiptDetailType(id, headers);
    }

    /**
     * 收款详情（分享）
     */
    @ApiOperation("收款详情（订单/非订单）")
    @ApiImplicitParam(required = true, name = "id", value = "款项Id")
    @GetMapping("/detail/share")
    public ResponseJson<ReceiptVo> getReceiptDetailShare(Integer id, @RequestHeader HttpHeaders headers) {
        return receiptService.getReceiptDetailShare(id, headers);
    }

    /**
     * 收款详情（供应商退款）
     * crm:/api/user/shop/shopRefund
     */
    @ApiOperation("收款详情（供应商退款）")
    @ApiImplicitParam(required = true, name = "id", value = "款项Id")
    @GetMapping("/refund")
    public ResponseJson<ReceiptVo> getReceiptRefundDetail(Integer id, @RequestHeader HttpHeaders headers) {
        return receiptService.getReceiptRefundDetail(id, headers);
    }

    /**
     * 收款详情（返佣款）
     * crm:/api/user/rebate/rebateDetail
     */
    @ApiOperation("收款详情（返佣款）")
    @ApiImplicitParam(required = true, name = "id", value = "款项Id")
    @GetMapping("/rebate")
    public ResponseJson<ReceiptVo> getReceiptRebateDetail(Integer id, @RequestHeader HttpHeaders headers) {
        return receiptService.getReceiptRebateDetail(id, headers);
    }

    /**
     * 收款详情-订单列表
     * crm:/api/user/receivables/getOrderList
     * crm:/api/user/receivables/getMoneyOrderList
     */
    @ApiOperation("收款详情-订单列表")
    @ApiImplicitParams({@ApiImplicitParam(required = false, name = "id", value = "收款ID"), @ApiImplicitParam(required = false, name = "type", value = "类型：0商品订单(默认)，1订金订单"), @ApiImplicitParam(required = false, name = "organizeId", value = "组织ID"), @ApiImplicitParam(required = false, name = "orderReceiptStatus", value = "订单收款状态：1待收款，2部分收款，3已收款"), @ApiImplicitParam(required = false, name = "keyword", value = "搜索关键词(客户名称/订单号)"), @ApiImplicitParam(required = false, name = "pageNum", value = "页码"), @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")})
    @GetMapping("/detail/orders")
    public ResponseJson<PageInfo<ShopOrderVo>> getReceiptOrderList(Integer id, Integer type, Integer organizeId, String orderReceiptStatus, String keyword, @RequestParam(value = "pageNum", defaultValue = "1") int pageNum, @RequestParam(value = "pageSize", defaultValue = "10") int pageSize, @RequestHeader HttpHeaders headers) {
        return receiptService.getReceiptOrderList(id, type, organizeId, orderReceiptStatus, keyword, pageNum, pageSize, headers);
    }

    /**
     * 收款详情-订单详情
     * crm:/api/user/receivables/orderDetail
     */
    @ApiOperation("收款详情-订单详情")
    @ApiImplicitParam(required = false, name = "shopOrderId", value = "子订单ID")
    @GetMapping("/detail/orders/info")
    public ResponseJson<ShopOrderVo> getReceiptOrderInfo(Integer shopOrderId, @RequestHeader HttpHeaders headers) {
        return receiptService.getReceiptOrderInfo(shopOrderId, headers);
    }

    /**
     * 供应商退款-子订单列表
     * crm:/api/user/shop/refundShopOrder
     */
    @ApiOperation("供应商退款-子订单列表")
    @ApiImplicitParams({@ApiImplicitParam(required = false, name = "id", value = "收款ID"), @ApiImplicitParam(required = false, name = "confirmedType", value = "0待确认，2已确认子订单"), @ApiImplicitParam(required = false, name = "shopName", value = "供应商名称"), @ApiImplicitParam(required = false, name = "keyword", value = "搜索关键词(客户名称/订单号)"), @ApiImplicitParam(required = false, name = "pageNum", value = "页码"), @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")})
    @GetMapping("/refund/orders")
    public ResponseJson<PageInfo<ShopOrderVo>> getRefundShopOrderList(Integer id, Integer confirmedType, String shopName, String keyword, @RequestParam(value = "pageNum", defaultValue = "1") int pageNum, @RequestParam(value = "pageSize", defaultValue = "10") int pageSize, @RequestHeader HttpHeaders headers) {
        return receiptService.getRefundShopOrderList(id, confirmedType, shopName, keyword, pageNum, pageSize, headers);
    }

    /**
     * 供应商退款-订单详情
     * crm:/api/user/rebate/orderInfo
     */
    @ApiOperation("供应商退款-订单详情")
    @ApiImplicitParam(required = false, name = "shopOrderId", value = "子订单ID")
    @GetMapping("/refund/orders/info")
    public ResponseJson<ShopOrderVo> getRefundShopOrderInfo(Integer shopOrderId, @RequestHeader HttpHeaders headers) {
        return receiptService.getRefundShopOrderInfo(shopOrderId, headers);
    }

    /**
     * 返佣款-子订单列表
     * crm:/api/user/rebate/rebateShopOrder
     */
    @ApiOperation("返佣款-子订单列表")
    @ApiImplicitParams({@ApiImplicitParam(required = false, name = "id", value = "返佣收款ID"), @ApiImplicitParam(required = false, name = "shopName", value = "供应商名称"), @ApiImplicitParam(required = false, name = "pageNum", value = "页码"), @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")})
    @GetMapping("/rebate/orders")
    public ResponseJson<PageInfo<ShopOrderVo>> getRebateShopOrderList(Integer id, String shopName, @RequestParam(value = "pageNum", defaultValue = "1") int pageNum, @RequestParam(value = "pageSize", defaultValue = "10") int pageSize, @RequestHeader HttpHeaders headers) {
        return receiptService.getRebateShopOrderList(id, shopName, pageNum, pageSize, headers);
    }

    /**
     * 返佣款-平台服务费
     */
    @ApiOperation("平台服务费-子订单列表")
    @ApiImplicitParams({@ApiImplicitParam(required = false, name = "id", value = "返佣收款ID"), @ApiImplicitParam(required = false, name = "shopName", value = "供应商名称"), @ApiImplicitParam(required = false, name = "pageNum", value = "页码"), @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")})
    @GetMapping("/platform/orders")
    public ResponseJson<PageInfo<ShopOrderVo>> getPlatformOrder(Integer id, String shopName, @RequestParam(value = "pageNum", defaultValue = "1") int pageNum, @RequestParam(value = "pageSize", defaultValue = "10") int pageSize, @RequestHeader HttpHeaders headers) {
        return receiptService.getPlatformOrder(id, shopName, pageNum, pageSize, headers);
    }

    @ApiOperation("确认收款-平台服务费")
    @PostMapping("/confirm/platform")
    public ResponseJson<Void> receiptPlatformFee(ReceiptOperateDto receiptOperateDto, @RequestHeader HttpHeaders headers) {
        if (null == receiptOperateDto.getId() || receiptOperateDto.getId() <= 0) {
            return ResponseJson.error("收款Id不能为空", null);
        }
        if (null == receiptOperateDto.getShopOrderId() || receiptOperateDto.getShopOrderId() <= 0) {
            return ResponseJson.error("关联平台服务费子订单ID不能为空", null);
        }
        return receiptService.receiptPlatformFee(receiptOperateDto, headers);
    }

    @ApiOperation("确认收款-平台服务费欠款")
    @PostMapping("/confirm/bill")
    public ResponseJson<Integer> receiptPlatformBill(ReceiptOperateDto receiptOperateDto, @RequestHeader HttpHeaders headers) {
        if (null == receiptOperateDto.getVerbalAmount()) {
            return ResponseJson.error("平台服务费欠款参数异常", null);
        }
        if (null == receiptOperateDto.getShopOrderId()) {
            return ResponseJson.error("关联返佣子订单ID不能为空", null);
        }
        return receiptService.receiptPlatformBill(receiptOperateDto, headers);
    }

    /**
     * 确认收款-关联订单款
     * crm:/api/user/receivables/confirmReceipt
     *
     * @param receiptOperateDto {
     *                          id: 收款Id
     *                          orderIds: 关联订单IDS
     *                          confirmType: 确认订单类型：1小额抹平确认，2大额抹平确认，3大额退款余额，4确认关联
     *                          noOrderReason: 非订单款说明
     *                          }
     */
    @ApiOperation("确认收款-关联订单款")
    @PostMapping("/confirm")
    public ResponseJson<Void> receiptAssociate(ReceiptOperateDto receiptOperateDto, @RequestHeader HttpHeaders headers) {
        if (null == receiptOperateDto.getId() || receiptOperateDto.getId() <= 0) {
            return ResponseJson.error("收款Id不能为空", null);
        }
        if (!StringUtils.hasLength(receiptOperateDto.getOrderIds())) {
            return ResponseJson.error("关联订单IDS不能为空", null);
        }
        return receiptService.receiptAssociate(receiptOperateDto, headers);
    }

    /**
     * 确认收款-关联供应商退款
     * crm:/api/user/shop/relation
     *
     * @param receiptOperateDto {
     *                          id:          收款Id
     *                          shopOrderId: 关联供应商退款子订单ID
     *                          }
     */
    @ApiOperation("确认收款-关联供应商退款")
    @PostMapping("/confirm/refund")
    public ResponseJson<Void> receiptAssociateRefund(ReceiptOperateDto receiptOperateDto, @RequestHeader HttpHeaders headers) {
        if (null == receiptOperateDto.getId() || receiptOperateDto.getId() <= 0) {
            return ResponseJson.error("收款Id不能为空", null);
        }
        if (null == receiptOperateDto.getShopOrderId() || receiptOperateDto.getShopOrderId() <= 0) {
            return ResponseJson.error("关联供应商退款子订单ID不能为空", null);
        }
        return receiptService.receiptAssociateRefund(receiptOperateDto, headers);
    }

    /**
     * 确认收款-关联返佣款
     * crm:/api/user/rebate/relation
     *
     * @param receiptOperateDto {
     *                          id:           收款Id
     *                          shopOrderId: 关联返佣子订单ID
     *                          rebateRemarks:关联返佣备注
     *                          }
     */
    @ApiOperation("确认收款-关联返佣款")
    @PostMapping("/confirm/rebate")
    public ResponseJson<Void> receiptAssociateRebate(ReceiptOperateDto receiptOperateDto, @RequestHeader HttpHeaders headers) {
        if (null == receiptOperateDto.getId() || receiptOperateDto.getId() <= 0) {
            return ResponseJson.error("收款Id不能为空", null);
        }
        if (null == receiptOperateDto.getShopOrderId()) {
            return ResponseJson.error("关联返佣子订单ID不能为空", null);
        }
        return receiptService.receiptAssociateRebate(receiptOperateDto, headers);
    }

    /**
     * 确认收款-关联口头佣金
     * crm:/api/user/rebate/verballyReceipt
     *
     * @param receiptOperateDto {
     *                          verbalAmount: 口头返佣佣金
     *                          shopOrderId: 关联返佣子订单ID
     *                          rebateRemarks:关联返佣备注
     *                          }
     */
    @ApiOperation("确认收款-关联口头佣金")
    @PostMapping("/confirm/rebate/verbal")
    public ResponseJson<Integer> associateVerbalRebate(ReceiptOperateDto receiptOperateDto, @RequestHeader HttpHeaders headers) {
        if (null == receiptOperateDto.getVerbalAmount()) {
            return ResponseJson.error("口头返佣佣金参数异常", null);
        }
        if (null == receiptOperateDto.getShopOrderId()) {
            return ResponseJson.error("关联返佣子订单ID不能为空", null);
        }
        return receiptService.associateVerbalRebate(receiptOperateDto, headers);
    }

    /**
     * 审核收款信息
     * crm:/api/user/receivables/checkReceipt
     *
     * @param receiptOperateDto {
     *                          id            收款Id
     *                          receiptStatus 收款状态
     *                          reviewReason  审核不通过原因
     *                          }
     */
    @ApiOperation("审核收款信息")
    @PostMapping("/audit")
    public ResponseJson<Void> receiptAudit(ReceiptOperateDto receiptOperateDto, @RequestHeader HttpHeaders headers) {
        if (null == receiptOperateDto.getId() || receiptOperateDto.getId() <= 0) {
            return ResponseJson.error("收款Id不能为空", null);
        }
        if (null == receiptOperateDto.getReceiptStatus() || receiptOperateDto.getReceiptStatus() <= 0) {
            return ResponseJson.error("收款状态不能为空", null);
        }
        return receiptService.receiptAudit(receiptOperateDto.getId(), receiptOperateDto.getReceiptStatus(), receiptOperateDto.getReviewReason(), headers);
    }


    @ApiOperation("待确认订单款供应商信息")
    @GetMapping("/shop")
    public ResponseJson<PageInfo<ShopOrderVo>> ReceiptDetailVo(String keyWord, Integer newReceiptType, @RequestParam(value = "pageNum", defaultValue = "1") int pageNum, @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (newReceiptType != 1) {
            //1采美已上线供应商  3，4：认证通供应商
            return receiptService.ReceiptDetails(keyWord, pageNum, pageSize);
        } else {
            return receiptService.ReceiptDetailVo(keyWord, pageNum, pageSize);
        }
    }

    @ApiOperation("订单返佣-非返佣订单信息")
    @GetMapping("/shoporder")
    public ResponseJson<PageInfo<ShopOrderVo>> ReceiptDetail(String keyWord, @RequestParam(value = "pageNum", defaultValue = "1") int pageNum, @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        return receiptService.ReceiptDetail(keyWord, pageNum, pageSize);
    }

}

