package com.caimei365.order.controller;

import com.alibaba.excel.EasyExcel;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.*;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.ShipService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

/**
 * 供应商发货API
 *
 * @author : Charles
 * @date : 2021/8/5
 */
@Api(tags = "供应商发货API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/shop")
@Slf4j
public class ShipApi {
    private final ShipService shipService;


    /**
     * 供应商订单列表
     */
    @ApiOperation("供应商订单列表（旧：/supplier/myOrderList）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "shopId", value = "供应商Id"),
            @ApiImplicitParam(required = false, name = "sendOutStatus", value = "发货状态"),
            @ApiImplicitParam(required = false, name = "payStatus", value = "结算状态"),
            @ApiImplicitParam(required = false, name = "shopOrderNo", value = "订单编号"),
            @ApiImplicitParam(required = false, name = "receiver", value = "买家名称(收货人)"),
            @ApiImplicitParam(required = false, name = "refundStatus", value = "退款状态"),
            @ApiImplicitParam(required = false, name = "receiptStatus", value = "收款状态"),
            @ApiImplicitParam(required = false, name = "organizeId", value = "订单来源"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/list")
    public ResponseJson<PageInfo<ShopOrderVo>> getShopOrderList(Integer shopId, Integer sendOutStatus, Integer payStatus,
                                                                String shopOrderNo, String receiver, Integer refundStatus, Integer receiptStatus, Integer organizeId,
                                                                @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                                @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (null == shopId) {
            return ResponseJson.error("供应商Id不能为空！", null);
        }
        return shipService.getShopOrderList(shopId, sendOutStatus, payStatus, shopOrderNo, receiver, refundStatus, receiptStatus, organizeId, pageNum, pageSize);
    }

    /**
     * 供应商结算列表
     */
    @ApiOperation("供应商结算列表（旧：/supplier/settlementList）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "shopId", value = "供应商Id"),
            @ApiImplicitParam(required = false, name = "shopOrderNo", value = "订单编号"),
            @ApiImplicitParam(required = false, name = "payStatus", value = "结算状态"),
            @ApiImplicitParam(required = false, name = "name", value = "买家名称"),
            @ApiImplicitParam(required = false, name = "beginTime", value = "订单开始时间"),
            @ApiImplicitParam(required = false, name = "endTime", value = "订单结束时间"),
            @ApiImplicitParam(required = false, name = "refundStatus", value = "退款状态"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/settlement/list")
    public ResponseJson<PageInfo<SettlementVo>> getSettlementList(Integer shopId, String shopOrderNo, Integer payStatus,
                                                                  String name, String beginTime, String endTime, Integer refundStatus,
                                                                  @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                                  @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (null == shopId) {
            return ResponseJson.error("供应商Id不能为空！", null);
        }
        return shipService.getSettlementList(shopId, payStatus, shopOrderNo, name, beginTime, endTime, refundStatus, pageNum, pageSize);
    }

    /**
     *导出结算列表
     * @param shopId
     * @param shopOrderNo
     * @param payStatus
     * @param name
     * @param beginTime
     * @param endTime
     * @param refundStatus
     * @param response
     */
    @PostMapping("/exportExcel")
    public void exportExcel(Integer shopId, String shopOrderNo, Integer payStatus, String name, String beginTime, String endTime, Integer refundStatus, HttpServletResponse response) {
        String file = "结算列表数据";
        try {
            List<SettlementDto> list = shipService.exportExcel(shopId, payStatus, shopOrderNo, name, beginTime, endTime, refundStatus);
            SettlementDto settLementDto = new SettlementDto();
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");
            // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
            String fileName = URLEncoder.encode(file, "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), settLementDto.getClass()).autoCloseStream(Boolean.TRUE).sheet("数据").doWrite(list);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 供应商订单详情
     */
    @ApiOperation("供应商订单详情（旧：/supplier/shopOrderDetails）")
    @ApiImplicitParam(required = true, name = "shopOrderId", value = "子订单Id")
    @GetMapping("/detail")
    public ResponseJson<Map<String, Object>> getShopOrderDetail(Integer shopOrderId) {
        if (null == shopOrderId) {
            return ResponseJson.error("参数不能为空！", null);
        }
        return shipService.getShopOrderDetail(shopOrderId);
    }

    /**
     * 供应商售货清单
     */
    @ApiOperation("供应商售货清单（旧：/supplier/salesList）")
    @ApiImplicitParam(required = true, name = "shopOrderId", value = "子订单Id")
    @GetMapping("/sales/list")
    public ResponseJson<Map<String, Object>> getSalesList(Integer shopOrderId) {
        if (null == shopOrderId) {
            return ResponseJson.error("参数不能为空！", null);
        }
        return shipService.getSalesList(shopOrderId);
    }

    /**
     * 发货页面订单数据
     */
    @ApiOperation("发货页面订单数据（旧：/supplier/shipmentsInfo）")
    @ApiImplicitParam(required = true, name = "shopOrderId", value = "子订单Id")
    @GetMapping("/ship/info")
    public ResponseJson<ShopOrderVo> getShopOrderInfo(Integer shopOrderId) {
        if (null == shopOrderId) {
            return ResponseJson.error("参数不能为空！", null);
        }
        return shipService.getShopOrderInfo(shopOrderId);
    }

    /**
     * 物流公司
     */
    @ApiOperation("物流公司（旧：/supplier/logisticsCompany）")
    @ApiImplicitParam(required = true, name = "value", value = "快递公司代码")
    @GetMapping("/ship/company")
    public ResponseJson<List<CompanyVo>> getLogisticsCompany(String value) {
        return shipService.getLogisticsCompany(value);
    }

    /**
     * 发货
     *
     * @param logisticsDto {
     *                     shopOrderId: 子订单Id,
     *                     note: 备注信息,
     *                     image: 图片备注以##隔开,
     *                     logistics：[  // 物流信息
     *                     {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'},
     *                     {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'}
     *                     ],
     *                     products：[   // 商品信息
     *                     {orderProductId:订单商品id,num:此次发货数量},
     *                     {orderProductId:订单商品id,num:此次发货数量}
     *                     ],
     *                     }
     */
    @ApiOperation("供应商发货（旧：/supplier/addLogistics）")
    @PostMapping("/ship/delivery")
    public ResponseJson<Integer> addDelivery(LogisticsDto logisticsDto) {
        if (null == logisticsDto.getLogistics()) {
            return ResponseJson.error("物流信息不能为空！", null);
        }
        if (null == logisticsDto.getShopOrderId()) {
            return ResponseJson.error("子订单Id不能为空！", null);
        }
        if (null == logisticsDto.getProducts()) {
            return ResponseJson.error("商品信息不能为空！", null);
        }
        return shipService.addDelivery(logisticsDto);
    }

    /**
     * 供应商发货记录
     */
    @ApiOperation("供应商发货记录（旧：/supplier/shipmentsRecord）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "shopOrderId", value = "子订单Id"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/ship/delivery/record")
    public ResponseJson<Map<String, Object>> getDeliveryRecord(Integer shopOrderId,
                                                               @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                               @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (null == shopOrderId) {
            return ResponseJson.error("子订单Id不能为空！", null);
        }
        return shipService.getDeliveryRecord(shopOrderId, pageNum, pageSize);
    }

    /**
     * 供应商撤销发货
     *
     * @param logisticsBatchDto {
     *                          logisticsBatchId: 发货批次表Id
     *                          }
     */
    @ApiOperation("供应商撤销发货（旧：/supplier/cancelShipment）")
    @PostMapping("/ship/delivery/cancel")
    public ResponseJson<Void> cancelDelivery(LogisticsBatchDto logisticsBatchDto) {
        if (null == logisticsBatchDto.getLogisticsBatchId()) {
            return ResponseJson.error("发货批次表Id不能为空！", null);
        }
        return shipService.cancelDelivery(logisticsBatchDto.getLogisticsBatchId());
    }

    /**
     * 当前批次发货商品数据
     */
    @ApiOperation("当前批次发货商品数据（旧：/supplier/logisticsRecord）")
    @ApiImplicitParam(required = true, name = "logisticsInfoId", value = "发货物流信息Id")
    @GetMapping("/ship/logistics/record")
    public ResponseJson<List<LogisticsRecordVo>> getLogisticsRecord(Integer logisticsBatchId) {
        if (null == logisticsBatchId) {
            return ResponseJson.error("发货物流信息Id不能为空！", null);
        }
        return shipService.getLogisticsRecord(logisticsBatchId);
    }

    /**
     * 发货记录-添加物流
     *
     * @param logisticsBatchDto {
     *                          logisticsBatchId: 发货批次表Id,
     *                          remark: 备注,
     *                          remarkImage: 图片备注,
     *                          logistics：[  // 物流信息
     *                          {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'},
     *                          {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'}
     *                          ]
     *                          }
     */
    @ApiOperation("发货记录-添加物流（旧：/supplier/addLogisticsInfo）")
    @PostMapping("/ship/logistics/add")
    public ResponseJson<Void> addLogisticsInfo(LogisticsBatchDto logisticsBatchDto) {
        if (null == logisticsBatchDto.getLogisticsBatchId()) {
            return ResponseJson.error("发货批次表Id不能为空！", null);
        }
        return shipService.addLogisticsInfo(logisticsBatchDto);
    }

    /**
     * 扫码添加物流
     *
     * @param number 快递单号
     * @param mobile 收/寄件人的电话号码(顺丰必填)
     */
    @ApiOperation("扫码添加物流（旧：/supplier/scanLogistics）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "number", value = "快递单号"),
            @ApiImplicitParam(required = true, name = "mobile", value = "收/寄件人的电话号码(顺丰必填)")
    })
    @PostMapping("/ship/logistics/scan")
    public ResponseJson<CompanyVo> scanGetLogisticsInfo(String number, String mobile) {
        if (StringUtils.isEmpty(number)) {
            return ResponseJson.error("快递单号不能为空！", null);
        }
        return shipService.scanGetLogisticsInfo(number, mobile);
    }

    /**
     * 供应商查看物流
     */
    @ApiOperation("供应商查看物流（旧：/supplier/logisticsInfo）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "shopOrderId", value = "子订单Id"),
            @ApiImplicitParam(required = true, name = "logisticsBatchId", value = "发货物流批次Id")
    })
    @GetMapping("/ship/logistics/info")
    public ResponseJson<Map<String, Object>> getLogisticsInfo(Integer shopOrderId, Integer logisticsBatchId) {
        if (null == shopOrderId) {
            return ResponseJson.error("子订单Id不能为空！", null);
        }
        if (null == logisticsBatchId) {
            return ResponseJson.error("发货物流批次Id不能为空！", null);
        }
        return shipService.getLogisticsInfo(shopOrderId, logisticsBatchId);
    }

    /**
     * 删除物流信息
     */
    @ApiOperation("删除物流信息（旧：/supplier/logisticsInfo/delete）")
    @ApiImplicitParam(required = true, name = "logisticsInfoId", value = "发货物流信息Id")
    @GetMapping("/ship/logistics/delete")
    public ResponseJson<Void> deleteLogisticsInfo(Integer logisticsBatchId) {
        if (null == logisticsBatchId) {
            return ResponseJson.error("发货物流信息Id不能为空！", null);
        }
        return shipService.deleteLogisticsInfo(logisticsBatchId);
    }

    /**
     * 小程序分享发货-获取分享码
     */
    @ApiOperation("小程序分享发货-获取分享码（旧：/supplier/shopShareShipments）")
    @ApiImplicitParam(required = true, name = "shopOrderId", value = "子订单Id")
    @GetMapping("/share/code")
    public ResponseJson<String> getShipShareCode(Integer shopOrderId) {
        if (null == shopOrderId) {
            return ResponseJson.error("子订单Id不能为空！", null);
        }
        return shipService.getShipShareCode(shopOrderId);
    }

    /**
     * 小程序分享发货-分享权限控制
     */
    @ApiOperation("小程序分享发货-分享权限控制（旧：/supplier/miniShipments）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "source", value = "来源：1:客服扫码,2:供应商运营人员查看,3:分享码查看"),
            @ApiImplicitParam(required = false, name = "shopOrderId", value = "子订单Id"),
            @ApiImplicitParam(required = false, name = "shareCode", value = "分享码"),
            @ApiImplicitParam(required = false, name = "code", value = "微信code"),
            @ApiImplicitParam(required = false, name = "encryptedData", value = "微信加密数据"),
            @ApiImplicitParam(required = false, name = "iv", value = "微信加密算法的初始向量")
    })
    @GetMapping("/share/code/check")
    public ResponseJson<Map<String, Object>> checkShipShareCode(Integer source, Integer shopOrderId, String shareCode,
                                                                String code, String encryptedData, String iv, @RequestHeader HttpHeaders headers) {
        if (null == source) {
            return ResponseJson.error("来源不能为空！", null);
        }
        if (null == shopOrderId) {
            return ResponseJson.error("子订单Id不能为空！", null);
        }
        if (StringUtils.isEmpty(code)) {
            return ResponseJson.error("微信code不能为空！", null);
        }
        return shipService.checkShipShareCode(source, shopOrderId, shareCode, code, encryptedData, iv, headers);
    }

    /*
     * 供应商分享发货-绑定运营人员(/supplier/shopBindingWx), 可以直接用 用户模块->运营人员绑定微信接口(/user/login/auth/bind)
     */


    /**
     * 查看商品资质信息
     */
    @ApiOperation("查看商品资质信息（旧：/supplier/qualification/data）")
    @ApiImplicitParam(required = true, name = "logisticsBatchId", value = "发货物流批次Id")
    @GetMapping("/qualification/list")
    public ResponseJson<List<QualificationVo>> getProductQualification(Integer logisticsBatchId) {
        if (null == logisticsBatchId) {
            return ResponseJson.error("发货物流信息Id不能为空！", null);
        }
        return shipService.getProductQualification(logisticsBatchId);
    }

    /**
     * 保存商品资质
     *
     * @param jsonParamsDto params:[{
     *                      recordId: 商品发货记录id,
     *                      sn:       sn码,
     *                      files:    [{fileName:"文件名称",ossName:"oss文件名称"},{fileName:"文件名称",ossName:"oss文件名称"}...]
     *                      images:   ["图片","图片",...]
     *                      }, {}]
     */
    @ApiOperation("上传商品资质（旧：/supplier/qualification）")
    @PostMapping("/qualification/save")
    public ResponseJson<Void> saveProductQualification(JsonParamsDto jsonParamsDto) {
        return shipService.saveProductQualification(jsonParamsDto);
    }

    /**
     * 更改收款状态 -- 确认收款
     *
     * @param amount
     * @param shopOrderId
     * @return
     */
    @GetMapping("/check/receipt")
    public ResponseJson checkReceipt(Double amount, Integer shopOrderId) {
        if (null == amount || null == shopOrderId) {
            return ResponseJson.error("参数不能为空！");
        }
        return shipService.checkReceipt(amount, shopOrderId);
    }

    /**
     * 订单备注详情
     *
     * @param shopOrderId
     * @return
     */
    @ApiOperation("订单备注详情")
    @GetMapping(value = "/remarksView")
    public ResponseJson<List<OrderRemarkDto>> remarksView(Integer shopOrderId) {
        List<OrderRemarkDto> remarksList = shipService.findList(shopOrderId);
        return ResponseJson.success(remarksList);
    }

    /**
     * 添加订单备注
     *
     * @param remark
     * @return
     */
    @ApiOperation("添加订单备注")
    @PostMapping(value = "/add/remark")
    public ResponseJson addRemark(@RequestBody OrderRemarkDto remark) {
        shipService.insertRemark(remark);
        return ResponseJson.success();
    }

    /**
     * 上传备注文件
     *
     * @param file
     * @param orderId
     * @return
     */
    @ApiOperation("上传备注文件")
    @PostMapping("/upload/remark")
    public ResponseJson<Map<String, Object>> uploadFile(MultipartFile file, Integer orderId) {
        return shipService.upload(file, orderId);
    }

    /**
     * 下载备注文件
     *
     * @param fileId
     * @param request
     * @param response
     * @throws IOException
     */
    @ApiOperation("下载备注文件")
    @PostMapping("/download/remark")
    public void downFile(Integer fileId, HttpServletRequest request, HttpServletResponse response) throws IOException {
        shipService.downFile(fileId, request, response);
    }
}
