package com.caimei365.order.mapper;

import com.caimei365.order.model.bo.OrderParamBo;
import com.caimei365.order.model.dto.VoucherDto;
import com.caimei365.order.model.dto.VoucherImageDto;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/19
 */
@Mapper
public interface OrderClubMapper {
    /**
     * 判断订单是否存在
     *
     * @param orderId 订单Id
     */
    int countMainOrderByOrderId(Integer orderId);

    /**
     * 判断子订单是否存在
     * @param shopOrderId
     * @return
     */
    int countMainShopOrderByOrderId(Integer shopOrderId);

    /**
     * 根据订单Id查询订单
     *
     * @param orderId 订单Id
     */
    OrderPo getMainOrderByOrderId(Integer orderId);

    /**
     * 根据子订单Id 查询子订单
     * @param shopOrderId
     * @return
     */
    ShopOrderVo getShopOrderById(Integer shopOrderId);

    /**
     * 根据订单Id查询订单商品列表
     *
     * @param orderId 订单Id
     */
    List<OrderProductPo> getOrderProductList(Integer orderId);

    /**
     * 根据子订单Id查询订单商品列表
     * @param shopOrderId
     * @return
     */
    List<OrderProductPo> getShopOrderProductList(Integer shopOrderId);

    /**
     * 获取订单中子订单状态
     * @param orderId
     * @return
     */
    List<Integer> getShopOrderStatus(Integer orderId);

    /**
     * 设置二手仪器已售
     *
     * @param productId 商品Id
     */
    void updateSecondStatus(Integer productId, Integer sold);

    /**
     * 根据用户Id更新用户余额
     *
     * @param userId 用户Id
     */
    void updateUserMoneyByUserId(double userMoney, Integer userId);

    /**
     * 根据用户Id更新用户可用余额
     *
     * @param userId 用户Id
     */
    void updateAbleUserMoneyByUserId(double ableUserMoney, Integer userId);

    /**
     * 更新订单状态
     *
     * @param order 主订单
     */
    void updateOrderStatus(OrderPo order);

    /**
     * 修改订单
     * @param orderId
     */
    void updateOrder(Integer orderId);

    /**
     * 更新子订单状态
     * @param shopOrderVo
     */
    void updateShopOrderStatus(ShopOrderVo shopOrderVo);

    /**
     * 取消 余额收支记录
     */
    void cancelBalanceRecord(BalanceRecordPo balanceRecord);

    /**
     * 取消 订单
     */
    void cancelMainOrder(OrderPo order);

    /**
     * 取消订单
     * @param shopOrderVo
     */
    void cancelMainShopOrder(ShopOrderVo shopOrderVo);

    /**
     * 逻辑删除主订单
     */
    void deleteMainOrder(Integer orderId, String note);

    /**
     * 逻辑删除子订单
     *
     * @param shopOrderIds 子订单Ids
     */
    void deleteShopOrder(@Param("shopOrderIds") List<String> shopOrderIds);

    /**
     * 获取收款记录Id列表
     *
     * @param orderId 订单Id
     */
    List<String> getOrderReceiptIds(Integer orderId);

    /**
     * 根据子订单 获取收款记录Id列表
     * @param orderId
     * @return
     */
    List<String> getShopOrderReceiptIds(Integer orderId);

    /**
     * 逻辑删除订单与收款记录关系表
     *
     * @param orderId 订单Id
     */
    void deleteOrderReceiptRelation(Integer orderId);

    /**
     * 逻辑删除订单与收款记录关系表
     * @param shopOrderId
     */
    void deleteShopOrderReceiptRelation(Integer shopOrderId);

    /**
     * 逻辑删除收款记录表
     *
     * @param receiptIds 收款记录Ids
     */
    void deleteDiscernReceipt(@Param("receiptIds") List<String> receiptIds);

    /**
     * 获取付款记录Id列表
     *
     * @param shopOrderIds 子订单Ids
     */
    List<String> getPayShopIds(@Param("shopOrderIds") List<String> shopOrderIds);

    /**
     * 逻辑删除订单与付款记录关系表
     *
     * @param shopOrderIds 子订单Ids
     */
    void deletePayShopRecord(@Param("shopOrderIds") List<String> shopOrderIds);

    /**
     * 逻辑删除付款记录表
     *
     * @param payIds 付款记录Ids
     */
    void deletePayShop(@Param("payIds") List<String> payIds);

    /**
     * 更新物流状态
     *
     * @param orderId 订单Id
     */
    void updateLogisticsBatch(Integer orderId, Integer shopOrderId);

    /**
     * 获取各状态订单数量
     *
     * @param userId     用户Id
     * @param orderState 1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     */
    Integer getOrderCount(Integer userId, Integer orderState);

    /**
     * 过滤不可见订单
     *
     * @param userId
     * @param orderState
     * @return
     */
    Integer getOrderSeenCount(Integer userId, Integer orderState);

    /**
     * 机构订单列表
     *
     * @param userId     用户Id
     * @param orderState 1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderNo    订单编号
     * @param beginTime  开始时间
     * @param endTime    截止时间
     */
    List<OrderVo> getOrderList(Integer userId, Integer orderState, String orderNo, String beginTime, String endTime);

    /**
     * 机构子订单列表
     * @param userId
     * @param status
     * @param shopOrderNo
     * @param beginTime
     * @param endTime
     * @return
     */
    List<ShopOrderVo> getShopOrderList(@Param("userId") Integer userId, @Param("status") Integer status, @Param("shopOrderNo") String shopOrderNo, @Param("beginTime") String beginTime, @Param("endTime") String endTime);

    /**
     * 根据关键词搜索订单
     *
     * @param searchWord 关键词
     * @param userId     用户Id
     */
    List<OrderVo> getOrderListByKeyword(String searchWord, Integer userId);

    /**
     * 根据关键词搜索子订单
     * @param searchWord
     * @param userId
     * @return
     */
    List<ShopOrderVo> getShopOrderListByKeyword(@Param("searchWord") String searchWord, @Param("userId") Integer userId);

    /**
     * 根据用户Id查找订单搜索历史记录
     *
     * @param userId 用户Id
     */
    List<SearchHistoryVo> getOrderSearchHistory(Integer userId);

    /**
     * 根据用户Id删除订单搜索历史记录
     *
     * @param userId 用户Id
     */
    void deleteOrderSearchHistory(Integer userId);

    /**
     * 退款记录
     *
     * @param orderId 订单Id
     */
    List<ReturnedPurchaseVo> getReturnedPurchaseList(Integer orderId);

    /**
     * 售后条款
     *
     * @param clauseId 条款Id
     */
    ClauseVo getClauseById(Integer clauseId);

    /**
     * 获取物流批次子订单Id列表
     *
     * @param orderId 订单Id
     * @param id      物流批次Id
     */
    List<String> getBatchShopOrderIds(Integer orderId, Integer id);

    /**
     * 获取数据库已有分享码
     *
     * @param orderId 订单Id
     */
    OrderShareCodeVo getOrderCodeById(Integer orderId);

    /**
     * 子订单Id 获取数据库已有分享码
     * @param shopOrderId
     * @return
     */
    OrderShareCodeVo getShopOrderCodeById(Integer shopOrderId);

    /**
     * 删除分享码使用记录
     *
     * @param orderId 订单Id
     */
    void deleteOrderShareCodeRecord(Integer orderId);

    /**
     * 删除分享码使用记录
     * @param shopOrderId
     */
    void deleteShopOrderShareCodeRecord(Integer shopOrderId);

    /**
     * 更新数据库的分享码
     */

    void updateOrderShareCode(OrderShareCodeVo orderShareCode);

    /**
     * 新增数据库的分享码
     */
    void insertOrderShareCode(OrderShareCodeVo orderShareCode);

    /**
     * 根据openId统计数据库内是否有有效分享码
     */
    int countShareCodeRecord(@Param("shopOrderId") Integer shopOrderId,@Param("openId") String openId);

    /**
     * 根据用户id获取用户openid列表(微信公众号openid,下单)
     */
    List<String> getOpenidListByPermission(String unionId);

    Integer getUserId(Integer orderId);

    void AddUnionId(String unionId, Integer orderId);

    String FromUnionId(Integer orderId);

    String UserUnionId(Integer userID);

    List<Integer> getProductOrder(Integer orderId);

    List<Integer> getProductShopOrder(Integer shopOrderId);

    /**
     * 根据orderId查询商品名称
     */
    List<String> getOrderIds(Integer orderId);

    /**
     * 根据shopOrderId查询商品名称
     * @param shopOrderId
     * @return
     */
    List<String> getShopOrderIds(Integer shopOrderId);

    /**
     * 根据orderId查询订单金额
     */
    Double getpayTotalFee(Integer orderId);

    /**
     * 根据orderId查询支付时间
     */
    String getpayTime(Integer orderId);

    /**
     * 根据orderId查询收款状态
     */
    Integer getReceiptStatus(Integer orderId);

    /**
     * 根据orderId查询二手订单标识
     */
    Integer getsecondHandOrderFlag(Integer orderId);

    /**
     * 根据orderId查询返佣订单标识
     */
    Integer getrebateFlag(Integer orderId);

    /**
     * 根据orderId查询商品编号
     */
    String getOrderNo(Integer orderId);

    /**
     * 根据shopOrderId查询商品编号
     * @param shopOrderId
     * @return
     */
    String getShopOrderNo(Integer shopOrderId);

    OrderVo getOrderMess(Integer orderId);

    OrderVo getcartNess(Integer userId);

    List<Integer> userIds();

    /**
     * 保存分享码使用记录
     */
    void insertOrderShareCodeRecord(OrderShareCodeRecordVo codeRecord);

    /**
     * 根据clubCouponId获取优惠券
     */
    CouponVo getClubCouponById(Integer clubCouponId, Integer userId);

    /**
     * 修改优惠券使用情况
     */
    void updateCouponClubStatus(Integer clubCouponId, Integer orderId);

    /**
     * 查询订单优惠信息
     */
    CouponOrderRecordPo getOrderCouponRecord(Integer orderId,Integer couponClubId);

    /**
     * 查询用户优惠券关联数据
     */
    CouponClubPo getCouponClubById(Integer clubCouponId);

    /**
     * 保存用户关联信息
     */
    void insertCouponClub(CouponClubPo couponClub);

    /**
     * 删除优惠券与订单的关联信息
     */
    void deleteCouponClubById(Integer clubCouponId);

    /**
     * 机构最新订单Id
     */
    Integer getLatestOrderId(Integer userId);

    Integer findUserId(Integer orderId);

    void insertCouponRecord(CouponRecordPo couponRecordPo);

    void addMessageCenter(MessageCenter messageCenter);

    Integer getVipHistoryCount(Integer userId);

    Integer clubIdCule(Integer userId);

    List<CouponRecordPo> findCouponRemain(Integer couponId);

    void updateCodeCoupon(CouponRecordPo couponRecordPo);

    OrderParamBo findUserName(Integer userId);

    List<Sku> findSkus(Integer productId);

    List<Sku> getOrganizeSku(@Param("productId") Integer productId);

    /**
     * 发票信息
     * @param orderId
     * @return
     */
    InvoicePo getOrderInvoice(@Param("orderId") Integer orderId);

    List<VoucherPo> getVoucher(@Param("shopOrderId") Integer shopOrderId);

    List<VoucherImageDto> getVoucherImage(@Param("voucherIds") List<Integer> voucherIds);

    void updateVoucher(VoucherDto voucherDto);

    void insertVoucher(VoucherDto voucherDto);

    void updateVoucherImage(VoucherImageDto voucherImageDto);

    void insertVoucherImage(VoucherImageDto voucherImageDto);

    Integer getShopAccountOwnership(@Param("shopId") Integer shopId);

    /**
     * 查询供应商银行卡信息
     * @param shopId
     * @return
     */
    ShopPo getShopBankAccount(@Param("shopId") Integer shopId);

    /**
     * 查询供应商采美银行卡信息
     * @param shopId
     * @return
     */
    ShopPo getShopCmBankAccount(@Param("shopId") Integer shopId);
}
