package com.caimei365.order.service;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.ReceiptDto;
import com.caimei365.order.model.dto.ReceiptOperateDto;
import com.caimei365.order.model.po.ReceiptPo;
import com.caimei365.order.model.vo.*;
import com.github.pagehelper.PageInfo;
import org.springframework.http.HttpHeaders;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2022/2/11
 */
public interface ReceiptService {
    /**
     * 线下收款用户微信授权
     *
     * @param code          微信授权code
     * @param encryptedData 微信加密数据
     * @param iv            加密算法的初始向量
     * @return receiptUser
     */
    ResponseJson<ReceiptUserVo> receiptAuthorization(String code, String encryptedData, String iv, HttpHeaders headers);

    /**
     * 线下收款用户密码登录
     *
     * @param mobile   手机号
     * @param password 密码
     */
    ResponseJson<ReceiptUserVo> passwordAuthorization(String mobile, String password, String code, String encryptedData, String iv, HttpHeaders headers);

    /**
     * 获取收款类型
     */
    ResponseJson<List<cmOfflineCollectionVo>> getReceiptType();

    /**
     * 识别收款短信
     *
     * @param smsContent 收款短信
     */
    ResponseJson<ReceiptVo> readSmsMessage(String smsContent, HttpHeaders headers);

    /**
     * 保存收款信息
     */
    ResponseJson<ReceiptPo> saveReceipt(ReceiptDto receiptDto, HttpHeaders headers);

    /**
     * 操作收款信息（作废，设为返佣/普通/供应商退款）
     *
     * @param receiptOperateDto {
     *                          id：收款Id
     *                          receiptType: 1订单款，2非订单款，3返佣款 4订单款或者非订单款（因财务阶段无法区分订单非订单）, 5供应商退款
     *                          noOrderReason: 非订单款说明
     *                          delFlag: 作废标记: 0否，其余是
     *                          }
     */
    ResponseJson<ReceiptPo> operateReceipt(ReceiptOperateDto receiptOperateDto, HttpHeaders headers);

    /**
     * 获取收款列表
     *
     * @param receiptType   收款款项类型：1订单款，2非订单款，3返佣款 4订单款或者非订单款（因财务阶段无法区分订单非订单）, 5供应商退款
     * @param receiptStatus 收款状态：1待确认、2已确认(待审核)、3审核通过、4审核未通过、5收款撤销【线上支付成功为审核通过】
     * @param smsContent    收款短信
     * @param startDate     筛选开始时间
     * @param endDate       筛选结束时间
     * @param pageNum       页码
     * @param pageSize      每页数量
     */
    ResponseJson<PageInfo<ReceiptVo>> getReceiptList(Integer newReceiptType, Integer receiptType, Integer receiptStatus, String smsContent, String startDate, String endDate, int pageNum, int pageSize, HttpHeaders headers);

    /**
     * 收款详情（订单/非订单）
     *
     * @param id 款项Id
     */
    ResponseJson<ReceiptVo> getReceiptDetail(Integer id, HttpHeaders headers);

    /**
     * 收款详情的款项类型
     */
    ResponseJson<Map<String, Integer>> getReceiptDetailType(Integer id, HttpHeaders headers);

    /**
     * 收款详情（分享）
     */
    ResponseJson<ReceiptVo> getReceiptDetailShare(Integer id, HttpHeaders headers);

    /**
     * 收款详情（供应商退款）
     */
    ResponseJson<ReceiptVo> getReceiptRefundDetail(Integer id, HttpHeaders headers);

    /**
     * 收款详情（返佣款）
     */
    ResponseJson<ReceiptVo> getReceiptRebateDetail(Integer id, HttpHeaders headers);

    /**
     * 收款详情-订单列表
     *
     * @param id                 收款ID
     * @param organizeId         组织ID
     * @param orderReceiptStatus 订单收款状态：1待收款，2部分收款，3已收款
     * @param keyword            搜索关键词(客户名称/订单号)
     * @param pageNum            页码
     * @param pageSize           每页数量
     */
    ResponseJson<PageInfo<ShopOrderVo>> getReceiptOrderList(Integer id, Integer type, Integer organizeId, String orderReceiptStatus, String keyword, int pageNum, int pageSize, HttpHeaders headers);

    /**
     * 收款详情-订单详情
     *
     * @param shopOrderId 子订单Id
     */
    ResponseJson<ShopOrderVo> getReceiptOrderInfo(Integer shopOrderId, HttpHeaders headers);

    /**
     * 供应商退款-子订单列表
     *
     * @param id            收款ID
     * @param confirmedType 0待确认，2已确认子订单
     * @param shopName      供应商名称
     * @param keyword       搜索关键词(客户名称/订单号)
     * @param pageNum       页码
     * @param pageSize      每页数量
     */
    ResponseJson<PageInfo<ShopOrderVo>> getRefundShopOrderList(Integer id, Integer confirmedType, String shopName, String keyword, int pageNum, int pageSize, HttpHeaders headers);

    /**
     * 供应商退款-订单详情
     *
     * @param shopOrderId 子订单Id
     */
    ResponseJson<ShopOrderVo> getRefundShopOrderInfo(Integer shopOrderId, HttpHeaders headers);

    /**
     * 返佣款-子订单列表
     *
     * @param id       返佣收款ID
     * @param shopName 供应商名称
     * @param pageNum  页码
     * @param pageSize 每页数量
     */
    ResponseJson<PageInfo<ShopOrderVo>> getRebateShopOrderList(Integer id, String shopName, int pageNum, int pageSize, HttpHeaders headers);


    /**
     * 确认收款-关联订单款
     *
     * @param receiptOperateDto {
     *                          id: 收款Id
     *                          orderIds: 关联订单IDS
     *                          confirmType: 确认订单类型：1小额抹平确认，2大额抹平确认，3大额退款余额，4确认关联
     *                          noOrderReason: 非订单款说明
     *                          }
     */
    ResponseJson<Void> receiptAssociate(ReceiptOperateDto receiptOperateDto, HttpHeaders headers);

    /**
     * 确认收款-关联供应商退款
     *
     * @param receiptOperateDto {
     *                          id:          收款Id
     *                          shopOrderId: 关联供应商退款子订单ID
     *                          }
     */
    ResponseJson<Void> receiptAssociateRefund(ReceiptOperateDto receiptOperateDto, HttpHeaders headers);

    /**
     * 确认收款-关联返佣款
     *
     * @param receiptOperateDto {
     *                          id:           收款Id
     *                          shopOrderId: 关联返佣子订单ID
     *                          rebateRemarks:关联返佣备注
     *                          }
     */
    ResponseJson<Void> receiptAssociateRebate(ReceiptOperateDto receiptOperateDto, HttpHeaders headers);

    /**
     * 确认收款-关联口头佣金
     * crm:/api/user/rebate/verballyReceipt
     *
     * @param receiptOperateDto {
     *                          verbalAmount: 口头返佣佣金
     *                          shopOrderId: 关联返佣子订单ID
     *                          rebateRemarks:关联返佣备注
     *                          }
     */
    ResponseJson<Integer> associateVerbalRebate(ReceiptOperateDto receiptOperateDto, HttpHeaders headers);

    /**
     * 审核收款信息
     *
     * @param id            收款Id
     * @param receiptStatus 收款状态
     * @param reviewReason  审核不通过原因
     */
    ResponseJson<Void> receiptAudit(Integer id, Integer receiptStatus, String reviewReason, HttpHeaders headers);

    ResponseJson<PageInfo<ShopOrderVo>> ReceiptDetailVo(String keyWord, int pageNum, int pageSize);

    ResponseJson<PageInfo<ShopOrderVo>> ReceiptDetail(String keyWord, int pageNum, int pageSize);

    ResponseJson<PageInfo<ShopOrderVo>> ReceiptDetails(String keyWord, int pageNum, int pageSize);

    /**
     * 微信收款信息推送(待确认超时)
     */
    void sendWechatMsgConfirmTimeout(String accessToken);

    /**
     * 微信收款信息推送(待审核超时)
     */
    void sendWechatMsgAuditTimeout(String accessToken);

    ResponseJson<Void> receiptPlatformFee(ReceiptOperateDto receiptOperateDto, HttpHeaders headers);

    ResponseJson<Integer> receiptPlatformBill(ReceiptOperateDto receiptOperateDto, HttpHeaders headers);

    ResponseJson<PageInfo<ShopOrderVo>> getPlatformOrder(Integer id, String shopName, int pageNum, int pageSize, HttpHeaders headers);
}
