package com.caimei365.order.service.impl;

import com.caimei365.order.mapper.AddressMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.AddressDto;
import com.caimei365.order.model.po.AddressPo;
import com.caimei365.order.model.vo.AddressSelectVo;
import com.caimei365.order.model.vo.AddressVo;
import com.caimei365.order.service.AddressService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/2
 */
@Slf4j
@Service
public class AddressServiceImpl implements AddressService {
    @Resource
    private AddressMapper addressMapper;

    /**
     * 收货地址列表
     *
     * @param userId   用户Id
     * @param pageNum
     * @param pageSize
     */
    @Override
    public ResponseJson<PageInfo<AddressVo>> getUserAddressList(Integer userId, int pageNum, int pageSize) {
        // 分页请求数据
        PageHelper.startPage(pageNum, pageSize);
        // 收货地址列表
        List<AddressVo> addressList = addressMapper.getUserAddressList(userId);
        PageInfo<AddressVo> pageInfo = new PageInfo(addressList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * 收货地址下拉选项列表
     *
     * @param type     选项类型：0省（默认），1市，2区
     * @param parentId 父级地址Id
     */
    @Override
    public ResponseJson<List<AddressSelectVo>> getSelectAddress(Integer type, Integer parentId) {
        if (0 == type) {
            // 获取所有省份列表
            List<AddressSelectVo> provinceList = addressMapper.getAllProvinceList();
            return ResponseJson.success(provinceList);
        } else if (1 == type) {
            // 获取当前省份下的所有城市列表
            List<AddressSelectVo> cityList = addressMapper.getCityListByProvinceId(parentId);
            return ResponseJson.success(cityList);
        } else if (2 == type) {
            // 获取当前城市下的所有地区列表
            List<AddressSelectVo> townList = addressMapper.getTownListByCityId(parentId);
            return ResponseJson.success(townList);
        }
        return ResponseJson.error("地址选项类型错误！", null);
    }

    @Override
    public ResponseJson<List<AddressSelectVo>> getAllSelectAddress() {
        // 获取所有省份列表
        List<AddressSelectVo> provinceList = addressMapper.getAllProvinceList();
        provinceList.forEach(province -> {
            // 获取当前省份下的所有城市列表
            List<AddressSelectVo> cityList = addressMapper.getCityListByProvinceId(province.getId());
            cityList.forEach(city -> {
                // 获取当前城市下的所有地区列表
                List<AddressSelectVo> townList = addressMapper.getTownListByCityId(city.getId());
                city.setChildren(townList);
            });
            province.setChildren(cityList);
        });
        return ResponseJson.success(provinceList);
    }

    /**
     * 添加收货地址
     *
     * @param addressDto {
     *                   userId       用户ID
     *                   addressId    用户地址Id
     *                   townId       区ID
     *                   address      详细地址
     *                   name         收货人
     *                   mobile       手机
     *                   defaultFlag  是否默认收货地址(0 不是默认,1 默认)
     */
    @Override
    public ResponseJson<AddressVo> addUserAddress(AddressDto addressDto) {
        AddressPo address = null;
        if (null != addressDto.getAddressId()) {
            // 获取地址信息
            address = addressMapper.getAddressById(addressDto.getAddressId());
        } else {
            address = new AddressPo();
        }
        // 非默认地址
        address.setDefaultFlag(0);
        // 如果设置当前地址为默认地址
        if (null != addressDto.getDefaultFlag() && 1 == addressDto.getDefaultFlag()) {
            // 获取默认地址Id
            Integer defaultId = addressMapper.getDefaultAddressId(addressDto.getUserId());
            // 当前地址非默认地址
            boolean flag = !(null != defaultId && null != address.getAddressId() && defaultId.equals(address.getAddressId()));
            if (flag) {
                // 设取消之前默认
                addressMapper.updateAddressDefault(defaultId, 0);
                // 设置当前地址默认
                address.setDefaultFlag(1);
            }
        }
        address.setAddressId(addressDto.getAddressId());
        address.setUserId(addressDto.getUserId());
        address.setTownId(addressDto.getTownId());
        address.setAddress(addressDto.getAddress());
        address.setReceiver(addressDto.getReceiver());
        address.setMobile(addressDto.getMobile());
        if (null == address.getAddressId()) {
            // 新增地址
            addressMapper.insertAddress(address);
        } else {
            // 更新地址
            addressMapper.updateAddress(address);
        }
        AddressVo addressVo = addressMapper.getAddressByTownId(addressDto.getTownId());
        addressVo.setAddress(addressDto.getAddress());
        addressVo.setUserId(addressDto.getUserId());
        addressVo.setReceiver(addressDto.getReceiver());
        addressVo.setMobile(addressDto.getMobile());
        addressVo.setDefaultFlag(address.getDefaultFlag());
        addressVo.setAddressId(address.getAddressId());
        return ResponseJson.success(addressVo);
    }

    /**
     * 删除收货地址
     *
     * @param addressDto {
     *                   userId       用户ID
     *                   addressId    用户地址Id
     */
    @Override
    public ResponseJson<Integer> deleteUserAddress(AddressDto addressDto) {
        // 删除地址
        addressMapper.deleteAddress(addressDto.getAddressId(), addressDto.getUserId());
        return ResponseJson.success(1);
    }

    /**
     * 设为默认地址
     * @param userId    用户ID
     * @param addressId 用户地址Id
     */
    @Override
    public ResponseJson<Integer> setDefaultAddress(Integer userId, Integer addressId) {
        // 获取默认地址Id
        Integer defaultId = addressMapper.getDefaultAddressId(userId);
        // 当前地址非默认地址
        if (!addressId.equals(defaultId)) {
            // 设取消之前默认
            addressMapper.updateAddressDefault(defaultId, 0);
            AddressPo address = addressMapper.getAddressById(addressId);
            if (null == address) {
                return ResponseJson.error("地址有误！", null);
            }
            // 设置当前地址默认
            address.setDefaultFlag(1);
            // 更新地址
            addressMapper.updateAddress(address);
        }
        return ResponseJson.success(1);
    }
}
