package com.caimei365.order.service.impl;

import com.caimei365.order.components.OrderCommonService;
import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.OrderSellerMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.vo.OrderVo;
import com.caimei365.order.model.vo.ShopOrderVo;
import com.caimei365.order.service.OrderSellerService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/19
 */
@Slf4j
@Service
public class OrderSellerServiceImpl implements OrderSellerService {
    @Resource
    private OrderSellerMapper orderSellerMapper;
    @Resource
    private OrderCommonService orderCommonService;
    @Resource
    private BaseMapper baseMapper;

    /**
     * 协销订单列表
     *
     * @param serviceProviderId 协销Id
     * @param listType          1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderSubmitType   1：自主订单， 2:协销或后台订单 ,3:组员订单
     * @param shopOrderNo           订单编号
     * @param pageNum           页码
     * @param pageSize          每页数量
     */
    @Override
    public ResponseJson<PageInfo<ShopOrderVo>> getOrderList(Integer serviceProviderId, Integer type,Integer listType, Integer orderSubmitType, String shopOrderNo, Integer groupServiceId, int pageNum, int pageSize) {
        // 获取该协销下的机构用户Id
        List<Integer> clubUserIds = new ArrayList<>();
        if(null==type){
            if (null != orderSubmitType && 3 == orderSubmitType) {
                if (null != groupServiceId && groupServiceId > 0) {
                    //组员协销id不为空的时候，查某个组员名下的单
                    clubUserIds = orderSellerMapper.getClubUserIds(groupServiceId);
                } else {
                    //查组员订单的时候查该协销是组长还是总管理员，组长查自己名下所有组员机构的订单，管理员查所有其他组员
                    clubUserIds = orderSellerMapper.getClubUserIdsByRole(serviceProviderId);
                }
            } else {
                clubUserIds = orderSellerMapper.getClubUserIds(serviceProviderId);
            }
        }else {
            if (null != orderSubmitType && 3 == orderSubmitType) {
                clubUserIds = orderSellerMapper.getDistributionClubUserIds(null,serviceProviderId);
            }else {
                clubUserIds = orderSellerMapper.getDistributionClubUserIds(serviceProviderId,null);
            }
        }

        List<ShopOrderVo> shopOrderList = new ArrayList<>();
        // 获取订单列表
        if (null != clubUserIds && clubUserIds.size() > 0) {
            // 分页请求数据
            PageHelper.startPage(pageNum, pageSize);
            shopOrderList = orderSellerMapper.getShopOrderList(clubUserIds, listType, orderSubmitType, shopOrderNo);
            //设置主订单数据
            shopOrderList.forEach(this::setSellerShopOrder);
            PageInfo<ShopOrderVo> pageInfo = new PageInfo(shopOrderList);
            return ResponseJson.success(pageInfo);
        } else {
            return ResponseJson.success(new PageInfo<ShopOrderVo>(shopOrderList));
        }
    }

    /**
     * 设置协销订单
     */
    private void setSellerOrder(OrderVo order) {
        // 设置订单状态
        orderCommonService.setOrderStatus(order);
        // 设置子订单数据
        orderCommonService.getShopOrderData(order);
        // 设置付款金额
        orderCommonService.getDiscernReceiptAndSetOrder(order);
        // 设置机构名称
        String clubName = baseMapper.getClubNameById(order.getClubId());
        order.setClubName(clubName);
        // 设置机构类型
        Integer svipUserId = baseMapper.getSvipUserIdByClubId(order.getClubId());
        order.setSvipUserFlag(null != svipUserId ? 1 : 0);
        Integer userIdentity = baseMapper.getIdentityByUserId(order.getUserId());
        order.setUserIdentity(userIdentity);
    }

    /**
     * 设置协销订单
     */
    private void setSellerShopOrder(ShopOrderVo shopOrder) {
        // 设置订单状态
        orderCommonService.setShopOrderStatus(shopOrder);
        // 设置子订单数据
        orderCommonService.setShopOrderInfo(shopOrder);
        // 设置付款金额
        orderCommonService.getDiscernReceiptAndSetShoporder(shopOrder);
        // 设置机构名称
        String clubName = baseMapper.getClubNameById(shopOrder.getClubId());
        shopOrder.setClubName(clubName);
        // 设置机构类型
        Integer svipUserId = baseMapper.getSvipUserIdByClubId(shopOrder.getClubId());
        shopOrder.setSvipUserFlag(null != svipUserId ? 1 : 0);
        Integer userIdentity = baseMapper.getIdentityByUserId(shopOrder.getUserId());
        shopOrder.setUserIdentity(userIdentity);
    }

    /**
     * 协销下机构总订单列表
     *
     * @param serviceProviderId 协销Id
     * @param clubId            机构Id
     * @param listType          1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderSubmitType   1：自主订单， 2:协销或后台订单
     * @param pageNum           页码
     * @param pageSize          每页数量
     */
    @Override
    public ResponseJson<PageInfo<ShopOrderVo>> getOrderListClub(Integer serviceProviderId, Integer clubId, Integer listType, Integer orderSubmitType, Integer type, int pageNum, int pageSize) {
        // 判断该机构是不是该协销下的 1342采美默认协销经理(官方账号)协销id1342为默认，不受限制
        Integer spId = orderSellerMapper.getSpIdByClubId(clubId);
        Integer identity = baseMapper.getUserIdentity(baseMapper.getUserIdByClubId(clubId), null,null);
        // 协销组长和总管理员也可以看
        Integer leaderId = orderSellerMapper.findLeaderId(spId);
        Integer managerId = orderSellerMapper.findManagerId();
        boolean role = serviceProviderId.equals(leaderId) || serviceProviderId.equals(managerId)||identity==8;
        if (!role && !serviceProviderId.equals(spId)) {
            if (1342 != spId) {
                //不是总管理员或组长，看是不是本人
                return ResponseJson.error("该机构不是该协销下的机构！", null);
            }
        }
        // 分页请求数据
        PageHelper.startPage(pageNum, pageSize);
        // 获取订单列表
        List<ShopOrderVo> shopOrderList = orderSellerMapper.getShopOrderListClub(clubId, listType, orderSubmitType);
        //设置主订单数据
        shopOrderList.forEach(this::setSellerShopOrder);
        PageInfo<ShopOrderVo> pageInfo = new PageInfo(shopOrderList);
        // 更新协销查看机构订单时间
        orderSellerMapper.updateClubLastCheckOrderDate(clubId, new Date());
        return ResponseJson.success(pageInfo);
    }

    /**
     * 协销根据关键词搜索订单
     *
     * @param serviceProviderId 协销Id
     * @param searchWord        搜索关键词
     * @param pageNum           页码
     * @param pageSize          每页数量
     */
    @Override
    public ResponseJson<PageInfo<ShopOrderVo>> getOrderListByKeyword(Integer serviceProviderId, String searchWord, int pageNum, int pageSize) {
        // 获取协销用户Id
        Integer spUserId = orderSellerMapper.getUserIdBySpId(serviceProviderId);
        if (null == spUserId) {
            return ResponseJson.error("协销用户异常！", null);
        }
        // 设置搜索关键词历史记录
        orderCommonService.setHistoryRecord(spUserId, searchWord);
        // 获取该协销下的机构用户Id
        List<Integer> clubUserIds = orderSellerMapper.getClubUserIds(serviceProviderId);
        // 分页请求数据
        PageHelper.startPage(pageNum, pageSize);
        // 获取订单列表
        List<ShopOrderVo> shopOrderList = orderSellerMapper.getShopOrderListByKeyword(clubUserIds, searchWord);
        //设置主订单数据
        shopOrderList.forEach(this::setSellerShopOrder);
        PageInfo<ShopOrderVo> pageInfo = new PageInfo(shopOrderList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * 获取协销各状态订单数量
     *
     * @param serviceProviderId 协销Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getSellerOrderCount(Integer serviceProviderId) {
        Map<String, Object> map = new HashMap<>(7);
        // 获取协销用户下的机构ID列表
        List<Integer> clubUserIds = orderSellerMapper.getClubIdsBySellerId(serviceProviderId);
        // 1.全部数量
        Integer totalCount = 0;
        if (null != clubUserIds && clubUserIds.size() > 0) {
            totalCount = orderSellerMapper.getSellerOrderCount(clubUserIds, 0);
        }
        map.put("totalCount", totalCount);
        // 2.待确认数量
        Integer confirmedCount = orderSellerMapper.getSellerOrderCount(clubUserIds, 1);
        map.put("confirmedCount", confirmedCount);
        // 3.待付款数量
        Integer paymentCount = orderSellerMapper.getSellerOrderCount(clubUserIds, 2);
        paymentCount = paymentCount == null ? 0 : paymentCount;
        map.put("paymentCount", paymentCount);
        // 4.待发货数量
        Integer waitShipmentsCount = orderSellerMapper.getSellerOrderCount(clubUserIds, 3);
        waitShipmentsCount = waitShipmentsCount == null ? 0 : waitShipmentsCount;
        map.put("waitShipmentsCount", waitShipmentsCount);
        // 5.已发货数量
        Integer shipmentsCount = orderSellerMapper.getSellerOrderCount(clubUserIds, 4);
        shipmentsCount = shipmentsCount == null ? 0 : shipmentsCount;
        map.put("shipmentsCount", shipmentsCount);
        // 6.退货款数量
        Integer salesReturnCount = orderSellerMapper.getSellerOrderCount(clubUserIds, 5);
        salesReturnCount = salesReturnCount == null ? 0 : salesReturnCount;
        map.put("salesReturnCount", salesReturnCount);
        // 7.未查看订单数量
        int uncheckedOrderCount = 0;
        for (Integer userId : clubUserIds) {
            // 最后查看订单时间
            Date lastCheckOrderDate = orderSellerMapper.getLastCheckOrderDate(userId);
            Integer i = orderSellerMapper.getUncheckedOrderCount(userId, lastCheckOrderDate);
            if (null != i) {
                uncheckedOrderCount += i;
            }
        }
        map.put("uncheckedOrderCount", uncheckedOrderCount);
        return ResponseJson.success(map);
    }
}
