package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.feign.CommodityFeign;
import com.caimei365.order.feign.ToolsFeign;
import com.caimei365.order.feign.UserFeign;
import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.MessagePushMapper;
import com.caimei365.order.model.vo.PushMessageUserVo;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.utils.CodeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 远程调用数据处理
 *
 * @author : Charles
 * @date : 2021/7/15
 */
@Slf4j
@Service
public class RemoteCallServiceImpl implements RemoteCallService {
    @Value("${spring.cloud.config.profile}")
    private String profile;
    @Resource
    private MessagePushMapper messagePushMapper;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private ToolsFeign toolsFeign;
    @Resource
    private UserFeign userFeign;
    @Resource
    private CommodityFeign commodityFeign;

    /**
     * 生成短链接
     *
     * @param length 链接长度
     * @param markId 跳转类型
     * @param url    跳转地址
     */
    @Override
    public String getShortLink(int length, int markId, String url) {
        String shortLink = CodeUtil.generateShortLink(length);
        Integer id = messagePushMapper.findIdByShortLink(shortLink);
        if (id != null && id > 0) {
            getShortLink(length, markId, url);
        }
        messagePushMapper.insertShortLink(markId, shortLink, url);
        return shortLink;
    }

    /**
     * 请求发短信接口
     *
     * @param markId  跳转类型
     * @param mobile  手机号
     * @param content 内容
     */
    @Override
    public boolean getSendSms(Integer type, int markId, String mobile, String content) {
        AtomicBoolean returnValue = new AtomicBoolean(false);
        try {
            //测试环境手机号允许发短信
            List<String> list = new ArrayList<>();
            list.add("18670511721");
            list.add("18229303772");
            list.add("15017915300");
            list.add("17798247695");
            list.add("15917362709");
            list.add("15814011616");
            list.add("15827317748");
            list.add("13100721916");
            list.add("15113936829");
            list.add("18175515644");
            if ("prod".equals(profile) || list.contains(mobile)) {
                if (StringUtils.isNotBlank(mobile) && mobile.length() == 11) {
                    String regex = "^(1[3-9]\\d{9}$)";
                    Pattern pattern = Pattern.compile(regex);
                    Matcher matcher = pattern.matcher(mobile);
                    if (matcher.matches()) {
                        // 调用 ToolsFeign 发送短信
                        String jsonStr = toolsFeign.getSendSms(type, mobile, content);
                        JSONObject parseObject = JSONObject.parseObject(jsonStr);
                        if (0 == parseObject.getInteger("code")) {
                            if (markId > 0) {
                                // 保存短信发送条数+count
                                messagePushMapper.updateSmsSendCount(markId, 1);
                            }
                            returnValue.set(true);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("短信推送异常:", e);
        }
        return returnValue.get();
    }

    /**
     * 微信小程序授权登录（调用user服务，发送服务间调用POST请求）
     *
     * @param code          微信code
     * @param encryptedData 微信加密数据
     * @param iv            微信加密算法的初始向量
     * @param headers       HttpHeaders
     */
    @Override
    public String appletsAuthorization(String code, String encryptedData, String iv, HttpHeaders headers) {
        AtomicReference<String> resultData = new AtomicReference<>("");
        // 获取当前微信小程序的环境
        String referer = headers.getFirst("Referer");
        try {
            // 调用 UserFeign 获取物流
            String jsonStr = userFeign.appletsAuthorization(code, encryptedData, iv, referer);
            JSONObject parseObject = JSONObject.parseObject(jsonStr);
            if (0 == parseObject.getInteger("code") || -4 == parseObject.getInteger("code")) {
                // 授权登录成功（游客-4）
                resultData.set(parseObject.getString("data"));
            }
        } catch (Exception e) {
            log.error("获取微信小程序授权失败:", e);
        }
        return resultData.get();
    }

    /**
     * 微信小程序授权登录（调用user服务，发送服务间调用POST请求） - 组织
     *
     * @param code          微信code
     * @param encryptedData 微信加密数据
     * @param iv            微信加密算法的初始向量
     * @param headers       HttpHeaders
     * @return
     */
    @Override
    public String appletsOrganizeAuthorization(String code, String encryptedData, String iv, HttpHeaders headers, Integer userId) {
        // 用户组织Id
        Integer organizeId = baseMapper.getOrganizeId(userId);
        AtomicReference<String> resultData = new AtomicReference<>("");
        // 获取当前微信小程序的环境
        String referer = headers.getFirst("Referer");
        try {
            // 调用 UserFeign 获取物流
            String jsonStr = "";
            if (0 != organizeId) {
                jsonStr = userFeign.appletsOrganizeAuthorization(code, encryptedData, iv, referer);
            } else {
                jsonStr = userFeign.appletsAuthorization(code, encryptedData, iv, referer);
            }
            log.info("jsonStr======"+jsonStr);
            JSONObject parseObject = JSONObject.parseObject(jsonStr);
            if (0 == parseObject.getInteger("code") || -4 == parseObject.getInteger("code")) {
                // 授权登录成功（游客-4）
                resultData.set(parseObject.getString("data"));
            }
        } catch (Exception e) {
            log.error("获取微信小程序授权失败:", e);
        }
        return resultData.get();
    }

    /**
     * 智能获取物流信息
     *
     * @param number      快递单号
     * @param companyCode 快递公司的编码
     * @param mobile      收/寄件人的电话号码(顺丰必填)
     */
    @Override
    public String getLogisticsByNumber(String number, String companyCode, String mobile) {
        AtomicReference<String> resultData = new AtomicReference<>("");
        try {
            // 调用 ToolsFeign 获取物流
            String jsonStr = toolsFeign.getLogisticsByNumber(number, companyCode, mobile);
            log.info("智能获取物流信息:" + jsonStr);
            JSONObject parseObject = JSONObject.parseObject(jsonStr);
            if (0 == parseObject.getInteger("code")) {
                // 获取物流信息成功
                resultData.set(parseObject.getString("data"));
            } else {
                // 获取物流失败，返回错误信息
                resultData.set(jsonStr);
            }
        } catch (Exception e) {
            log.error("获取物流信息失败:", e);
        }
        return resultData.get();
    }

    /**
     * 更新商品销量
     *
     * @param productInfo [ // 商品id，数量
     *                    {"productId": 2789, "productCount": 1},
     *                    {"productId": 2790, "productCount": 2}
     *                    ]
     */
    @Override
    public String productSaleUpdate(String productInfo) {
        AtomicReference<String> resultData = new AtomicReference<>("");
        try {
            // 调用 ToolsFeign 获取物流
            String jsonStr = commodityFeign.productSaleUpdate(productInfo);
            log.info("更新商品销量:" + productInfo);
            resultData.set(jsonStr);
        } catch (Exception e) {
            log.error("更新商品销量失败:", e);
        }
        return resultData.get();
    }

    @Override
    public void getOldUser() {

//        List<PushMessageUserVo> pushUsers = new ArrayList<>();
        List<PushMessageUserVo> pushUsers = messagePushMapper.findOldUser();
        // https://www.caimei365.com/page-390.html Zero peel点震刀
        // String url = "https://www.caimei365.com/page-390.html";
        List<String> list = new ArrayList<String>();
        /*list.add("15917362709");
        list.add("18476937515");
        list.add("18605868824");*/
        list.add("18670511721");
        list.add("17798247695");
        list.add("15017915300");
        list.forEach(l -> {
            PushMessageUserVo pushMessageUserVo = new PushMessageUserVo();
            pushMessageUserVo.setMobile(l);
            pushUsers.add(pushMessageUserVo);
        });
        pushUsers.forEach(p -> {
            /*p.setJumpLink(url);
            String link = getLink();
            p.setShortLink(link);
            p.setMarkId(23);
            messagePushMapper.insertNewLink(p);*/
            // 优惠来袭！韩国进口皮肤管理仪器，Zero peel点震刀，即刻下单立减1000元，想要了解点此链接 caimei365.com/t/{s20} ，退订回T
            // 【采美365】尊敬的客户您好，我是采美网客服经理Mary，原我司销售经理张顺心因个人原因已离职，后续产品咨询和服务将由我来接替，有任何需求联系[微信/企微]15338851365
            String  content="尊敬的客户您好，我是采美网客服经理Mary，原我司销售经理张顺心因个人原因已离职，后续产品咨询和服务将由我来接替，有任何需求联系[微信/企微]15338851365";
//            String content = "【西班牙ROSS】亲，ROSS抖音挑战赛开始啦，发布ROSS仪器视频赢网红探店和千元导胶！戳此 caimei365.com/t/" + link + " 查看视频发布教程，退订回T;";
            getSendSms(1,23, p.getMobile(), content);
            log.info("userId-------------------->" + p.getUserId() + "发送手机号------------------------>" + p.getMobile() + "短信-------------->" + content);
        });
    }

    @Override
    public void getOldUserTow() {
//        List<PushMessageUserVo> pushUsers = new ArrayList<>();
        List<PushMessageUserVo> pushUsers = messagePushMapper.findOldUserTow();
        // https://www.caimei365.com/product-7729.html水光仪
        String url = "https://www.caimei365.com/product-7729.html";
        List<String> list = new ArrayList<String>();
        list.add("15017915300");
        list.add("18670511721");
        list.add("17798247695");
//        list.add("15827317748");
//        list.add("18476937515");
//        list.add("18605868824");
//        list.add("18670511721");
//        list.add("17798247695");
        list.forEach(l -> {
            PushMessageUserVo pushMessageUserVo = new PushMessageUserVo();
            pushMessageUserVo.setMobile(l);
            pushUsers.add(pushMessageUserVo);
        });
        pushUsers.forEach(p -> {
            p.setJumpLink(url);
            String link = getLink();
            p.setShortLink(link);
            p.setMarkId(23);
            messagePushMapper.insertNewLink(p);
//水光仪首付2万！国货之光|颜层水光仪|有证一一采美超级会员优享，点击 caimei365.com/t/{s20} 立刻了解详情，退订回T
//            String content = "欢庆元旦，采美平台直接让利，下单立减2000元，限时优惠券发放中，速领！戳https://www.caimei365.com/t/" + link + "，退订回T";
            String content = "水光仪首付2万！国货之光|颜层水光仪|有证一一采美超级会员优享，点击 caimei365.com/t/" + link + " 立刻了解详情，退订回T";
            getSendSms(3,23, p.getMobile(), content);
            log.info("userId-------------------->" + p.getUserId() + "发送手机号------------------------>" + p.getMobile() + "短信-------------->" + content);
        });
    }

    public String getLink() {
        String shortLink = CodeUtil.generateShortLink(9);
        Integer id = messagePushMapper.findIdByShortLink(shortLink);
        if (id != null && id > 0) {
            getLink();
        }
        return shortLink;
    }
}
