package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.OrderCommonService;
import com.caimei365.order.components.OssService;
import com.caimei365.order.components.WeChatService;
import com.caimei365.order.mapper.*;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.*;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.service.ShipService;
import com.caimei365.order.utils.ImageUtil;
import com.caimei365.order.utils.MathUtil;
import com.caimei365.order.utils.OSSUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import static com.alibaba.fastjson.JSON.parseArray;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/8/5
 */
@Slf4j
@Service
public class ShipServiceImpl implements ShipService {
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private ShipMapper shipMapper;
    @Resource
    private AddressMapper addressMapper;
    @Resource
    private OrderCommonService orderCommonService;
    @Resource
    private OrderCommonMapper orderCommonMapper;
    @Resource
    private OrderClubMapper orderClubMapper;
    @Resource
    private RemoteCallService remoteCallService;
    @Resource
    private OssService ossService;
    @Resource
    private OSSUtils ossUtils;
    @Resource
    private CmReportingClubMapper cmReportingClubMapper;
    @Resource
    private CmReportingMemberMapper cmReportingMemberMapper;
    @Resource
    private WeChatService weChatService;

    /**
     * 供应商订单列表
     *
     * @param shopId        供应商Id
     * @param sendOutStatus 发货状态
     * @param payStatus     结算状态
     * @param shopOrderNo   订单编号
     * @param receiver      买家名称(收货人)
     * @param pageNum       页码
     * @param pageSize      每页数量
     */
    @Override
    public ResponseJson<PageInfo<ShopOrderVo>> getShopOrderList(Integer shopId, Integer sendOutStatus, Integer payStatus, String shopOrderNo, String receiver, Integer refundStatus, Integer receiptStatus, Integer organizeId, int pageNum, int pageSize) {
        // 分页请求数据
        PageHelper.startPage(pageNum, pageSize);
        List<ShopOrderVo> shopOrderList = shipMapper.getShopOrderList(shopId, sendOutStatus, payStatus, shopOrderNo, receiver, refundStatus, receiptStatus, organizeId);
        shopOrderList.forEach(shopOrder -> {
            if (null != shopOrder) {
                setShopOrderInfo(shopOrder);
            }
            // 联合丽格 [待付款]或者线下支付[部分付款] 的子订单 状态
            if ((4 != shopOrder.getStatus() && 5 != shopOrder.getStatus()) && ((1 == shopOrder.getReceiptStatus() && (2 == shopOrder.getOnlinePay() || 0 == shopOrder.getOnlinePay())) || (2 == shopOrder.getReceiptStatus() && 2 == shopOrder.getOnlinePay()))) {
                shopOrder.setCheckFlag(2);
            } else {
                shopOrder.setCheckFlag(1);
            }
        });
        PageInfo<ShopOrderVo> pageInfo = new PageInfo(shopOrderList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * 供应商结算列表
     *
     * @param shopId      供应商Id
     * @param payStatus   结算状态
     * @param shopOrderNo 订单编号
     * @param name        买家名称
     * @param pageNum     页码
     * @param pageSize    每页数量
     */
    @Override
    public ResponseJson<PageInfo<SettlementVo>> getSettlementList(Integer shopId, Integer payStatus, String shopOrderNo, String name, String beginTime, String endTime, Integer refundStatus, int pageNum, int pageSize) {
        // 分页请求数据
        PageHelper.startPage(pageNum, pageSize);
        List<SettlementVo> settlementList = shipMapper.getSettlementList(shopId, payStatus, shopOrderNo, name, beginTime, endTime, refundStatus);
        settlementList.forEach(settlement -> {
            if (null != settlement) {
                List<OrderProductVo> orderProductList = orderCommonMapper.getShopOrderProduct(settlement.getShopOrderId());
                settlement.setOrderProductList(orderProductList);
            }
            // 退款金额
            AtomicDouble refund = new AtomicDouble(0);
            // 付款手续费
            double procedureFee = 0d;
            // 付款记录
            List<DiscernReceiptVo> shopOrderRelation = shipMapper.getShopOrderRelation(settlement.getShopOrderId());
            for (DiscernReceiptVo discernReceipt : shopOrderRelation) {
                // 本次支付金额,单位/元
                double splitAmount = discernReceipt.getAssociateAmount();
                double payAmount;
                if (1 == discernReceipt.getPayWay()) {
                    if (12 == discernReceipt.getPayType()) {
                        // 网银支付
                        payAmount = 10.00;
                    } else if (17 == discernReceipt.getPayType()) {
                        //b2c网银
                        //b2c 0.2%
                        payAmount = MathUtil.mul(splitAmount, 0.002, 2).doubleValue();
                        //b2c最低手续费0.1
                        if (payAmount < 0.1) {
                            payAmount = 0.1;
                        }
                    } else if (8 == discernReceipt.getPayType() || 13 == discernReceipt.getPayType() || 15 == discernReceipt.getPayType()) {
                        //微信0.65%手续费
                        //微信0.65%
                        payAmount = MathUtil.mul(splitAmount, 0.0065, 2).doubleValue();
                    } else if (29 == discernReceipt.getPayType()) {
                        //快捷支付 借记卡支付29，

                        payAmount = MathUtil.mul(splitAmount, 0.003, 2).doubleValue();
                        if (payAmount < 0.1) {
                            payAmount = 0.1;
                        }
                    } else if (30 == discernReceipt.getPayType()) {
                        //快捷支付 贷记卡支付30
                        payAmount = MathUtil.mul(splitAmount, 0.006, 2).doubleValue();
                        if (payAmount < 0.1) {
                            payAmount = 0.1;
                        }
                    } else {
                        //手续费 其他0.25%
                        payAmount = MathUtil.mul(splitAmount, 0.0025, 2).doubleValue();
                    }
                    if (MathUtil.compare(payAmount, 0.01) <= 0) {
                        payAmount = 0.01;
                    }
                    procedureFee = MathUtil.add(procedureFee, payAmount).doubleValue();
                }
            }
            // 退款记录
            List<CmReturnedPurchase> refundPurchase = shipMapper.getRefundPurchase(settlement.getShopOrderId());
            for (CmReturnedPurchase refundPur : refundPurchase) {
                refund.set(MathUtil.add(refund.get(), refundPur.getReturnedPurchaseFee()).doubleValue());
            }
            // 支付手续费
            settlement.setPaymentCommission(procedureFee);
            // 分帐记录
            Double splitAccount = shipMapper.getSplitAccount(settlement.getShopOrderId());
            // 分帐手续费
            settlement.setProrateCommission(MathUtil.mul(splitAccount, 0.001).doubleValue());
            // 总退款金额
            settlement.setRefundAmount(refund.get());
        });
        PageInfo<SettlementVo> pageInfo = new PageInfo(settlementList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * 导出excel
     *
     * @param shopId
     * @param payStatus
     * @param shopOrderNo
     * @param name
     * @param beginTime
     * @param endTime
     * @param refundStatus
     * @return
     */
    @Override
    public List<SettlementDto> exportExcel(Integer shopId, Integer payStatus, String shopOrderNo, String name, String beginTime, String endTime, Integer refundStatus) {
        List<SettlementVo> settlementList = shipMapper.getSettlementList(shopId, payStatus, shopOrderNo, name, beginTime, endTime, refundStatus);
        List<SettlementDto> settlementDtoList = new ArrayList<>();
        settlementList.forEach(settlement -> {
            if (null != settlement) {
                List<OrderProductVo> orderProductList = orderCommonMapper.getShopOrderProduct(settlement.getShopOrderId());
                settlement.setOrderProductList(orderProductList);
            }
            // 退款金额
            AtomicDouble refund = new AtomicDouble(0);
            // 付款手续费
            double procedureFee = 0d;
            // 付款记录
            List<DiscernReceiptVo> shopOrderRelation = shipMapper.getShopOrderRelation(settlement.getShopOrderId());
            for (DiscernReceiptVo discernReceipt:shopOrderRelation) {
                // 本次支付金额,单位/元
                double splitAmount = discernReceipt.getAssociateAmount();
                double payAmount;
                if (12 == discernReceipt.getPayType()) {
                    // 网银支付
                    payAmount = 10.00;
                } else if (17 == discernReceipt.getPayType()) {
                    //b2c网银
                    //b2c 0.2%
                    payAmount = MathUtil.mul(splitAmount, 0.002, 2).doubleValue();
                    //b2c最低手续费0.1
                    if (payAmount < 0.1) {
                        payAmount = 0.1;
                    }
                } else if (8 == discernReceipt.getPayType() || 13 == discernReceipt.getPayType() || 15 == discernReceipt.getPayType()) {
                    //微信0.65%手续费
                    //微信0.65%
                    payAmount = MathUtil.mul(splitAmount, 0.0065, 2).doubleValue();
                } else if (29 == discernReceipt.getPayType()) {
                    //快捷支付 借记卡支付29，

                    payAmount = MathUtil.mul(splitAmount, 0.003, 2).doubleValue();
                    if (payAmount < 0.1) {
                        payAmount = 0.1;
                    }
                }else if (30 == discernReceipt.getPayType()) {
                    //快捷支付 贷记卡支付30
                    payAmount = MathUtil.mul(splitAmount, 0.006, 2).doubleValue();
                    if (payAmount < 0.1) {
                        payAmount = 0.1;
                    }
                } else {
                    //手续费 其他0.25%
                    payAmount = MathUtil.mul(splitAmount, 0.0025, 2).doubleValue();
                }
                if (MathUtil.compare(payAmount, 0.01) <= 0) {
                    payAmount = 0.01;
                }
                procedureFee = MathUtil.add(procedureFee, payAmount).doubleValue();
            }
            // 退款记录
            List<CmReturnedPurchase> refundPurchase = shipMapper.getRefundPurchase(settlement.getShopOrderId());
            for (CmReturnedPurchase refundPur : refundPurchase) {
                refund.set(MathUtil.add(refund.get(), refundPur.getReturnedPurchaseFee()).doubleValue());
            }
            // 支付手续费
            settlement.setPaymentCommission(procedureFee);
            // 分帐记录
            Double splitAccount = shipMapper.getSplitAccount(settlement.getShopOrderId());
            // 分帐手续费
            settlement.setProrateCommission(MathUtil.mul(splitAccount, 0.001).doubleValue());
            // 总退款金额
            settlement.setRefundAmount(refund.get());
            // 设置返回数据
            SettlementDto lementDto = new SettlementDto();
            lementDto.setShopOrderNo(settlement.getShopOrderNo());
            lementDto.setOrderTime(settlement.getOrderTime());
            lementDto.setName(settlement.getName());
            if(settlement.getOrderProductList().size() > 0) {
                String names = "";
                for (OrderProductVo sett : settlement.getOrderProductList()) {
                    names = names + sett.getName();
                }
                lementDto.setProductInfo(names);
            }
            lementDto.setTotalAmount(settlement.getTotalAmount());
            if (1 == settlement.getRefundStatus()) {
                lementDto.setRefundStatus("无退款");
            } else {
                lementDto.setRefundStatus("有退款");
            }
            lementDto.setRefundAmount(settlement.getRefundAmount());
            lementDto.setPaymentCommission(settlement.getPaymentCommission());
            lementDto.setProrateCommission(settlement.getProrateCommission());
            if (1 == settlement.getSettlestatus()) {
                lementDto.setPayStatus("未结算");
            } else if (2 == settlement.getSettlestatus()){
                lementDto.setPayStatus("部分结算");
            } else {
                lementDto.setPayStatus("已结算");
            }
            lementDto.setShouldPayShopAmount(settlement.getShouldPayShopAmount());
            lementDto.setSettleAmount(settlement.getSettleAmount());
            lementDto.setUnsettledAmount(MathUtil.sub(settlement.getShouldPayShopAmount(), settlement.getSettleAmount()).doubleValue());
            settlementDtoList.add(lementDto);
        });
        return settlementDtoList;
    }

    /**
     * 供应商子订单详情
     *
     * @param shopOrderId 子订单Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getShopOrderDetail(Integer shopOrderId) {
        ShopOrderVo shopOrder = shipMapper.getShopOrder(shopOrderId);
        if (null == shopOrder) {
            return ResponseJson.error("子订单异常,shopOrderId:" + shopOrderId, null);
        }
        setShopOrderInfo(shopOrder);
        // 联合丽格 [待付款]或者线下支付[部分付款] 的子订单 状态
        if ((4 != shopOrder.getStatus() && 5 != shopOrder.getStatus()) && ((1 == shopOrder.getReceiptStatus() && (2 == shopOrder.getOnlinePay() || 0 == shopOrder.getOnlinePay())) || (2 == shopOrder.getReceiptStatus() && 2 == shopOrder.getOnlinePay()))) {
            shopOrder.setCheckFlag(2);
        } else {
            shopOrder.setCheckFlag(1);
        }
        Map<String, Object> map = new HashMap<>(2);
        map.put("shopOrder", shopOrder);
        // 结算记录
        List<PayShopRecordPo> payShopRecord = shipMapper.getPayShopRecordList(shopOrderId);
        map.put("payShopRecord", payShopRecord);
        return ResponseJson.success(map);
    }

    /**
     * 供应商售货清单
     *
     * @param shopOrderId 子订单Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getSalesList(Integer shopOrderId) {
        ShopOrderVo shopOrder = shipMapper.getShopOrder(shopOrderId);
        // 供应商名称
        String shopName = baseMapper.getShopNameById(shopOrder.getShopId());
        shopOrder.setShopName(shopName);
        // 收货地址
        OrderUserinfoVo userInfo = addressMapper.getOrderUserinfo(shopOrder.getOrderId());
        shopOrder.setUserInfo(userInfo);
        // 订单商品
        List<OrderProductVo> orderProductList = orderCommonMapper.getShopOrderProduct(shopOrder.getShopOrderId());
        shopOrder.setOrderProductList(orderProductList);

        Map<String, Object> map = new HashMap<>(2);
        String currentTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());
        map.put("shopOrder", shopOrder);
        map.put("currentTime", currentTime);
        return ResponseJson.success(map);
    }

    /**
     * 发货页面子订单数据
     *
     * @param shopOrderId 子订单Id
     */
    @Override
    public ResponseJson<ShopOrderVo> getShopOrderInfo(Integer shopOrderId) {
        ShopOrderVo shopOrder = shipMapper.getShopOrder(shopOrderId);
        if (null != shopOrder) {
            setShopOrderInfo(shopOrder);
        }
        // 过滤已发货商品
        shopOrder.getOrderProductList().removeIf(p -> (p.getNotOutStore() - p.getActualCancelNum()) < 1);
        return ResponseJson.success(shopOrder);
    }


    private void setShopOrderInfo(ShopOrderVo shopOrder) {
        // 设置订单状态
        orderCommonService.setShopOrderStatus(shopOrder);
        // 收货地址
        OrderUserinfoVo userInfo = addressMapper.getOrderUserinfo(shopOrder.getOrderId());
        shopOrder.setUserInfo(userInfo);
        // （收款买家）收款状态：1待收款、2部分收款、3已收款
        Integer receiptStatus = shipMapper.getShopOrderReceiptStatus(shopOrder.getShopOrderId());
        //（收款买家）收款状态：1待收款、2部分收款、3已收款
        shopOrder.setReceiptStatus(receiptStatus);
        // 供应商名称
        String shopName = baseMapper.getShopNameById(shopOrder.getShopId());
        shopOrder.setShopName(shopName);

        if (null != userInfo) {
            String address = userInfo.getProvince() + userInfo.getCity() + userInfo.getTown() + userInfo.getAddress();
            userInfo.setAddress(address);
        }
        List<OrderProductVo> orderProductList = orderCommonMapper.getShopOrderProduct(shopOrder.getShopOrderId());
        orderProductList.forEach(orderProduct -> {
            if (null != orderProduct) {
                orderProduct.setImage(ImageUtil.getImageUrl("product", orderProduct.getImage(), domain));
                //已发货数量
                orderProduct.setShipmentsNum(orderProduct.getNum() + orderProduct.getPresentNum() - orderProduct.getNotOutStore());
                //已退货数量
                Integer returnedNum = orderCommonMapper.countReturnedNum(orderProduct.getShopOrderId(), orderProduct.getProductId());
                returnedNum = null != returnedNum ? returnedNum : 0;
                orderProduct.setReturnedNum(returnedNum);
                //已取消发货数量
                Integer actualCancelNum = orderCommonMapper.countActualCancelNum(orderProduct.getShopOrderId(), orderProduct.getProductId());
                actualCancelNum = null != actualCancelNum ? actualCancelNum : 0;
                orderProduct.setActualCancelNum(actualCancelNum);
            }
        });
        // 发票信息
        InvoicePo orderInvoice = orderClubMapper.getOrderInvoice(shopOrder.getOrderId());
        shopOrder.setInvoice(orderInvoice);
        // 线下支付凭证
        List<VoucherPo> voucher = orderClubMapper.getVoucher(shopOrder.getShopOrderId());
        // 支付凭证图片
        if (null != voucher) {
            List<Integer> ids = voucher.stream().map(VoucherPo::getId).collect(Collectors.toList());
            List<VoucherImageDto> voucherImage = orderClubMapper.getVoucherImage(ids);
            voucher.forEach(v -> {
                List<VoucherImageDto> imageList = voucherImage.stream().filter(m -> m.getVoucherId().equals(v.getId())).collect(Collectors.toList());
                v.setImgs(imageList);
            });
        }
        // 子订单信息
        orderCommonService.setShopOrderInfo(shopOrder);
        // 设置子订单付款金额
        orderCommonService.getDiscernReceiptAndSetShoporder(shopOrder);
        shopOrder.setVoucher(voucher);
        shopOrder.setOrderProductList(orderProductList);
        shopOrder.setShopLogo(ImageUtil.getImageUrl("shopLogo", shopOrder.getShopLogo(), domain));
        shopOrder.setOrderProductList(orderProductList);
    }

    /**
     * 物流公司
     *
     * @param value 快递公司代码
     */
    @Override
    public ResponseJson<List<CompanyVo>> getLogisticsCompany(String value) {
        List<CompanyVo> companyList = shipMapper.getLogisticsCompany(value);
        return ResponseJson.success(companyList);
    }

    /**
     * 发货
     *
     * @param logisticsDto {
     *                     shopOrderId: 子订单Id,
     *                     note: 备注信息,
     *                     image: 图片备注以##隔开,
     *                     logistics：[  // 物流信息
     *                     {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'},
     *                     {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'}
     *                     ],
     *                     products：[   // 商品信息
     *                     {orderProductId:订单商品id,num:此次发货数量},
     *                     {orderProductId:订单商品id,num:此次发货数量}
     *                     ],
     *                     }
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseJson<Integer> addDelivery(LogisticsDto logisticsDto) {
        JSONArray logistics;
        JSONArray products;
        try {
            logistics = parseArray(logisticsDto.getLogistics());
            products = parseArray(logisticsDto.getProducts());
        } catch (Exception e) {
            log.info("发货参数:" + logisticsDto.toString());
            log.error("【发货】>>>发货参数解析异常try-catch:", e);
            return ResponseJson.error("发货参数解析异常！", null);
        }

        String image = logisticsDto.getImage();
        String note = logisticsDto.getNote();
        ShopOrderVo shopOrder = shipMapper.getShopOrder(logisticsDto.getShopOrderId());
        if (null == shopOrder) {
            return ResponseJson.error("订单异常,shopOrderId:" + logisticsDto.getShopOrderId(), null);
        }
        // 本次发货数量
        int shipmentsNum = 0;
        Date date = new Date();
        // 统计物流批次
        int batchCount = shipMapper.countLogisticsBatch(shopOrder.getOrderId());
        // 物流批次
        LogisticsBatchPo logisticsBatch = new LogisticsBatchPo();
        if (batchCount > 0) {
            logisticsBatch.setOutStoreTimes(batchCount + 1);
        } else {
            logisticsBatch.setOutStoreTimes(0);
        }
        logisticsBatch.setShopOrderId(shopOrder.getShopOrderId());
        logisticsBatch.setOrderId(shopOrder.getOrderId());
        logisticsBatch.setStatus("0");
        logisticsBatch.setMailer("1");
        logisticsBatch.setShopId(shopOrder.getShopId());
        logisticsBatch.setUpdateDate(date);
        logisticsBatch.setDeliveryTime(date);
        logisticsBatch.setRemarkImage(image);
        logisticsBatch.setRemark(note);
        // 保存物流批次
        shipMapper.insertLogisticsBatch(logisticsBatch);
        log.info("【供应商发货】>>>>>>>>>>>>>>>>>>>>>>>>>>新增物流批次(insert[cm_logistics_batch])shopOrderId：" + shopOrder.getShopOrderId() + "，logisticsBatchId：" + logisticsBatch.getId());

        /* products：[   // 发货商品信息
         *      {orderProductId:订单商品id,num:此次发货数量},
         *      {orderProductId:订单商品id,num:此次发货数量}]
         */
        for (Object productObj : products) {
            JSONObject productInfo = (JSONObject) productObj;
            Integer orderProductId = (Integer) productInfo.get("orderProductId");
            Integer num = (Integer) productInfo.get("num");
            if (null == num || 0 == num) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ResponseJson.error("发货数量异常！", null);
            }
            OrderProductVo orderProduct = shipMapper.getOrderProductById(orderProductId);
            if (null == orderProduct || orderProduct.getNotOutStore() < num) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ResponseJson.error("订单商品异常,orderProductId:" + orderProductId, null);
            }
            shipmentsNum += num;
            LogisticsRecordPo logisticsRecord = new LogisticsRecordPo();
            logisticsRecord.setLogisticsBatchId(logisticsBatch.getId());
            logisticsRecord.setShopOrderId(orderProduct.getShopOrderId());
            logisticsRecord.setOrderId(orderProduct.getOrderId());
            logisticsRecord.setOrderProductId(orderProductId);
            logisticsRecord.setBuyNum((orderProduct.getNum() + orderProduct.getPresentNum()));
            logisticsRecord.setNum(num);
            logisticsRecord.setProductId(orderProduct.getProductId());
            logisticsRecord.setProductName(orderProduct.getName());
            logisticsRecord.setImage(orderProduct.getImage());
            // 保存 发货物流记录
            shipMapper.insertLogisticsRecord(logisticsRecord);
            log.info("【供应商发货】>>>>>>>>>>>>>>>>>>>>>>>>>>新增物流记录(insert[cm_logistics_record])shopOrderId：" + shopOrder.getShopOrderId() + "，logisticsRecordId：" + logisticsRecord.getId());

            // 保存订单商品未出库数量
            int notOutStore = orderProduct.getNotOutStore() - num;
            shipMapper.updateNotOutStore(orderProductId, notOutStore);
        }

        /* logistics：[  // 物流跟踪信息
         *   {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'},
         *   {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'}]
         */
        for (Object logisticObj : logistics) {
            JSONObject logistic = (JSONObject) logisticObj;
            String number = (String) logistic.get("number");
            String logisticsCompanyName = (String) logistic.get("logisticsCompanyName");
            String logisticsCompanyCode = (String) logistic.get("logisticsCompanyCode");
            if (StringUtils.isBlank(number) || StringUtils.isBlank(logisticsCompanyName) || StringUtils.isBlank(logisticsCompanyCode)) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ResponseJson.error("物流信息参数异常！", null);
            }
            LogisticsInformationPo logisticsInformation = new LogisticsInformationPo();
            logisticsInformation.setLogisticsBatchId(logisticsBatch.getId());
            logisticsInformation.setType("1");
            logisticsInformation.setShopOrderId(shopOrder.getShopOrderId());
            logisticsInformation.setOrderId(shopOrder.getOrderId());
            logisticsInformation.setNu(number);
            logisticsInformation.setLogisticsCompanyName(logisticsCompanyName);
            logisticsInformation.setLogisticsCompanyCode(logisticsCompanyCode);
            logisticsInformation.setShopId(shopOrder.getShopId());
            logisticsInformation.setUpdateDate(date);
            // 保存 物流跟踪信息
            shipMapper.insertLogisticsInformation(logisticsInformation);
        }

        /*
         * 修改子订单发货状态
         */
        if (null == shopOrder.getOutStoreNum()) {
            shopOrder.setOutStoreNum(shipmentsNum);
        } else {
            shopOrder.setOutStoreNum(shopOrder.getOutStoreNum() + shipmentsNum);
        }
        shopOrder.setOutStoreTimes(logisticsBatch.getOutStoreTimes());
        shopOrder.setPresentNum(shopOrder.getPresentNum() == null ? 0 : shopOrder.getPresentNum());
        int productNum = shopOrder.getPresentNum() + shopOrder.getItemCount();
        if (productNum > shopOrder.getOutStoreNum()) {
            shopOrder.setSendOutStatus(2);
        } else if (productNum == shopOrder.getOutStoreNum()) {
            shopOrder.setSendOutStatus(3);
        } else {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("发货数量异常！", null);
        }
        shopOrder.setSplitFlag(0);
        shipMapper.updateShopOrderShip(shopOrder.getShopOrderId(), shopOrder.getOutStoreNum(), shopOrder.getOutStoreTimes(), shopOrder.getSendOutStatus());
        /*
         * 修改主订单发货状态
         */
        setMainOrderStatus(shopOrder);
        /**
         * 发货后消息推送
         */
//        if (1 == orderClubMapper.getReceiptStatus(logisticsDto.getShopOrderId())) {
//            if (0 == orderClubMapper.getsecondHandOrderFlag(logisticsDto.getShopOrderId()) && 0 == orderClubMapper.getrebateFlag(logisticsDto.getShopOrderId())) {
//                SubmitServiceImpl s=new SubmitServiceImpl();
//                try {
//                    LogisticsInformationPo logisticsInformation = new LogisticsInformationPo();
//                    String accessToken = weChatService.getAccessToken();
//                    List<String> openidList = shipMapper.getOpenidListByPermission(shopOrder.getUserId(),s.unionId);
//                    openidList.removeIf(Objects::isNull);
//                    String company = logisticsInformation.getLogisticsCompanyName();
//                    String companyno = logisticsInformation.getNu();
//                    String associateTitle = "尊敬的采美客户，您购买的订单已经发货啦~";
//                    String time = logisticsBatch.getDeliveryTime().toString();
//                    String orderno = shopOrder.getOrderNo();
//                    String remarkText = "收到货后请记得确认收货哟~";
//                    // 跳转到【跳转到小程序订单详情页面】
//                    String pagePath = "pages/user/order/order-details?orderId=" + shipMapper.getorderID(logisticsDto.getShopOrderId());
//                    for (String openid : openidList) {
//                        // sendTemplateMsg(openid, 标题, 金额, 收款日期, 备注, 跳转链接)
//                        weChatService.sendTemplateMsgy(accessToken, openid, associateTitle, orderno, time, company, companyno, remarkText, pagePath);
//                    }
//                } catch (Exception e) {
//                    log.error("【订单发货通知】获取微信公众号access_token异常！", e);
//                }
//            }
//        }
                return ResponseJson.success(logisticsBatch.getId());

    }
    /**
     * 修改主订单发货状态
     * @param shopOrder 子订单
     */
    private void setMainOrderStatus(ShopOrderVo shopOrder) {
        OrderVo order = orderCommonMapper.getOrderByOrderId(shopOrder.getOrderId());
        // 主订单发货状态
        Integer sendOutStatus = order.getSendOutStatus();
        // 发货状态：1待发货、2部分发货、3已发货
        List<Integer> allShopSendOutStatus = shipMapper.getAllSendOutStatus(shopOrder.getOrderId());
        String statusSuffix = "";
        if (allShopSendOutStatus.stream().allMatch(s -> 3 == s)) {
            // 所有的子订单都已经完成发货
            statusSuffix = "3";
            order.setSendOutStatus(3);
        } else {
            statusSuffix = "2";
            order.setSendOutStatus(2);
        }
        if (order.getStatus() >= 10) {
            String statusTmp = order.getStatus().toString().charAt(0) + statusSuffix;
            order.setStatus(Integer.valueOf(statusTmp));
        }
        // 修改主订单发货状态
        shipMapper.updateOrderShip(order.getOrderId(), order.getSendOutStatus(), order.getStatus());
        //发货短信推送（取消推送）
        /*String bindMobile = baseMapper.getBindMobileByUserId(order.getUserId());
        if (StringUtils.isNotBlank(bindMobile) && 0 == order.getRebateFlag()) {
            boolean sendSms = false;
            if (1 == sendOutStatus && 2 == order.getSendOutStatus()){
                String shortLink = remoteCallService.getShortLink(8, 7, domain + "/user/order/detail.html?orderId=" + order.getOrderId());
                String content = "您的订单（订单编号：" + order.getOrderNo() + "）已部分发货。您可关注采美公众号或者访问采美微信小程序和网站查看订单。平台公众号：微信搜索“采美365网”; " +
                        "微信小程序：微信搜索“采美采购商城”；网址：www.caimei365.com/t/" + shortLink;
                sendSms = remoteCallService.getSendSms(7, bindMobile, content);
            } else if (3 == order.getSendOutStatus()) {
                String shortLink = remoteCallService.getShortLink(8, 8, domain + "/user/order/detail.html?orderId=" + order.getOrderId());
                String content = "您的订单（订单编号：" + order.getOrderNo() + "）已发货完毕。您可关注采美公众号或者访问采美微信小程序和网站查看订单。平台公众号：微信搜索“采美365网”; " +
                        "微信小程序：微信搜索“采美采购商城”；网址：www.caimei365.com/t/" + shortLink;
                sendSms = remoteCallService.getSendSms(8, bindMobile, content);
            }
            if (!sendSms) {
                log.info("发货短信推送失败，orderId>>>>" + order.getOrderId());
            }
        }*/
    }

    /**
     * 供应商发货记录
     *
     * @param shopOrderId 子订单Id
     * @param pageNum     页码
     * @param pageSize    每页数量
     */
    @Override
    public ResponseJson<Map<String, Object>> getDeliveryRecord(Integer shopOrderId, int pageNum, int pageSize) {
        Integer orderId = shipMapper.getOrderIdByShopOrder(shopOrderId);
        if (null == orderId) {
            return ResponseJson.error("订单异常,shopOrderId:" + shopOrderId, null);
        }
        OrderUserinfoVo userInfo = addressMapper.getOrderUserinfo(orderId);
        // 分页请求数据
        PageHelper.startPage(pageNum, pageSize);
        // 获取物流批次列表
        List<LogisticsBatchVo> batchList = orderCommonMapper.getLogisticsBatchList(orderId);
        if (null != batchList && batchList.size() > 0) {
            for (LogisticsBatchVo batchVo : batchList) {
                batchVo.setShopOrderId(shopOrderId);
                if (StringUtils.isNotBlank(batchVo.getRemarkImage())) {
                    String[] remarkImages = batchVo.getRemarkImage().split("##");
                    batchVo.setRemarkImages(remarkImages);
                }
                // 获取发货物流记录
                List<LogisticsRecordVo> recordList = orderCommonMapper.getLogisticsRecord(shopOrderId, batchVo.getId());
                if (null != recordList && recordList.size() > 0) {
                    for (LogisticsRecordVo recordVo : recordList) {
                        // 设置 已发，已退，已取消发货数量
                        setLogisticsRecordNum(recordVo);
                    }
                }
                batchVo.setLogisticsRecordList(recordList);
                // 获取物流信息
                List<LogisticsInformationVo> logisticsInfoList = orderCommonMapper.getLogisticsInfoList(batchVo.getId());
                logisticsInfoList.forEach(logisticsInfo -> {
                    List<LogisticsRouterVo> routers = JSONArray.parseArray(logisticsInfo.getInfo(), LogisticsRouterVo.class);
                    if (!CollectionUtils.isEmpty(routers)) {
                        routers.forEach(router -> {
                            if (org.apache.commons.lang3.StringUtils.isEmpty(router.getDesc())) {
                                router.setDesc(router.getContext());
                            }
                        });
                        logisticsInfo.setRouterList(routers);
                    }
                });
                batchVo.setLogisticsInformationList(logisticsInfoList);
            }
            batchList.removeIf(logisticsBatch -> null == logisticsBatch.getLogisticsRecordList() || 0 == logisticsBatch.getLogisticsRecordList().size());
        }
        PageInfo<ShopOrderVo> pageInfo = new PageInfo(batchList);
        Map<String, Object> map = new HashMap<>(2);
        map.put("userInfo", userInfo);
        map.put("logisticsBatchPage", pageInfo);
        return ResponseJson.success(map);
    }

    /**
     *  设置 已发，已退，已取消发货数量
     * @param recordVo 发货记录
     */
    private void setLogisticsRecordNum(LogisticsRecordVo recordVo) {
        OrderProductVo orderProduct = shipMapper.getOrderProductById(recordVo.getOrderProductId());
        recordVo.setBuyNum(orderProduct.getNum() + orderProduct.getPresentNum());
        recordVo.setImage(orderProduct.getImage());
        recordVo.setUnit(orderProduct.getProductUnit());
        // 已发货数量
        int shipmentsNum = orderProduct.getNum() + orderProduct.getPresentNum() - orderProduct.getNotOutStore();
        recordVo.setShipmentsNum(shipmentsNum);
        // 已退货数量
        Integer returnedNum = orderCommonMapper.countReturnedNum(orderProduct.getShopOrderId(), orderProduct.getProductId());
        returnedNum = null != returnedNum ? returnedNum : 0;
        recordVo.setReturnedNum(returnedNum);
        // 已取消发货数量
        Integer actualCancelNum = orderCommonMapper.countActualCancelNum(orderProduct.getShopOrderId(), orderProduct.getProductId());
        actualCancelNum = null != actualCancelNum ? actualCancelNum : 0;
        // 子订单退款状态
        Integer refundStatus = orderCommonMapper.getRefundStatus(orderProduct.getShopOrderId());
        // 有退款 未发货置为 0
        if (3 == refundStatus) {
            actualCancelNum = 0;
        }
        // 未发货数量
        recordVo.setNotShippedNum(recordVo.getBuyNum() - shipmentsNum + actualCancelNum);
    }

    /**
     * 供应商撤销发货
     *
     * @param logisticsBatchId 发货物流批次Id
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseJson<Void> cancelDelivery(Integer logisticsBatchId) {
        // 发货物流批次
        LogisticsBatchVo logisticsBatch = shipMapper.getLogisticsBatchById(logisticsBatchId);
        if (null == logisticsBatch) {
            return ResponseJson.error("物流异常,logisticsBatchID:" + logisticsBatchId, null);
        }
        // 获取发货物流记录
        List<LogisticsRecordVo> recordList = orderCommonMapper.getLogisticsRecord(logisticsBatch.getShopOrderId(), logisticsBatch.getId());
        // 本批次发货数量
        int num = 0;
        for (LogisticsRecordVo logisticsRecord : recordList) {
            num += logisticsRecord.getNum();
            // 订单商品修改
            OrderProductVo orderProduct = shipMapper.getOrderProductById(logisticsRecord.getOrderProductId());
            int notOutStore = orderProduct.getNotOutStore() + logisticsRecord.getNum();
            // 保存订单商品未出库数量
            shipMapper.updateNotOutStore(orderProduct.getOrderProductId(), notOutStore);
        }
        ShopOrderVo shopOrder = shipMapper.getShopOrder(logisticsBatch.getShopOrderId());
        if (null == shopOrder) {
            return ResponseJson.error("订单异常,shopOrderId:" + logisticsBatch.getShopOrderId(), null);
        }
        // 子订单修改
        int outStoreNum = shopOrder.getOutStoreNum() - num;
        shopOrder.setOutStoreNum(outStoreNum);
        int productNum = shopOrder.getPresentNum() + shopOrder.getItemCount();
        if (productNum > shopOrder.getOutStoreNum() && shopOrder.getOutStoreNum() > 0) {
            shopOrder.setSendOutStatus(2);
        } else if (productNum == shopOrder.getOutStoreNum()) {
            shopOrder.setSendOutStatus(3);
        } else if (shopOrder.getOutStoreNum() == 0) {
            shopOrder.setSendOutStatus(1);
        } else {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("发货数量异常", null);
        }
        // 修改子订单
        shipMapper.updateShopOrderShip(shopOrder.getShopOrderId(), shopOrder.getOutStoreNum(), shopOrder.getOutStoreTimes(), shopOrder.getSendOutStatus());
        // 修改主订单发货状态
        setMainOrderStatus(shopOrder);
        // 撤销发货
        shipMapper.deleteLogisticsBatchByBatchId(logisticsBatchId);
        shipMapper.deleteLogisticsRecordByBatchId(logisticsBatchId);
        shipMapper.deleteLogisticsInfoByBatchId(logisticsBatchId);
        return ResponseJson.success(null);
    }

    /**
     * 当前批次发货商品数据
     *
     * @param logisticsBatchId 发货物流批次Id
     */
    @Override
    public ResponseJson<List<LogisticsRecordVo>> getLogisticsRecord(Integer logisticsBatchId) {
        List<LogisticsRecordVo> recordList = shipMapper.getLogisticsRecord(logisticsBatchId);
        return ResponseJson.success(recordList);
    }

    /**
     * 发货记录-添加物流
     *
     * @param logisticsBatchDto {
     *                          logisticsBatchId: 发货批次表Id,
     *                          remark: 备注,
     *                          remarkImage: 图片备注,
     *                          logistics：[  // 物流信息
     *                              {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'},
     *                              {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'}
     *                          ]
     *                          }
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseJson<Void> addLogisticsInfo(LogisticsBatchDto logisticsBatchDto) {
        // 发货物流批次
        LogisticsBatchVo logisticsBatch = shipMapper.getLogisticsBatchById(logisticsBatchDto.getLogisticsBatchId());
        if (null == logisticsBatch) {
            return ResponseJson.error("物流批次异常,logisticsBatchID:" + logisticsBatchDto.getLogisticsBatchId(), null);
        }
        /* logistics：[  // 物流跟踪信息
         *   {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'},
         *   {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'}]
         */
        JSONArray logistics;
        try {
            logistics = parseArray(logisticsBatchDto.getLogistics());
        } catch (Exception e) {
            log.info("发货参数:" + logisticsBatchDto.toString());
            log.error("【添加物流】>>>发货参数解析异常try-catch:", e);
            return ResponseJson.error("添加物流参数解析异常！", null);
        }
        if (logistics.isEmpty()) {
            return ResponseJson.error("添加物流参数异常！", null);
        }
        Date date = new Date();
        for (Object logisticObj: logistics) {
            JSONObject logistic = (JSONObject) logisticObj;
            String number = (String) logistic.get("number");
            String logisticsCompanyName = (String) logistic.get("logisticsCompanyName");
            String logisticsCompanyCode = (String) logistic.get("logisticsCompanyCode");
            if (StringUtils.isBlank(number) || StringUtils.isBlank(logisticsCompanyName) || StringUtils.isBlank(logisticsCompanyCode)) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ResponseJson.error("物流信息参数异常！", null);
            }

            //推送信息中心-交易物流
            String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
            MessageCenter messageCenter=new MessageCenter();
            messageCenter.setShopID(logisticsBatch.getShopId());
            messageCenter.setClubID(null);
            messageCenter.setUserType(2);
            messageCenter.setMessageType(1);
            messageCenter.setOrderMessageType(5);
            messageCenter.setOrderID(logisticsBatch.getOrderId());
            messageCenter.setContent(null);
            messageCenter.setTime(curDateStr);
            orderClubMapper.addMessageCenter(messageCenter);

            LogisticsInformationPo logisticsInformation = new LogisticsInformationPo();
            logisticsInformation.setLogisticsBatchId(logisticsBatch.getId());
            logisticsInformation.setType("1");
            logisticsInformation.setShopOrderId(logisticsBatch.getShopOrderId());
            logisticsInformation.setOrderId(logisticsBatch.getOrderId());
            logisticsInformation.setNu(number);
            logisticsInformation.setLogisticsCompanyName(logisticsCompanyName);
            logisticsInformation.setLogisticsCompanyCode(logisticsCompanyCode);
            logisticsInformation.setShopId(logisticsBatch.getShopId());
            logisticsInformation.setUpdateDate(date);
            // 保存 物流跟踪信息
            shipMapper.insertLogisticsInformation(logisticsInformation);
        }
        logisticsBatch.setId(logisticsBatchDto.getLogisticsBatchId());
        logisticsBatch.setRemarkImage(logisticsBatchDto.getRemarkImage());
        logisticsBatch.setRemark(logisticsBatchDto.getRemark());
        // 更新物流批次
        shipMapper.updateLogisticsBatchRemark(logisticsBatch);
        return ResponseJson.success(null);
    }

    /**
     * 扫码添加物流
     * @param number      快递单号
     * @param mobile      收/寄件人的电话号码(顺丰必填)
     */
    @Override
    public ResponseJson<CompanyVo> scanGetLogisticsInfo(String number, String mobile) {
        String result = remoteCallService.getLogisticsByNumber(number, null, mobile);
        String errorMsg = "查询失败！";
        if (StringUtils.isNotEmpty(result)){
            JSONObject resultJson = JSONObject.parseObject(result);
            String comCode = resultJson.getString("com");
            if (StringUtils.isNotEmpty(comCode)) {
                List<CompanyVo> companyList = shipMapper.getLogisticsCompany(comCode);
                return ResponseJson.success(companyList.get(0));
            }
            // 查询失败，返回错误信息
            String msg = resultJson.getString("msg");
            if (StringUtils.isNotEmpty(msg)) {
                errorMsg = msg;
            }
        }
        return ResponseJson.error(errorMsg, null);
    }

    /**
     * 供应商查看物流
     *
     * @param shopOrderId      子订单Id
     * @param logisticsBatchId 发货物流批次Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getLogisticsInfo(Integer shopOrderId, Integer logisticsBatchId) {
        // 发货物流批次
        LogisticsBatchVo logisticsBatch = shipMapper.getLogisticsBatchById(logisticsBatchId);
        if (null == logisticsBatch) {
            return ResponseJson.error("物流异常,logisticsBatchID:" + logisticsBatchId, null);
        }
        OrderUserinfoVo userInfo = addressMapper.getOrderUserinfo(logisticsBatch.getOrderId());
        // 获取发货物流记录
        List<LogisticsRecordVo> recordList = orderCommonMapper.getLogisticsRecord(shopOrderId, logisticsBatch.getId());
        if (null != recordList && recordList.size() > 0) {
            for (LogisticsRecordVo recordVo : recordList) {
                // 设置 已发，已退，已取消发货数量
                setLogisticsRecordNum(recordVo);
            }
        }
        // 获取物流信息
        List<LogisticsInformationVo> logisticsInfoList = orderCommonMapper.getLogisticsInfoList(logisticsBatch.getId());
        logisticsInfoList.forEach(logisticsInfo -> {
            List<LogisticsRouterVo> routers = JSONArray.parseArray(logisticsInfo.getInfo(), LogisticsRouterVo.class);
            if (!CollectionUtils.isEmpty(routers)) {
                routers.forEach(router -> {
                    if (org.apache.commons.lang3.StringUtils.isEmpty(router.getDesc())) {
                        router.setDesc(router.getContext());
                    }
                });
                logisticsInfo.setRouterList(routers);
            }
        });
        logisticsBatch.setLogisticsRecordList(recordList);
        logisticsBatch.setLogisticsInformationList(logisticsInfoList);
        if (StringUtils.isNotBlank(logisticsBatch.getRemarkImage())) {
            String[] remarkImages = logisticsBatch.getRemarkImage().split("##");
            logisticsBatch.setRemarkImages(remarkImages);
        }
        Map<String, Object> map = new HashMap<>(2);
        map.put("userInfo", userInfo);
        map.put("logisticsBatch", logisticsBatch);
        return ResponseJson.success(map);
    }

    /**
     * 删除物流信息
     *
     * @param logisticsInfoId 发货物流信息Id
     */
    @Override
    public ResponseJson<Void> deleteLogisticsInfo(Integer logisticsInfoId) {
        shipMapper.deleteLogisticsInfoByInfoId(logisticsInfoId);
        return ResponseJson.success(null);
    }

    /**
     * 小程序分享发货-获取分享码
     *
     * @param shopOrderId 子订单Id
     */
    @Override
    public ResponseJson<String> getShipShareCode(Integer shopOrderId) {
        // 子订单所属供应商Id
        Integer dbShopId = shipMapper.getShopIdByShopOrderId(shopOrderId);
        if (null == dbShopId) {
            return ResponseJson.error("订单异常", null);
        }
        // 生成分享码
        String shareCode = RandomStringUtils.random(4, false, true);
        // 过期时间
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, 3);
        // 获取数据库已有分享码
        OrderShareCodeVo orderCodeVo = shipMapper.getShipShareCode(null, shopOrderId);
        if (orderCodeVo != null && orderCodeVo.getExpiredTime().compareTo(date) >= 0 ) {
            // 旧分享码还未过期，直接返回
            return ResponseJson.success(orderCodeVo.getShareCode());
        } else if (orderCodeVo != null) {
            // 如果旧分享码已经过期，删除分享码使用记录
            shipMapper.deleteShipShareCodeRecord(shopOrderId);
            // 更新数据库的分享码
            orderCodeVo.setShareCode(shareCode);
            orderCodeVo.setExpiredTime(calendar.getTime());
            orderCodeVo.setDelFlag(0);
            shipMapper.updateShipShareCode(orderCodeVo);
        } else {
            // 如果没有分享记录,新增数据库的分享码
            OrderShareCodeVo newShareCodeVo = new OrderShareCodeVo();
            newShareCodeVo.setOrderId(shopOrderId);
            newShareCodeVo.setShareCode(shareCode);
            newShareCodeVo.setExpiredTime(calendar.getTime());
            newShareCodeVo.setAddTime(date);
            newShareCodeVo.setDelFlag(0);
            shipMapper.insertShipShareCode(newShareCodeVo);
        }
        // 返回新生成的分享码
        return ResponseJson.success(shareCode);
    }

    /**
     * 小程序分享发货-分享权限控制
     *
     * @param source        来源：1:客服扫码,2:供应商运营人员查看,3:分享码查看
     * @param shopOrderId   子订单Id
     * @param shareCode     分享码
     * @param code          微信code
     * @param encryptedData 微信加密数据
     * @param iv            微信加密算法的初始向量
     * @param headers       HttpHeaders
     */
    @Override
    public ResponseJson<Map<String, Object>> checkShipShareCode(Integer source, Integer shopOrderId, String shareCode, String code, String encryptedData, String iv, HttpHeaders headers) {
        // 微信小程序授权登录（调用user服务，发送服务间调用POST请求）
        String loginData = remoteCallService.appletsAuthorization(code, encryptedData, iv, headers);
        if (StringUtils.isEmpty(loginData)){
            return ResponseJson.error("微信授权登录异常！", null);
        }
        // 解析登录后得到 userIdentity,  openId等
        JSONObject userInfo = JSONObject.parseObject(loginData);
        String openId = userInfo.getString("openId");
        if (StringUtils.isEmpty(openId)) {
            return ResponseJson.error("微信授权登录异常！", null);
        }
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("shopOrderId", shopOrderId);
        if (1 == source) {
            // 客服扫码进入
            String name = shipMapper.getConsignmentName(openId);
            if (StringUtils.isBlank(name)) {
                return ResponseJson.error("暂无权限查看", null);
            }
            log.info("小程序发货,客服登入>>>>>>" + name);
            resultMap.put("resultCode", 1);
            return ResponseJson.success("客服权限", resultMap);
        }
        // 用户身份: 1协销 2会员机构 3供应商 4普通机构
        Integer userIdentity = userInfo.getInteger("userIdentity");
        Integer loginShopId = userInfo.getInteger("shopId");
        // 供应商身份
        boolean shopFlag = (null != userIdentity && 3 == userIdentity);
        // 子订单所属供应商Id
        Integer dbShopId = shipMapper.getShopIdByShopOrderId(shopOrderId);
        if (shopFlag && loginShopId.equals(dbShopId)) {
            // 供应商进入
            resultMap.put("resultCode", 2);
            return ResponseJson.success("供应商运营人员权限", resultMap);
        }
        if (2 == source) {
            resultMap.put("resultCode", 2);
            Integer loginUserId = userInfo.getInteger("userId");
            String unionId = userInfo.getString("unionId");
            Integer shopUserId = shipMapper.getUserIdByShopId(dbShopId);
            resultMap.put("openid", openId);
            resultMap.put("unionId", unionId);
            resultMap.put("userId", shopUserId);
            resultMap.put("shopId", dbShopId);
            resultMap.put("loginUserId", loginUserId);
            resultMap.put("loginShopId", loginUserId);
            return ResponseJson.error(-1, "绑定供应商运营人员", resultMap);
        } else {
            // 分享码进入
            resultMap.put("resultCode", 3);
            // 获取采美供应商发货分享码使用记录
            Integer shareCodeRecordId = shipMapper.getShipShareCodeRecordId(openId, shopOrderId);
            if (null != shareCodeRecordId && shareCodeRecordId > 0) {
                return ResponseJson.success("分享码人员权限", resultMap);
            }
            if (StringUtils.isBlank(shareCode)) {
                return ResponseJson.error("请联系分享人获取分享码", null);
            }
            OrderShareCodeVo orderCodeVo = shipMapper.getShipShareCode(shareCode, shopOrderId);
            Date date = new Date();
            if (null == orderCodeVo) {
                return ResponseJson.error("分享码错误", null);
            } else if (orderCodeVo.getExpiredTime().compareTo(date) < 0) {
                return ResponseJson.error("分享码已失效", null);
            } else {
                // 分享码有效, 保存分享码使用记录
                OrderShareCodeRecordVo codeRecord = new OrderShareCodeRecordVo();
                codeRecord.setOpenId(openId);
                codeRecord.setAddTime(date);
                codeRecord.setOrderId(shopOrderId);
                codeRecord.setDelFlag(0);
                codeRecord.setShareCodeId(orderCodeVo.getId());
                shipMapper.insertShipShareCodeRecord(codeRecord);
                return ResponseJson.success("分享码人员权限", resultMap);
            }
        }
    }

    /**
     * 查看商品资质信息
     *
     * @param logisticsBatchId 发货物流批次Id
     */
    @Override
    public ResponseJson<List<QualificationVo>> getProductQualification(Integer logisticsBatchId) {
        // 商品资质信息
        List<QualificationVo> qualificationList = shipMapper.getProductQualification(logisticsBatchId);
        Iterator<QualificationVo> iterator = qualificationList.iterator();
        while (iterator.hasNext()) {
            QualificationVo qualification = iterator.next();
            // 商品资质图片
            List<String> images = shipMapper.getQualificationImage(qualification.getRecordId());
            qualification.setImageList(images);
            // 商品资质文件
            List<QualificationPo> fileList = shipMapper.getQualificationFile(qualification.getRecordId());
            qualification.setFileList(fileList);
            if (StringUtils.isBlank(qualification.getSn()) && fileList.isEmpty() && images.isEmpty()) {
                iterator.remove();
            }
        }
        return ResponseJson.success(qualificationList);
    }

    /**
     * 保存商品资质
     *
     * @param jsonParamsDto params:[{
     *                         recordId: 商品发货记录id,
     *                         sn:       sn码,
     *                         files:    [{fileName:"文件名称",ossName:"oss文件名称"},{fileName:"文件名称",ossName:"oss文件名称"}...]
     *                         images:   ["图片","图片",...]
     *                         }, {}]
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseJson<Void> saveProductQualification(JsonParamsDto jsonParamsDto) {
        try {
            JSONArray paramsArray = parseArray(jsonParamsDto.getParams());
            for (Object paramObj: paramsArray) {
                JSONObject jsonObject = (JSONObject) paramObj;
                Integer recordId = jsonObject.getInteger("recordId");
                String sn = jsonObject.getString("sn");
                JSONArray filesArray = jsonObject.getJSONArray("files");
                // 删除旧的资质文件
                shipMapper.deleteQualificationFile(recordId);
                if (!filesArray.isEmpty()) {
                    QualificationPo qualificationFile = new QualificationPo();
                    qualificationFile.setRecordId(recordId);
                    for (Object fileObj: filesArray) {
                        JSONObject file = (JSONObject) fileObj;
                        qualificationFile.setFileName(file.getString("fileName"));
                        qualificationFile.setOssName(file.getString("ossName"));
                        // 保存资质文件
                        shipMapper.insertQualificationFile(qualificationFile);
                    }
                }
                JSONArray imagesArray = jsonObject.getJSONArray("images");
                // 删除旧的资质图片
                shipMapper.deleteQualificationImage(recordId);
                if (!imagesArray.isEmpty()) {
                    for (Object imageObj: imagesArray) {
                        String image = String.valueOf(imageObj);
                        // 保存资质图片
                        shipMapper.insertQualificationImage(image, recordId);
                    }
                }
                if (StringUtils.isNotBlank(sn)) {
                    // 更新商品发货记录SN码
                    shipMapper.updateLogisticsRecordSn(recordId, sn);
                }
            }
            return ResponseJson.success(null);
        } catch (Exception e) {
            log.info("保存商品资质参数:" + jsonParamsDto.toString());
            log.error("【保存商品资质】>>>参数解析异常try-catch:", e);
            return ResponseJson.error("保存商品资质参数解析异常！", null);
        }
    }

    /**
     * 更改收款状态
     *
     * @param amount
     * @param shopOrderId
     * @return
     */
    @Override
    public ResponseJson checkReceipt(Double amount, Integer shopOrderId) {
        Double wait = shipMapper.findWaitPay(shopOrderId);
        if (amount > wait) {
            return ResponseJson.error("支付金额大于待付!");
        }
        // 修改cm_order状态
        Integer orderId = shipMapper.getOrderId(shopOrderId);
        ReceiptVo receiptVo = new ReceiptVo();
        // 31 线下凭证
        receiptVo.setPayType(31);
        receiptVo.setPayWay(2);
        receiptVo.setNewReceiptType(2);
        receiptVo.setReceiptType(1);
        receiptVo.setReceiptStatus(3);
        receiptVo.setReceiptAmount(amount);
        shipMapper.insertOrderReceipt(receiptVo);
        OrderReceiptRelationPo record = new OrderReceiptRelationPo();
        record.setReceiptAmount(amount);
        record.setRelationType(2);
        record.setReceiptId(receiptVo.getId());
        record.setShopOrderId(shopOrderId);
        record.setOrderId(orderId);
        shipMapper.insertOrderRelation(record);
        // 根据金额修改cm_Shop_order状态
        Integer status = 2;
        if (0 == wait - amount) {
            status = 3;
            //分销人员下单修改报备记录
            Integer identity = baseMapper.getUserIdentity(null,orderId, null);
            if (identity == 8) {
                CmReportingClub byCmReportingClub = cmReportingClubMapper.getByCmReportingClub(new CmReportingClub()
                        .orderId(orderId)
                        .orderStatus(0)
                );
                ShopOrderVo shopOrder = orderCommonMapper.getShopOrderByOrderId(shopOrderId);
                //商品报备佣金
                BigDecimal commission = cmReportingClubMapper.getDistributionProductCommission(byCmReportingClub.productId());
                byCmReportingClub
                        .distributionId(shopOrder.getSpId())
                        .lockStatus(1)
                        .lockTime(new Date())
                        .orderStatus(1)
                        .splitCode(cmReportingClubMapper.getDistributionSplitCode(byCmReportingClub.distributionId()))
                        .commission(commission)
                        .splitSettleStatus(2)
                        .settleStatus(0);
                cmReportingClubMapper.updateCmReportingClub(byCmReportingClub);
                List<CmReportingMember> cmReportingMemberList = cmReportingMemberMapper.getCmReportingMemberList(new CmReportingMember()
                        .clubReportingId(Integer.valueOf(byCmReportingClub.id()))
                        .auditStatus(2)
                );
                //商品报备成员佣金
                BigDecimal memberCommission = MathUtil.div(commission, cmReportingMemberList.size(), 2);
                cmReportingMemberList.forEach(s -> {
                    s.commission(memberCommission).settleStatus(2);
                    cmReportingMemberMapper.updateCmReportingMember(s);
                });
            }
        }
        shipMapper.updateReceiptStatus(status, shopOrderId);
        // 获取订单下子订单收款状态
        List<Integer> receiptStatus = shipMapper.getReceiptStatus(orderId);
        Integer sta = 1;
        if (receiptStatus.stream().allMatch(r -> r==3)) {
            sta = 3;
        } else if (receiptStatus.stream().allMatch(r -> r==1)) {
            sta = 1;
        } else {
            sta = 2;
        }
        shipMapper.updateOrderReceiptStatus(sta, orderId);
        return ResponseJson.success();
    }

    @Override
    public List<OrderRemarkDto> findList(Integer orderId) {
        List<OrderRemarkDto> list = shipMapper.findList(orderId);
        list.forEach(l -> {
            l.setImages(shipMapper.findImages(l.getId()));
            l.setOssFiles(shipMapper.findFiles(l.getId()));
        });
        return list;
    }

    @Override
    public void insertRemark(OrderRemarkDto remark) {
        // CmMallAdminUser user = (CmMallAdminUser) SecurityUtils.getSubject().getPrincipal();
        // remark.setCreateBy(user.getId());
        shipMapper.insert(remark);
        List<String> images = remark.getImages();
        List<OssFileDto> ossFiles = remark.getOssFiles();
        if (null != images && images.size() > 0) {
            images.forEach(i -> {
                RemarkImageDto remarkImage = new RemarkImageDto();
                remarkImage.setRemarkImage(i);
                remarkImage.setRemarkId(remark.getId());
                remarkImage.setOrderId(remark.getOrderId());
                shipMapper.insertImage(remarkImage);
            });
        }
        if (null != ossFiles && ossFiles.size() > 0) {
            ossFiles.forEach(o -> {
                o.setRemarkId(remark.getId());
                o.setOrderId(remark.getOrderId());
                shipMapper.insertFile(o);
            });
        }
    }

    @Override
    public ResponseJson<Map<String, Object>> upload(MultipartFile file, Integer orderId) {
        Map<String, Object> map = new HashMap<>();
        String fileAllName = file.getOriginalFilename();
        String fileType = fileAllName.substring(fileAllName.lastIndexOf(".") + 1);
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        String filePath = uuid + "." + fileType;
        String contentType = ossUtils.getContentType(fileAllName);
        try {
            //保存本地
            File fil = ossUtils.ossUpload(file);
            //判断文件的唯一性,转换成16进制md5值
            String md5Hex = DigestUtils.md5Hex(new FileInputStream(fil));
            OssFileDto remarkFile = shipMapper.findFileByMd5(md5Hex);
            String url = "";
            if (remarkFile == null) {
                log.info("默认路径>>>" + fil.getAbsolutePath());
                //上传oss
                url = ossUtils.ossUpload(filePath, fil, contentType);
            } else {
                filePath = remarkFile.getOssName();
            }
            //删除本地文件
            ossUtils.deleteFile(fil);
            //保存关联关系
            OssFileDto orderRemarkFile = new OssFileDto();
            orderRemarkFile.setOrderId(orderId);
            orderRemarkFile.setName(fileAllName);
            orderRemarkFile.setOssName(filePath);
            orderRemarkFile.setMd5Hex(md5Hex);
            map.put("url", url);
            map.put("ossFiles", orderRemarkFile);
            return ResponseJson.success(map);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("备注文件上传异常！！！");
            return ResponseJson.error("备注文件上传异常");
        }
    }

    @Override
    public void downFile(Integer fileId, HttpServletRequest request, HttpServletResponse response) throws IOException {
        OssFileDto remarkFile = shipMapper.findRemarkById(fileId);
        if (null != remarkFile) {
            String fileName = remarkFile.getName();
            ossUtils.downFile(remarkFile.getOssName(), fileName);
            ossService.download(request, response, fileName);
        }
    }
}
