package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.ProductService;
import com.caimei365.order.components.WeChatService;
import com.caimei365.order.constant.Constant;
import com.caimei365.order.mapper.*;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.bo.PayParamBo;
import com.caimei365.order.model.dto.RechargeDto;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.dto.SubmitDto;
import com.caimei365.order.model.bo.OrderParamBo;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.service.SubmitService;
import com.caimei365.order.utils.CodeUtil;
import com.caimei365.order.utils.ImageUtil;
import com.caimei365.order.utils.MathUtil;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

import static com.alibaba.fastjson.JSON.parseArray;
import static com.alibaba.fastjson.JSON.parseObject;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/6
 */
@Slf4j
@Service
public class SubmitServiceImpl implements SubmitService {
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private SubmitMapper submitMapper;
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private ProductService productService;
    @Resource
    private OrderClubMapper orderClubMapper;
    @Resource
    private WeChatService weChatService;
    @Resource
    private CartClubMapper cartClubMapper;
    @Resource
    private CartSellerMapper cartSellerMapper;
    @Resource
    private RemoteCallService remoteCallService;
    @Resource
    private ShipMapper shipMapper;
    @Resource
    private CmReportingClubMapper cmReportingClubMapper;


    /**
     * 生成订单
     *
     * @param submitDto {
     *                  "clubId": 10708,            //机构ID
     *                  "unionId":"",               //微信unionId
     *                  "addressId": 2732,          //地址ID
     *                  "orderSource": 2,           //订单来源：1WWW、2CRM、4客服[适用后台下单]、5外单[适用后台下单]、6小程序[采美,星范]、7呵呵商城小程序、8维沙小程序
     *                  "cartType":3,               //购买类型:（1自主下单(购物车),2自主下单(立即购买)，3协销下单）
     *                  "serviceProviderId": 1378,  //协销ID（小程序忽略）
     *                  "clubCouponId": ""          //优惠券Id
     *                  <p>
     *                  购物车结算/order/club/cart/settlement组装商品
     *                  "orderInfo": [              //【订单商品】
     *                  { "shopId":1001,                         // 供应商Id
     *                  "splitCode:E123123"       //分账号
     *                  "note":备注,
     *                  "productInfo":[                      // skuId，数量，赠品数,商品类型
     *                  {"skuId": 2789, "productNum": 1, "presentNum":0,"productType":2},
     *                  {"skuId": 2789, "productNum": 1, "presentNum":0,"productType":0}
     *                  ]
     *                  },{多个供应商商品数据结构同上}
     *                  ],
     *                  "payInfo": {                //【订单金额】
     *                  "orderShouldPayFee": 609.11,
     *                  "balancePayFlag": 0,
     *                  "clauseId": "2",
     *                  "postage": "15",
     *                  "postageFlag": 1, //运费标志：0包邮 -1到付 1遵循运费规则
     *                  "userBeans": 100,//抵扣采美豆数量
     *                  "rebateFlag":0,
     *                  "rebateFee":0.00 //普通订单返佣服务费
     *                  },
     *                  "orderInvoice":             //【发票信息】
     *                  {"type": 0 // 不开发票  }
     *                  或： { // 普通发票
     *                  "type": 1,
     *                  "invoiceTitle": "企业抬头",
     *                  "invoiceTitleType": 1,
     *                  "corporationTaxNum": "XXX4156465465",
     *                  "invoiceContent": "明细 "
     *                  }
     *                  或： { // 增值税发票
     *                  "type": 2,
     *                  "invoiceTitle": "单位名称",
     *                  "corporationTaxNum": "NSRSBM97897",
     *                  "registeredAddress": "注册地址",
     *                  "registeredPhone": "15814011616",
     *                  "openBank": "开户银行",
     *                  "bankAccountNo": "987987465465464"
     *                  },
     *                  orderSeen:订单对机构可见度，1可见，2不可见
     *                  }
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseJson<Map<String, Object>> generateOrder(SubmitDto submitDto) {
        log.info("submitDto===" + submitDto);
        // 获取机构用户Id
        Integer clubUserId = baseMapper.getUserIdByClubId(submitDto.getClubId());

        if (null == clubUserId || clubUserId == 0) {
            return ResponseJson.error("机构用户信息异常！", null);
        }
        // 用户组织Id
        Integer organizeId = baseMapper.getOrganizeId(clubUserId);
        JSONArray orderInfo = null;
        JSONObject payInfo = null;
        JSONObject orderInvoice = null;
        try {
            orderInfo = parseArray(submitDto.getOrderInfo());
            payInfo = parseObject(submitDto.getPayInfo());
            orderInvoice = parseObject(submitDto.getOrderInvoice());
        } catch (Exception e) {
            log.error("订单参数解析异常try-catch:", e);
            return ResponseJson.error("订单参数解析异常！", null);
        }
        if (null == orderInfo || orderInfo.isEmpty()) {
            return ResponseJson.error("订单商品数据异常！", null);
        }
        if (null == payInfo || payInfo.isEmpty()) {
            return ResponseJson.error("订单金额数据异常！", null);
        }
        if (null == orderInvoice || orderInvoice.isEmpty()) {
            return ResponseJson.error("发票信息数据异常！", null);
        }
        // 打印参数
        if (3 == submitDto.getCartType()) {
            log.info("******************** 【协销订单】提交订单参数：" + submitDto);
        } else if (2 == submitDto.getCartType()) {
            log.info("******************** 【自主订单】立即购买提交订单参数：" + submitDto);
        } else {
            log.info("******************** 【自主订单】购物车提交订单参数：" + submitDto);
        }
        // 机构用户
        OrderParamBo orderParamBo = submitMapper.getOrderUserBoById(clubUserId);
        //订单来源
        orderParamBo.setOrderSource(submitDto.getOrderSource());
        //订单商品
        orderParamBo.setOrderInfo(orderInfo);
        // 余额支付标识，0不使用，1使用
        Integer balancePayFlag = (Integer) payInfo.get("balancePayFlag");
        // 判断用户可用余额是否为0
        if (1 == balancePayFlag && null != orderParamBo.getAbleUserMoney() && MathUtil.compare(orderParamBo.getAbleUserMoney(), BigDecimal.ZERO) == 0) {
            return ResponseJson.error("用户可用余额为0.00元！", null);
        }
        orderParamBo.setBalancePayFlag(balancePayFlag);
        //订单应付总额
        String orderShouldPayFee = (String) payInfo.get("orderShouldPayFee");
        orderParamBo.setOrderShouldPayFee(Double.parseDouble(orderShouldPayFee));
        // 运费标志：0包邮 -1到付 1遵循运费规则
        // Integer postageFlag = (Integer) payInfo.get("postageFlag");
        // 运费
        /*String postage = (String) payInfo.get("postage");
        if (null == postageFlag || null == postage) {
            return ResponseJson.error("运费数据异常！", null);
        }
        orderParamBo.setPostageFlag(postageFlag);
        orderParamBo.setPostage(Double.parseDouble(postage));*/
        // 采美豆抵扣运费，1：100，到付默认30元，抵扣3000豆
        Integer userBeans = (Integer) payInfo.get("userBeans");
        if (null != userBeans && userBeans > 0) {
            // 计算需抵扣采美豆数量
            /*int offsetBeans = MathUtil.mul(postage, 100).intValue();
            if (-1 == postageFlag) {
                // 到付默认30元，抵扣3000豆
                offsetBeans = 3000;
            }*/
            // 与前端传入采美豆数量比较
            /*if (null == orderParamBo.getUserBeans() || !userBeans.equals(offsetBeans)) {
                return ResponseJson.error("采美豆数据异常！", null);
            }*/
            if (orderParamBo.getUserBeans() <= 0) {
                return ResponseJson.error("用户没有剩余采美豆！", null);
            }
        } else {
            userBeans = 0;
        }
        orderParamBo.setOffsetBeans(userBeans);
        // 发票类型 0不开发票 1普通发票 2增值税发票
        Integer invoiceType = (Integer) orderInvoice.get("type");
        if (null == invoiceType) {
            return ResponseJson.error("发票类型不能为空！", null);
        }
        orderParamBo.setInvoiceType(invoiceType);
        /*if (organizeId == 0) {
            // 采美
            if (1 == invoiceType || 2 == invoiceType) {
                InvoicePo invoice = new InvoicePo();
                String invoiceTitle = (String) orderInvoice.get("invoiceTitle");
                if (StringUtils.isEmpty(invoiceTitle)) {
                    return ResponseJson.error("发票抬头信息不正确！", null);
                }
                String corporationTaxNum = (String) orderInvoice.get("corporationTaxNum");
                if (StringUtils.isEmpty(corporationTaxNum)) {
                    return ResponseJson.error("纳税人识别号信息不正确！", null);
                }
                invoice.setType(invoiceType);
                invoice.setInvoiceTitle(invoiceTitle);
                invoice.setCorporationTaxNum(corporationTaxNum);
                if (1 == invoiceType) {
                    // 普通发票：发票类型、发票内容（商品明细）、抬头(公司名称)、纳税人识别号[普通发票的公司]
                    String invoiceContent = (String) orderInvoice.get("invoiceContent");
                    Integer invoiceTitleType = (Integer) orderInvoice.get("invoiceTitleType");
                    if (StringUtils.isEmpty(invoiceContent) || null == invoiceTitleType) {
                        return ResponseJson.error("发票信息不完整！", null);
                    }
                    invoice.setInvoiceContent(invoiceContent);
                    invoice.setInvoiceTitleType(invoiceTitleType);
                }
                if (2 == invoiceType) {
                    // 增值税发票：发票类型、发票、抬头(公司名称)、纳税人识别号、注册地址、注册电话、开户银行、开户银行账户
                    String registeredAddress = (String) orderInvoice.get("registeredAddress");
                    String registeredPhone = (String) orderInvoice.get("registeredPhone");
                    String openBank = (String) orderInvoice.get("openBank");
                    String bankAccountNo = (String) orderInvoice.get("bankAccountNo");
                    if (StringUtils.isEmpty(registeredAddress) || StringUtils.isEmpty(registeredPhone) || StringUtils.isEmpty(openBank) || StringUtils.isEmpty(bankAccountNo)) {
                        return ResponseJson.error("发票信息不完整！", null);
                    }
                    invoice.setRegisteredAddress(registeredAddress);
                    invoice.setRegisteredPhone(registeredPhone);
                    invoice.setOpenBank(openBank);
                    invoice.setBankAccountNo(bankAccountNo);
                }
                orderParamBo.setOrderInvoice(invoice);
            }
        } else {
            // 联合丽格*/
        if (1 == invoiceType || 2 == invoiceType) {
            InvoicePo invoice = new InvoicePo();
            String invoiceTitle = (String) orderInvoice.get("invoiceTitle");
            if (StringUtils.isEmpty(invoiceTitle)) {
                return ResponseJson.error("发票抬头信息不正确！", null);
            }
            invoice.setType(invoiceType);
            invoice.setInvoiceTitle(invoiceTitle);
            String corporationTaxNum = (String) orderInvoice.get("corporationTaxNum");
            if (1 == invoiceType) {
                // 普通发票：发票类型、发票内容（商品明细）、抬头(公司名称)、纳税人识别号[普通发票的公司]
                Integer invoiceTitleType = (Integer) orderInvoice.get("invoiceTitleType");
                String registeredAddress = (String) orderInvoice.get("registeredAddress");
                String registeredPhone = (String) orderInvoice.get("registeredPhone");
                String openBank = (String) orderInvoice.get("openBank");
                String bankAccountNo = (String) orderInvoice.get("bankAccountNo");
                // 抬头类型
                Integer headingType = (Integer) orderInvoice.get("headingType");
                if (0 != headingType) {
                    if (StringUtils.isEmpty(corporationTaxNum)) {
                        return ResponseJson.error("纳税人识别号信息不正确！", null);
                    }
                }
                invoice.setCorporationTaxNum(corporationTaxNum);
                invoice.setHeadingType(headingType);
                invoice.setInvoiceTitleType(invoiceTitleType);
                invoice.setRegisteredAddress(registeredAddress);
                invoice.setRegisteredPhone(registeredPhone);
                invoice.setOpenBank(openBank);
                invoice.setBankAccountNo(bankAccountNo);
            }
            if (2 == invoiceType) {
                // 增值税发票：发票类型、发票、抬头(公司名称)、纳税人识别号、注册地址、注册电话、开户银行、开户银行账户
                String registeredAddress = (String) orderInvoice.get("registeredAddress");
                String registeredPhone = (String) orderInvoice.get("registeredPhone");
                String openBank = (String) orderInvoice.get("openBank");
                String bankAccountNo = (String) orderInvoice.get("bankAccountNo");
                if (StringUtils.isEmpty(registeredAddress) || StringUtils.isEmpty(registeredPhone) || StringUtils.isEmpty(openBank) || StringUtils.isEmpty(bankAccountNo)) {
                    return ResponseJson.error("发票信息不完整！", null);
                }
                invoice.setCorporationTaxNum(corporationTaxNum);
                invoice.setRegisteredAddress(registeredAddress);
                invoice.setRegisteredPhone(registeredPhone);
                invoice.setOpenBank(openBank);
                invoice.setBankAccountNo(bankAccountNo);
            }
            orderParamBo.setOrderInvoice(invoice);
        }
        // 返佣订单标识 0非返佣订单，1返佣订单,2普通订单含有返佣服务费
        Integer rebateFlag = (Integer) payInfo.get("rebateFlag");
        // rebateflag=2时，有rebateFee
        if (null != rebateFlag && 2 == rebateFlag) {
            String re = (String) payInfo.get("rebateFee");
            Double rebateFee = Double.parseDouble(re);
            orderParamBo.setRebateFee(rebateFee);
        }
        orderParamBo.setRebateFlag(rebateFlag);
        // 售后条款id
        Integer clauseId = (Integer) payInfo.get("clauseId");
        Integer isColdChina = (Integer) payInfo.get("isColdChina");
        orderParamBo.setClauseId(clauseId);
        // 购买类型:（1自主下单,2直接购买 3协销下单）
        orderParamBo.setCartType(submitDto.getCartType());
        // 下单人
        Integer buyUserId = null;
        if (1 == submitDto.getCartType() || 2 == submitDto.getCartType()) {
            // 自主下单
            buyUserId = clubUserId;
            if (StringUtils.isNotEmpty(submitDto.getUnionId())) {
                // 运营人员Id
                Integer operationId = submitMapper.getOperationIdByUnionId(submitDto.getUnionId(), clubUserId);
                orderParamBo.setUnionId(submitDto.getUnionId());
                if (null != operationId && operationId > 0) {
                    buyUserId = operationId;
                }
            }
        } else if (3 == submitDto.getCartType()) {

            Integer spUserId = null;
            if (orderParamBo.getUserIdentity() == 8) {
                // 分销人员下单, 获取协销用户Id
                spUserId = submitMapper.getDistributionUserId(submitDto.getServiceProviderId());
                orderParamBo.setClubReportingId(submitDto.getClubReportingId());
            } else {
                // 协销下单, 获取协销用户Id
                spUserId = submitMapper.getServiceProviderUserId(submitDto.getServiceProviderId());
            }
            if (null != spUserId && spUserId > 0) {
                buyUserId = spUserId;
            } else {
                return ResponseJson.error("协销用户异常！", null);
            }
        } else {
            return ResponseJson.error("购买类型不正确！", null);
        }
        orderParamBo.setBuyUserId(buyUserId);
        // 超级会员用户标识
        Integer svipUserId = baseMapper.getSvipUserIdByUserId(clubUserId);
        boolean svipUserFlag = null != svipUserId;
        orderParamBo.setSvipUserFlag(svipUserFlag);
        orderParamBo.setServiceProviderId(submitDto.getServiceProviderId());
        orderParamBo.setAddressId(submitDto.getAddressId());
        orderParamBo.setClubCouponId(submitDto.getClubCouponId());
        orderParamBo.setOrderSeen(submitDto.getOrderSeen());
        orderParamBo.setOrderMiniType(submitDto.getOrderMiniType());
        orderParamBo.setIsColdChain(isColdChina);
        /*
         * 保存订单
         */
        return saveOrder(orderParamBo);
    }

    public ResponseJson<Map<String, Object>> saveOrder(OrderParamBo orderParamBo) {
        // 用户组织Id
        Integer organizeId = baseMapper.getOrganizeId(orderParamBo.getUserId());
        log.info("******************** 提交订单逻辑处理 start *******************");
        /**
         * 初始化主订单
         */
        OrderPo mainOrder = new OrderPo();
        // 订单可见度
        mainOrder.setOrderSeen(orderParamBo.getOrderSeen());
        // 订单来源
        mainOrder.setOrderSource(orderParamBo.getOrderSource());
        // 生成订单号
        if (0 != organizeId) {
            orderParamBo.setOrderSource(10);
        }
        String orderNo = CodeUtil.generateOrderNo(orderParamBo.getOrderSource());
        mainOrder.setOrderNo(orderNo);
        // 用户Id
        mainOrder.setUserId(orderParamBo.getUserId());
        mainOrder.setClubId(orderParamBo.getClubId());
        mainOrder.setBuyUserId(orderParamBo.getBuyUserId());
        // 订单提交时间
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        mainOrder.setOrderTime(curDateStr);
        mainOrder.setUpdateDate(curDateStr);
        // 订单状态 0 有效  其它无效
        mainOrder.setDelFlag(0);
        //协销下单
        if (3 == orderParamBo.getCartType()) {
            mainOrder.setSpId(orderParamBo.getServiceProviderId());
            // 协销订单
            mainOrder.setOrderType(0);
            // 3:协销帮会所下单
            mainOrder.setOrderSubmitType(3);
            // 订单未确认
            mainOrder.setConfirmFlag(0);
        } else {
            // 普通订单
            mainOrder.setOrderType(1);
            // 其他默认 2:员工帮会所下单
            mainOrder.setOrderSubmitType(2);
            // 2买家确认
            mainOrder.setConfirmFlag(2);
        }
        if (1 == organizeId) {
            // 联合丽格订单
            mainOrder.setOrderType(3);
        }
        // 抵扣采美豆数量
        mainOrder.setUserBeans(orderParamBo.getOffsetBeans());
        // 默认可以线上支付：0可以 1不可以
        mainOrder.setOnlinePayFlag(0);
        // 默认订单可以拆分：1可拆分 0不可拆分
        mainOrder.setSplitFlag(1);
        // 默认订单未支付：未支付0 已支付1
        mainOrder.setPayFlag(0);
        // 默认待向买家收款：1待收款、2部分收款、3已收款
        mainOrder.setReceiptStatus(1);
        // 默认未付款供应商：1待付款、2部分付款、3已付款
        mainOrder.setPayStatus(1);
        // 默认有成本
        mainOrder.setZeroCostFlag(0);
        // 发货状态：1待发货、2部分发货、3已发货
        mainOrder.setSendOutStatus(1);
        // 退货退款类型：0未发生退款、1部分退、2全部退
        mainOrder.setRefundType(0);
        // 未确认付款供应商
        mainOrder.setAffirmPaymentFlag(0);
        // 是否开发票
        mainOrder.setInvoiceFlag(orderParamBo.getInvoiceType());

        /**
         * 整理订单商品信息
         */
        // 商品总数量
        AtomicInteger productCount = new AtomicInteger(0);
        // 赠品数量
        AtomicInteger presentCount = new AtomicInteger(0);
        // 促销赠品数量
        AtomicInteger promotionalGiftsCount = new AtomicInteger(0);
        // 促销满减优惠
        AtomicDouble promotionFullReduction = new AtomicDouble(0);
        // 超级会员优惠
        AtomicDouble svipFullReduction = new AtomicDouble(0);
        // 商品总金额 （商品单价乘以数量，再加上税费）
        AtomicDouble productTotalFee = new AtomicDouble(0);
        // 订单总额（小计金额减去经理折扣后，再加上运费[默认0])
        AtomicDouble payTotalFee = new AtomicDouble(0);
        // 真实支付金额（订单总额减去抵扣的账户余额）
        AtomicDouble payableAmount = new AtomicDouble(0);
        // 余额支付金额
        AtomicDouble balancePayFee = new AtomicDouble(0);
        // 总运费
        AtomicDouble postageFee = new AtomicDouble(0);
        // 统计销量用
        JSONArray salesInfo = new JSONArray();

        // 二手订单标记（二手订单不能同正常商品下单，只能单个商品立即购买下单）
        boolean secondHandOrderFlag = false;
        // 是否包含活动商品
        boolean hasActProductFlag = false;
        // 是否能线上支付
        AtomicBoolean onlinePay = new AtomicBoolean(true);
        // 用余额线下
        if (null != orderParamBo.getBalancePayFlag() && 1 == orderParamBo.getBalancePayFlag()) {
            onlinePay.set(false);
        }
        //促销活动ids
        List<Integer> promotionsIds = new ArrayList<>();
        // 促销活动信息
        List<PromotionsVo> promotionList = new ArrayList<>();
        // 订单商品列表
        List<OrderProductPo> orderProductList = new ArrayList<>();
        List<String> productIdList = new ArrayList<>();
        List<String> skuIdList = new ArrayList<>();

        // 余额抵扣金额
        AtomicReference<Double> balance = new AtomicReference(0);
        if (1 == orderParamBo.getBalancePayFlag()) {
            balance.set(orderParamBo.getAbleUserMoney());
        }
        // 子订单订单列表
        List<OrderShopPo> shopOrderList = new ArrayList<>();
        JSONArray orderInfo = orderParamBo.getOrderInfo();
        for (Object infoObject : orderInfo) {
            JSONObject shopInfo = (JSONObject) infoObject;
            Integer shopId = (Integer) shopInfo.get("shopId");
            String shopNote = (String) shopInfo.get("note");
            String postageString = (String) shopInfo.get("postage");
            Integer postageFlag = (Integer) shopInfo.get("postageFlag");
            Integer isColdChinaShop = (Integer) shopInfo.get("isColdChina");
            if (null == shopId) {
                return ResponseJson.error("供应商数据异常！", null);
            }
            JSONArray productArr = (JSONArray) shopInfo.get("productInfo");
            if (null == productArr) {
                return ResponseJson.error("订单商品数据异常！", null);
            }
            if (1252 == shopId) {
                secondHandOrderFlag = true;
            }
            // 运费方式为空： 赠品 包邮
            postageFlag = null == postageFlag ? 0 : postageFlag;
            log.info("postageFlag*****************" + postageFlag);
            // 运费
            Double postage = 0d;
            // 在 运费类型为有运费时 运费生效
            if (null != postageFlag && 1 == postageFlag) {
                postage = Double.parseDouble(postageString);
            }
            // 供应商名称
            String shopName = baseMapper.getShopNameById(shopId);
            /*
             * 初始化子订单
             */
            OrderShopPo shopOrder = new OrderShopPo();
            shopOrder.setPromotionFullReduction(0d);
            shopOrder.setShopId(shopId);
            shopOrder.setShopName(shopName);
            shopOrder.setNote(shopNote);
            // 是否勾选冷链运输费
            shopOrder.setIsColdChina(isColdChinaShop);
            // 用户Id
            shopOrder.setUserId(orderParamBo.getUserId());
            shopOrder.setClubId(orderParamBo.getClubId());
            if (3 == orderParamBo.getCartType()) {
                /**
                 * todo 协销订单，在添加保存时子订单的协销Id应该保存下单协销的协销Id
                 *      21年改为保存下单人userId，会对订单后续处理产生疑惑，后续应该继续保存协销id
                 */
                shopOrder.setSpId(orderParamBo.getServiceProviderId());
            }
            // 商品总数量
            AtomicInteger shopProductCount = new AtomicInteger(0);
            // 佣金 =  应付采美
            AtomicDouble brokerage = new AtomicDouble(0);
            // 需要支付金额
            AtomicDouble needPayAmount = new AtomicDouble(0);
            // 商品总金额
            AtomicDouble shopAmount = new AtomicDouble(0);
            // 付供应商 商品费
            AtomicDouble shopProductAmount = new AtomicDouble(0);
            // 付供应商税费
            AtomicDouble shopTaxFee = new AtomicDouble(0);
            // 店铺促销活动状态
            PromotionsVo shopPromotions = baseMapper.getPromotionByShopId(shopId);
            if (shopPromotions != null && shopPromotions.getSeen() != null && shopPromotions.getSeen() == 2 && 3 == orderParamBo.getCartType()) {
                shopPromotions = null;
            }
            // 商品费
            AtomicDouble shopProductFee = new AtomicDouble(0);
            // 供应商下参与店铺促销的商品总价
            AtomicDouble shopPromotionFee = new AtomicDouble(0);
            // 供应商超级会员优惠
            AtomicDouble svipShopReduction = new AtomicDouble(0);

            // 遍历所有商品
            for (Object productObject : productArr) {
                JSONObject productTemp = (JSONObject) productObject;
                Integer skuId = (Integer) productTemp.get("skuId");
                Integer productNum = (Integer) productTemp.get("productNum");
                Integer presentNum = (Integer) productTemp.get("presentNum");
                Integer productType = (Integer) productTemp.get("productType");
                productType = (null == productType) ? 0 : productType;
                if (null == skuId) {
                    return ResponseJson.error("订单商品数据异常！", null);
                }
                if (null == productNum || productNum == 0) {
                    return ResponseJson.error("商品购买数量异常！", null);
                }
                // 统计商品总数量
                productCount.updateAndGet(v -> v + productNum);
                // 赠品数
                presentCount.updateAndGet(v -> v + presentNum);
                // 获取数据库商品信息
                OrderProductPo dbProduct = null;
                if (0 == organizeId) {
                    //分销人员生成机构
                    if (orderParamBo.getUserIdentity() == 8) {
                        dbProduct = submitMapper.getDistributionProductDetails(skuId);
                    } else {
                        dbProduct = submitMapper.getProductDetails(skuId);
                    }
                } else {
                    dbProduct = submitMapper.getProductOrganizeDetails(skuId);
                }
                OrderProductPo product = new OrderProductPo();
                BeanUtils.copyProperties(dbProduct, product);
                log.info("splitcode-----------------------》" + product.getSplitCode());
                if (null == product) {
                    return ResponseJson.error("订单商品不存在！", null);
                }

                // 是否二手商品
                if (null != product.getProductCategory() && 2 == product.getProductCategory()) {
                    secondHandOrderFlag = true;
                }
                // 是否添加税费，不含税商品 开票需添加税费
                boolean taxFlag = (Integer.valueOf(0).equals(product.getIncludedTax()) && (Integer.valueOf(1).equals(product.getInvoiceType()) || Integer.valueOf(2).equals(product.getInvoiceType())));

                Double shopTax = 0d;
                // 图片路径
                String image = ImageUtil.getImageUrl("product", product.getImage(), domain);
                product.setImage(image);
                product.setShopName(shopName);
                product.setNum(productNum);
                product.setPresentNum(presentNum);
                product.setProductType(productType);
                product.setSvipPriceFlag(0);
                product.setSvipReduction(0d);
                // 组织信息
                product.setOrganizeId(organizeId);

                // 超级会员优惠商品详情
                SvipProductPo svipProductPo = submitMapper.getSvipProductDetails(product.getSkuId());
                // 是否以超级会员优惠价格购买
                boolean svipPriceFlag = null != svipProductPo && 1 == svipProductPo.getSvipProductFlag() && orderParamBo.getSvipUserFlag();
                // 是否是促销赠品
                if (productType == 2) {
                    // 促销赠品计算商品费
                    Double productPrice = product.getPrice();
                    Double discountPrice = product.getPrice();
                    // 商品税费
                    Double productTax = 0d;
                    Double discountTax = 0d;
                    if (taxFlag) {
                        productTax = MathUtil.div(MathUtil.mul(productPrice, product.getTaxRate()), 100, 2).doubleValue();
                        discountTax = MathUtil.div(MathUtil.mul(discountPrice, product.getTaxRate()), 100, 2).doubleValue();
                    } else if (!Integer.valueOf(1).equals(product.getIncludedTax())) {
                        // 不含税不可开票商品和未知商品，税率置为0
                        product.setTaxRate(0d);
                    }
                    log.info("促销赠品计算：productTax===" + productTax);
                    // 商品价格
                    if (Integer.valueOf(0).equals(product.getIncludedTax())) {
                        if (null != product.getTaxRate() && product.getTaxRate() > 0d) {
                            product.setPrice(MathUtil.add(productPrice, MathUtil.mul(productPrice, MathUtil.div(product.getTaxRate(), 100, 2))).doubleValue());
                        }
                    } else {
                        product.setPrice(productPrice);
                    }
                    // 商品税后价格
                    Double productTaxPrice = MathUtil.add(productPrice, productTax).doubleValue();
                    // 折后税后单价
                    Double discountTaxPrice = MathUtil.add(discountPrice, discountTax).doubleValue();
                    // 单个商品的金额
                    Double productAmount = MathUtil.mul(productTaxPrice, productNum).doubleValue();
                    // 单个商品的金额
                    product.setTotalAmount(productAmount);
                    product.setPrice(productPrice);
                    // 折后单价
                    product.setDiscountPrice(discountTaxPrice);
                    // 促销赠品数+1
                    promotionalGiftsCount.incrementAndGet();
                    product.setDiscount(0d);
                    product.setTotalAmount(0d);
                    product.setTotalFee(0d);
                    product.setTaxRate(0d);
                    product.setAddedValueTax(0d);
                    product.setTotalAddedValueTax(0d);
                } else {
                    // 不是赠品，获取商品购买价格(超级会员优惠商品价格>>>活动价格>>>阶梯价格>>>复购价格库>>>商品原始价)
                    Double productPrice = product.getPrice();
                    Double itemPrice = product.getPrice();
                    Double discountPrice = product.getPrice();
                    // 商品税费
                    Double productTax = 0d;
                    Double discountTax = 0d;
                    // 商品是否处于活动状态
                    PromotionsVo promotions = null;
                    // 店铺促销优先
                    if (null == shopPromotions) {
                        promotions = baseMapper.getPromotionByProductId(product.getProductId());
                        if (null != promotions && 1 == promotions.getType() && 1 == promotions.getMode()) {
                            //单品促销的优惠价时，touchPrice取当前sku的
                            promotions.setTouchPrice(baseMapper.getTouchPriceBySku(skuId, promotions.getId()));
                        }
                    }
                    //促销活动如果协销不可见，置为空
                    if (promotions != null && promotions.getSeen() != null && promotions.getSeen() == 2 && 3 == orderParamBo.getCartType()) {
                        promotions = null;
                    }
                    // 计算单价
                    // 超级会员单价折扣计算
                    if (0 == organizeId && orderParamBo.getUserIdentity() != 8) {
                        if (null != svipProductPo && 1 == svipProductPo.getSvipProductFlag()) {
                            // 超级会员优惠商品,不参与促销活动（普通机构购买该商品可参与店铺促销）/阶梯价/复购价
                            if (orderParamBo.getSvipUserFlag()) {
                                if (1 == svipProductPo.getSvipPriceType()) {
                                    // 折扣价
                                    discountPrice = MathUtil.div(MathUtil.mul(productPrice, svipProductPo.getSvipDiscount()), 100, 2).doubleValue();
                                } else if (2 == svipProductPo.getSvipPriceType()) {
                                    // 直接优惠价
                                    discountPrice = svipProductPo.getSvipDiscountPrice();
                                }
                                // 商品超级会员优惠
                                BigDecimal svipReduction = MathUtil.sub(productPrice, discountPrice);
                                if (taxFlag) {
                                    // 商品税费超级会员优惠
                                    BigDecimal svipTaxReduction = MathUtil.div(MathUtil.mul(svipReduction, product.getTaxRate()), 100, 2);
                                    svipReduction = MathUtil.add(svipReduction, svipTaxReduction);
                                }
                                // 统计超级会员优惠
                                product.setSvipReduction(svipReduction.doubleValue());
                                svipShopReduction.set(MathUtil.add(svipShopReduction, MathUtil.mul(svipReduction, productNum)).doubleValue());
                                svipFullReduction.set(MathUtil.add(svipFullReduction, MathUtil.mul(svipReduction, productNum)).doubleValue());
                                // 保存订单商品svip数据
                                product.setSvipPriceFlag(1);
                                product.setSvipPriceType(svipProductPo.getSvipPriceType());
                                product.setSvipDiscount(1 == svipProductPo.getSvipPriceType() ? svipProductPo.getSvipDiscount() : 0.00d);
                            }
                        }
                        // 有单品优惠价促销活动
                        else if (null != promotions || null != shopPromotions) {
                            // 是否包含活动商品(受订单未支付自动关闭时间影响)  0 否 1 是
                            hasActProductFlag = true;
                            // 关闭阶梯价格，活动优先
                            product.setLadderPriceFlag(0);
                            product.setActProduct(1);
                            if (null != promotions && promotions.getType() == 1 && promotions.getMode() == 1) {
                                discountPrice = promotions.getTouchPrice();
                                // 单品促销 作用到付供应商 计算商品费用
                                productPrice = promotions.getTouchPrice();
                            }
                        }

                        //有阶梯价
                        else if (1 == product.getLadderPriceFlag()) {
                            // 启用了阶梯价格
                            List<LadderPriceVo> ladderPrices = baseMapper.getLadderPriceList(skuId);
                            // 判断阶梯价格的购买数量校验
                            int minBuyNumber = null != ladderPrices.get(0) ? ladderPrices.get(0).getBuyNum() : 0;
                            if (productNum < minBuyNumber) {
                                return ResponseJson.error("商品购买量低于最小起订量！", null);
                            }
                            //根据商品购买数量获取商品对应阶梯价格
                            for (LadderPriceVo ladderPrice : ladderPrices) {
                                if (productNum >= ladderPrice.getBuyNum()) {
                                    discountPrice = ladderPrice.getBuyPrice();
                                    // 有阶梯价 作用到付供应商 计算商品费用
                                    productPrice = ladderPrice.getBuyPrice();
                                }
                            }
                            product.setLadderPrices(ladderPrices);
                            product.setActProduct(2);
                        }
                        /*else {
                            // 复购价
                            Double repurchase = baseMapper.getRepurchasePrice(skuId, orderParamBo.getUserId());
                            if (null != repurchase && repurchase > 0) {
                                discountPrice = repurchase;
                            }
                            product.setActProduct(0);
                        }*/
                    }
                    if (MathUtil.compare(discountPrice, BigDecimal.ZERO) == 0) {
                        return ResponseJson.error("商品购买价格不能为0！", null);
                    }
                    // 不含税可开票商品计算税费
                    if (null == product.getTaxRate() || product.getTaxRate() <= 0) {
                        product.setTaxRate(0d);
                    }
                    if (taxFlag) {
                        productTax = MathUtil.div(MathUtil.mul(productPrice, product.getTaxRate()), 100, 2).doubleValue();
                        discountTax = MathUtil.div(MathUtil.mul(discountPrice, product.getTaxRate()), 100, 2).doubleValue();
                    } else if (!Integer.valueOf(1).equals(product.getIncludedTax())) {
                        // 不含税不可开票商品和未知商品，税率置为0
                        product.setTaxRate(0d);
                    }
                    log.info("productTax===" + productTax);
                    // 商品价格
                    if (Integer.valueOf(0).equals(product.getIncludedTax())) {
                        if (null != product.getTaxRate() && product.getTaxRate() > 0d) {
                            product.setPrice(MathUtil.add(productPrice, MathUtil.mul(productPrice, MathUtil.div(product.getTaxRate(), 100, 2))).doubleValue());
                        }
                    } else {
                        product.setPrice(productPrice);
                    }
                    // 商品税后价格
                    Double productTaxPrice = MathUtil.add(productPrice, productTax).doubleValue();
                    // 折后税后单价
                    Double discountTaxPrice = MathUtil.add(discountPrice, discountTax).doubleValue();
                    // 折扣率 = 折后单价/机构价
                    Double discountRate = MathUtil.mul(MathUtil.div(discountTaxPrice, productTaxPrice), 100).doubleValue();
                    // 单个商品的金额
                    Double productAmount = MathUtil.mul(productTaxPrice, productNum).doubleValue();
                    // 单个商品的折后金额
                    Double productFee = MathUtil.mul(discountTaxPrice, productNum).doubleValue();
                    // 不包含单品满减的商品金额
                    Double realProductFee = productFee;
                    // 单个商品总税费
                    Double taxFee = MathUtil.mul(discountTax, productNum).doubleValue();
                    // 商品售价还原
                    productPrice = itemPrice;
                    product.setPrice(productPrice);
                    if (hasActProductFlag && null != promotions) {
                        // 单品促销
                        if (0 == organizeId && orderParamBo.getUserIdentity() != 8) {
                            if (promotions.getType() == 1) {
                                // 是否满足单品优惠条件（满足优惠价或者满减在这里进行设值）
                                if (promotions.getMode() == 1 || MathUtil.compare(productFee, promotions.getTouchPrice()) >= 0) {
                                    // 满减
                                    if (promotions.getMode() == 2) {
                                        if (promotions.getDiscount() != null && promotions.getDiscount() == 1) {
                                            //叠加优惠计算
                                            //叠加倍数
                                            Double floor = Math.floor(MathUtil.div(productFee, promotions.getTouchPrice()).doubleValue());
                                            promotionFullReduction.set(MathUtil.add(promotionFullReduction.get(), MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                            productFee = MathUtil.sub(productFee, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue();
                                            shopOrder.setPromotionFullReduction(MathUtil.add(shopOrder.getPromotionFullReduction(), MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());

                                        } else {
                                            // 统计订单总满减金额
                                            promotionFullReduction.set(MathUtil.add(promotionFullReduction.get(), promotions.getReducedPrice()).doubleValue());
                                            // 统计单个商品的折后金额
                                            productFee = MathUtil.sub(productFee, promotions.getReducedPrice()).doubleValue();
                                            shopOrder.setPromotionFullReduction(MathUtil.add(shopOrder.getPromotionFullReduction(), promotions.getReducedPrice()).doubleValue());
                                        }
                                    }
                                    // 添加到总促销
                                    if (!promotionsIds.contains(promotions.getId())) {
                                        promotionsIds.add(promotions.getId());
                                        promotions.setPromotionsId(promotions.getId());
                                        if (taxFlag) {
                                            promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(), MathUtil.div(MathUtil.mul(promotions.getTouchPrice(), product.getTaxRate()), 100, 2)).doubleValue());
                                        }
                                        promotionList.add(promotions);
                                    }
                                    product.setOrderPromotionsId(promotions.getId());
                                }
                                // 凑单优惠
                            } else if (promotions.getType() == 2) {
                                // 商品添加到总促销
                                PromotionPriceVo promotionPrice = new PromotionPriceVo();
                                promotionPrice.setProductId(product.getProductId());
                                promotionPrice.setNumber(productNum);
                                promotionPrice.setPrice(discountTaxPrice);
                                if (null == promotions.getProductList()) {
                                    promotions.setProductList(new ArrayList<>());
                                }
                                promotions.getProductList().add(promotionPrice);
                                if (!promotionsIds.contains(promotions.getId())) {
                                    promotionsIds.add(promotions.getId());
                                    promotions.setPromotionsId(promotions.getId());
                                    if (taxFlag) {
                                        promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(), MathUtil.div(MathUtil.mul(promotions.getTouchPrice(), product.getTaxRate()), 100, 2)).doubleValue());
                                    }
                                    promotionList.add(promotions);
                                } else {
                                    PromotionsVo finalPromotions = promotions;
                                    promotionList.forEach(promotionsTemp -> {
                                        if (finalPromotions.getId().equals(promotionsTemp.getId())) {
                                            promotionsTemp.getProductList().add(promotionPrice);
                                        }
                                    });
                                }
                                product.setOrderPromotionsId(promotions.getId());
                            }
                        }
                    }
                    // 折后单价
                    product.setDiscountPrice(discountTaxPrice);
                    // 折扣率 = 折后单价/机构价
                    product.setDiscount(discountRate);
                    if (svipPriceFlag || (null != promotions && promotions.getType() == 1 && promotions.getMode() == 1)) {
                        // 超级会员价/单品优惠折扣率设置为100%
                        product.setDiscount(100d);
                    }
                    // 单个商品的金额
                    product.setTotalAmount(productAmount);
                    // 单个商品的折后金额
                    product.setTotalFee(productFee);
                    product.setShouldPayFee(productFee);
                    product.setRealProductFee(realProductFee);
                    // 税费
                    product.setAddedValueTax(discountTax);
                    product.setTotalAddedValueTax(taxFee);
                    // 销量
                    JSONObject salesObject = new JSONObject();
                    salesObject.put("productId", product.getProductId());
                    salesObject.put("productCount", product.getNum());
                    salesInfo.add(salesObject);
                }

                // 成本价
                Double costPrice = null;
                /**
                 * 判断是否选中固定成本价
                 * 固定成本、比例成本都需要计算税费
                 */
                if (null != product.getCostPrice() && 1 == product.getCostCheckFlag() && product.getCostPrice() >= 0d) {
                    // 通过售价*比例得到成本价
                    double costTax = 0d;
                    if (taxFlag) {
                        costTax = MathUtil.div(MathUtil.mul(product.getCostPrice(), product.getTaxRate()), 100, 2).doubleValue();
                    }
                    costPrice = MathUtil.add(product.getCostPrice(), costTax).doubleValue();
                }
                // 判断是否选中比例成本价
                if (null != product.getCostProportional() && 2 == product.getCostCheckFlag() && product.getCostProportional() > 0d) {
                    // 通过售价*比例得到成本价
                    /*double discountTax = 0d;
                    if (taxFlag) {
                        discountTax = MathUtil.div(MathUtil.mul(product.getDiscountPrice(), product.getTaxRate()), 100, 2).doubleValue();
                    }*/
                    costPrice = MathUtil.div(MathUtil.mul(product.getDiscountPrice(), product.getCostProportional()), 100, 2).doubleValue();
                }
                if (null == costPrice) {
                    return ResponseJson.error("订单商品成本异常！", null);
                }
                product.setCostPrice(costPrice);
                // 促销赠品 售价归 0
                if (productType == 2) {
                    product.setPrice(0d);
                    product.setDiscountPrice(0d);
                }
                // 付供应商税费
                if (null == product.getShopTaxRate() || product.getShopTaxRate() <= 0) {
                    product.setShopTaxRate(product.getTaxRate());
                }
                if (taxFlag) {
                    shopTax = MathUtil.div(MathUtil.mul(MathUtil.div(costPrice, MathUtil.div(MathUtil.add(product.getTaxRate(), 100), 100)), product.getShopTaxRate()), 100, 2).doubleValue();
                } else if (!Integer.valueOf(1).equals(product.getIncludedTax())) {
                    // 不含税不可开票商品和未知商品，税率置为0
                    product.setShopTaxRate(0d);
                }
                // 单个付供应商税费
                product.setSingleShouldPayTotalTax(shopTax);
                // 单个商品付供应商总税费
                product.setShouldPayTotalTax(MathUtil.mul(shopTax, productNum).doubleValue());
                // 付供应商 商品费=成本价*(购买数量  + 赠品数量)
                Double costAmount = MathUtil.mul(costPrice, MathUtil.add(productNum, presentNum)).doubleValue();
                product.setShopProductAmount(costAmount);
                log.info("product.getShopProductAmount()***" + product.getShopProductAmount());
                //应付供应商(单)=成本价+供应商税费(单)
                Double singleShopFee = MathUtil.add(product.getCostPrice(), shopTax).doubleValue();
                product.setSingleShopFee(singleShopFee);
                // 应付供应商(总)=应付供应商(单) * 商品数量
                Double shopFee = MathUtil.add(product.getCostPrice(), shopTax).doubleValue();
                product.setShopFee(shopFee);
                // 付第三方，默认0
                product.setOtherFee(0d);
                product.setSingleOtherFee(0d);
                //应付采美(单)=单价+机构税费(单)-(成本(单)+供应商税费(单))
                Double singleCmFee = MathUtil.sub(MathUtil.add(BigDecimal.valueOf(product.getPrice()), product.getAddedValueTax()), MathUtil.add(product.getCostPrice(), shopTax)).doubleValue();
                product.setSingleCmFee(singleCmFee);
                // 应付采美(总)=应付采美(单)*商品数量
                Double cmFee = MathUtil.mul(singleCmFee, productNum).doubleValue();
                product.setCmFee(cmFee);
                // 默认未支付
                product.setPayStatus(0);
                // 默认非再次购买商品
                product.setBuyAgainFlag(0);
                // 未出库数量
                product.setNotOutStore(productNum);
                /*
                 * 子订单金额计算
                 */
                // 供应商商品数量
                shopProductCount.set(shopProductCount.get() + product.getNum());
                // 佣金 = 应付采美
                brokerage.set(MathUtil.add(brokerage.get(), product.getCmFee()).doubleValue());
                // 需要支付金额 shouldPayFee +运费
                needPayAmount.set(MathUtil.add(needPayAmount.get(), product.getShouldPayFee()).doubleValue());
                // 统计 总金额 包括税费
                shopProductFee.set(MathUtil.add(shopProductFee.get(), product.getTotalFee()).doubleValue());
                // 统计子订单内可参与店铺促销的商品总金额
                if (!svipPriceFlag) {
                    // 超级会员购买优惠商品，不能参与店铺促销
                    shopPromotionFee.set(MathUtil.add(shopPromotionFee.get(), product.getTotalFee()).doubleValue());
                }
                // 商品总金额
                shopAmount.set(MathUtil.add(shopAmount.get(), product.getTotalAmount()).doubleValue());
                // 付供应商 商品费
                log.info("shopProductAmount.get()" + shopProductAmount.get());
                log.info("product.getShopProductAmount()" + product.getShopProductAmount());
                shopProductAmount.set(MathUtil.add(shopProductAmount.get(), product.getShopProductAmount()).doubleValue());
                // 付供应商税费
                shopTaxFee.set(MathUtil.add(shopTaxFee.get(), product.getShouldPayTotalTax()).doubleValue());
                // 成本 | 佣金比例  商城sku 成本 | 配置比例
                CmOrganizeSkuPo organizeSkuInfo = null;
                if (orderParamBo.getUserIdentity() == 8) {
                    organizeSkuInfo = submitMapper.getDistributionSkuInfo(skuId, organizeId);
                } else {
                    organizeSkuInfo = submitMapper.getOrganizeSkuInfo(skuId, organizeId);
                }
                if (StringUtils.isNotEmpty(organizeSkuInfo.getCostCheckFlag()) && "1".equals(organizeSkuInfo.getCostCheckFlag())) {
                    // 固定成本
                    if (taxFlag) {
                        organizeSkuInfo.setOrganizeCostPrice(MathUtil.add(organizeSkuInfo.getOrganizeCostPrice(), MathUtil.div(MathUtil.mul(organizeSkuInfo.getOrganizeCostPrice(), product.getTaxRate()), 100)).doubleValue());
                        organizeSkuInfo.setCmCostPrice(MathUtil.add(organizeSkuInfo.getCmCostPrice(), MathUtil.div(MathUtil.mul(organizeSkuInfo.getCmCostPrice(), product.getTaxRate()), 100)).doubleValue());
                    }
                    product.setOrganizeCostPrice(organizeSkuInfo.getOrganizeCostPrice());
                    product.setCmCostPrice(organizeSkuInfo.getCmCostPrice());
                }
                if (StringUtils.isNotEmpty(organizeSkuInfo.getCostCheckFlag()) && "2".equals(organizeSkuInfo.getCostCheckFlag())) {
                    // 比例成本
                    product.setOrganizeCostPrice(MathUtil.div(MathUtil.mul(product.getPrice(), organizeSkuInfo.getOrganizePercent()), 100).doubleValue());
                    product.setCmCostPrice(MathUtil.div(MathUtil.mul(product.getPrice(), organizeSkuInfo.getCmPercent()), 100).doubleValue());

                    product.setCmPercent(organizeSkuInfo.getCmPercent());
                    product.setOrganizePercent(organizeSkuInfo.getOrganizePercent());
                    product.setShopPercent(organizeSkuInfo.getShopPercent());
                }
                // 加入订单商品列表
                // 商品价格设置为 折后价格
                // product.setPrice(product.getDiscountPrice());
                orderProductList.add(product);
                productIdList.add(product.getProductId().toString());
                skuIdList.add(product.getSkuId().toString());
            }
            // 设置运费
            if (1 == postageFlag && 0 == mainOrder.getUserBeans()) {
                shopOrder.setShopPostFee(postage);
            }
            // 需要支付金额 shouldPayFee +运费
            needPayAmount.set(MathUtil.add(needPayAmount.get(), postage).doubleValue());
            log.info("postage=-***====" + postage);
            // 统计总运费
            postageFee.set(MathUtil.add(postageFee.get(), postage).doubleValue());
            log.info("统计总运费+--+-+-" + postageFee.get());
            Integer count = submitMapper.findSplitResult(productIdList);
            if (count > 0) {
                //有商品无分帐号，走线下
                onlinePay.set(false);
            }
            if (0 == organizeId && orderParamBo.getUserIdentity() != 8) {
                if (null != shopPromotions) {
                    // 是否满足满减满赠
                    if (MathUtil.compare(shopPromotionFee.get(), shopPromotions.getTouchPrice()) >= 0) {
                        // 是否包含活动商品(受订单未支付自动关闭时间影响)  0 否 1 是
                        hasActProductFlag = true;
                        // 满减
                        if (shopPromotions.getMode() == 2) {
                            if (shopPromotions.getDiscount() != null && shopPromotions.getDiscount() == 1) {
                                //叠加优惠计算
                                //叠加倍数
                                Double floor = Math.floor(MathUtil.div(shopPromotionFee.get(), shopPromotions.getTouchPrice()).doubleValue());
                                // 统计订单总满减金额
                                promotionFullReduction.set(MathUtil.add(promotionFullReduction.get(), MathUtil.mul(shopPromotions.getReducedPrice(), floor)).doubleValue());
                                // 统计店铺商品总金额
                                shopProductFee.set(MathUtil.sub(shopProductFee.get(), MathUtil.mul(shopPromotions.getReducedPrice(), floor)).doubleValue());
                                // 统计需要支付金额
                                needPayAmount.set(MathUtil.sub(needPayAmount.get(), MathUtil.mul(shopPromotions.getReducedPrice(), floor)).doubleValue());
                                // 统计需要支付金额
                                shopAmount.set(MathUtil.sub(shopAmount.get(), MathUtil.mul(shopPromotions.getReducedPrice(), floor)).doubleValue());
                                // 供应商满减金额
                                log.info("shopOrder.getPromotionFullReduction()---------------------" + shopOrder.getPromotionFullReduction());
                                log.info("setPromotionFullReduction-----------------------" + MathUtil.mul(shopPromotions.getReducedPrice(), floor));
                                ;
                                shopOrder.setPromotionFullReduction(MathUtil.add(shopOrder.getPromotionFullReduction(), MathUtil.mul(shopPromotions.getReducedPrice(), floor)).doubleValue());
                            } else {
                                // 统计订单总满减金额
                                promotionFullReduction.set(MathUtil.add(promotionFullReduction.get(), shopPromotions.getReducedPrice()).doubleValue());
                                // 统计店铺商品总金额
                                shopProductFee.set(MathUtil.sub(shopProductFee.get(), shopPromotions.getReducedPrice()).doubleValue());
                                // 统计需要支付金额
                                needPayAmount.set(MathUtil.sub(needPayAmount.get(), shopPromotions.getReducedPrice()).doubleValue());
                                // 统计需要支付金额
                                shopAmount.set(MathUtil.sub(shopAmount.get(), shopPromotions.getReducedPrice()).doubleValue());
                                // 供应商满减金额
                                log.info("shopOrder.getPromotionFullReduction()---------------------" + shopOrder.getPromotionFullReduction());
                                log.info("setPromotionFullReduction-----------------------" + shopPromotions.getReducedPrice());
                                shopOrder.setPromotionFullReduction(MathUtil.add(shopOrder.getPromotionFullReduction(), shopPromotions.getReducedPrice()).doubleValue());
                            }
                        }
                        // 添加到总促销
                        if (!promotionsIds.contains(shopPromotions.getId())) {
                            promotionsIds.add(shopPromotions.getId());
                            shopPromotions.setPromotionsId(shopPromotions.getId());
                            promotionList.add(shopPromotions);
                        }
                        shopOrder.setOrderPromotionsId(shopPromotions.getId());
                    }
                }
            }
            // 佣金 采美应收
            shopOrder.setBrokerage(brokerage.get());
            // 可退款金额 = 余额抵扣金额
            shopOrder.setCanRefundAmount(needPayAmount.get());
            // 购买商品数
            shopOrder.setItemCount(shopProductCount.get());
            // 超级会员优惠金额
            shopOrder.setSvipShopReduction(svipShopReduction.get());
            // 总价(含税) = totalFee + 运费
            shopOrder.setTotalAmount(MathUtil.add(shopProductFee.get(), postage).doubleValue());
            // 总金额 = 订单商品totalAmount
            shopOrder.setProductAmount(shopAmount.get());
            // 需要支付金额 shouldPayFee +运费
            shopOrder.setNeedPayAmount(needPayAmount.get());
            // 付供应商 商品费=成本价*(购买数量  + 赠品数量)
            shopOrder.setShopProductAmount(shopProductAmount.get());
            // 付给供应商税费
            shopOrder.setShopTaxFee(shopTaxFee.get());
            // 付供应商 = 商品费 + 运费
            // 版本不加 税费
            shopOrder.setShouldPayShopAmount(MathUtil.add(shopProductAmount.get(), postageFee.get()).doubleValue());
            // 付给供应商运费
            // shopOrder.setShopPostFee(0d);
            // 已付供应商金额
            shopOrder.setPayedShopAmount(0d);
            // (付款供应商)付款状态：1待付款、2部分付款、3已付款
            shopOrder.setPayStatus(1);
            if (3 == orderParamBo.getCartType()) {
                // 协销订单
                shopOrder.setOrderType(0);
            } else {
                // 普通订单
                shopOrder.setOrderType(1);
                shopOrder.setConfirmTime(curDateStr);
            }
            // 余额支付标识，0不使用，1使用
            log.info("balance.get()=====" + balance.get());
            if (1 == orderParamBo.getBalancePayFlag() && 3 == orderParamBo.getCartType() && balance.get() > 0) {
                // 使用余额支付
                shopOrder.setUseBalanceFlag(1);
                // 部分抵扣
                if (MathUtil.compare(shopOrder.getNeedPayAmount(), balance.get()) > 0) {
                    // 余额抵扣金额
                    shopOrder.setAccountAmount(balance.get());
                } else {
                    // 余额抵扣金额
                    shopOrder.setAccountAmount(shopOrder.getNeedPayAmount());
                }
                balance.updateAndGet(b -> balance.get() - shopOrder.getNeedPayAmount());
            } else {
                // 使用余额支付
                shopOrder.setUseBalanceFlag(0);
                // 余额抵扣金额
                shopOrder.setAccountAmount(0d);
            }
            log.info("shopOrder.getAccountAmount()====" + shopOrder.getAccountAmount());
            // 运费标记
            shopOrder.setShopPostFlag(postageFlag);
            // 添加到子订单列表
            shopOrderList.add(shopOrder);
            // 统计商品总金额
            productTotalFee.set(MathUtil.add(productTotalFee.get(), shopProductFee.get()).doubleValue());
        }
        if (orderProductList.isEmpty()) {
            return ResponseJson.error("订单商品数据异常！", null);
        }
        // 设置是否是二手订单
        if (secondHandOrderFlag) {
            mainOrder.setSecondHandOrderFlag(1);
        } else {
            mainOrder.setSecondHandOrderFlag(0);
        }
        if (0 == organizeId && orderParamBo.getUserIdentity() != 8) {
            // 是否包含活动商品(受订单未支付自动关闭时间影响)  0 否 1 是
            if (!hasActProductFlag) {
                mainOrder.setHasActProduct(0);
            } else {
                mainOrder.setHasActProduct(1);
                // 促销活动优惠计算
                Iterator<PromotionsVo> promotionsIterator = promotionList.iterator();
                while (promotionsIterator.hasNext()) {
                    PromotionsVo promotions = promotionsIterator.next();
                    // 凑单优惠计算（单品与店铺已再前面分开计算了）
                    if (promotions.getType() == 2) {
                        // 这里计算满减满赠
                        if (promotions.getMode() == 2 || promotions.getMode() == 3) {
                            // 实际该促销商品总额
                            AtomicDouble tempProductFee = new AtomicDouble(0);
                            promotions.getProductList().forEach(item -> tempProductFee.set(MathUtil.add(tempProductFee.get(), MathUtil.mul(item.getNumber(), item.getPrice())).doubleValue()));
                            //判断是否达到满减满赠要求
                            if (MathUtil.compare(tempProductFee.get(), promotions.getTouchPrice()) >= 0) {
                                if (promotions.getMode() == 2) {
                                    if (promotions.getDiscount() != null && promotions.getDiscount() == 1) {
                                        //叠加优惠计算
                                        //叠加倍数
                                        Double floor = Math.floor(MathUtil.div(tempProductFee.get(), promotions.getTouchPrice()).doubleValue());
                                        // 满减
                                        promotionFullReduction.set(MathUtil.add(promotionFullReduction.get(), MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                        // 统计商品总金额
                                        productTotalFee.set(MathUtil.sub(productTotalFee.get(), MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                        // 当前版本凑单促销仅支持综合供应商 ，shopid 1161
                                        shopOrderList.forEach(s -> {
                                            if (1161 == s.getShopId()) {
                                                s.setPromotionFullReduction(MathUtil.mul(promotions.getReducedPrice(), floor).doubleValue());
                                            }
                                        });
                                    } else {
                                        // 满减
                                        promotionFullReduction.set(MathUtil.add(promotionFullReduction.get(), promotions.getReducedPrice()).doubleValue());
                                        // 统计商品总金额
                                        productTotalFee.set(MathUtil.sub(productTotalFee.get(), promotions.getReducedPrice()).doubleValue());
                                        // 当前版本凑单促销仅支持综合供应商 ，shopid 1161
                                        shopOrderList.forEach(s -> {
                                            if (1161 == s.getShopId()) {
                                                s.setPromotionFullReduction(promotions.getReducedPrice());
                                            }
                                        });
                                    }
                                }
                            } else {
                                // 删除不满足条件的凑单促销
                                promotionsIterator.remove();
                            }
                        }
                    }
                }
                // 促销满减优惠
                mainOrder.setPromotionFullReduction(promotionFullReduction.get());
            }
        }
        // 商品总数量
        mainOrder.setProductCount(productCount.get());
        // 赠品数量
        mainOrder.setPresentCount(presentCount.get());
        //促销赠品数量
        mainOrder.setPromotionalGiftsCount(promotionalGiftsCount.get());
        /**
         * 处理优惠券
         */
        Double couponAmount = 0d;
        CouponVo coupon = null;
        if (null != orderParamBo.getClubCouponId() && orderParamBo.getClubCouponId() > 0) {
            coupon = orderClubMapper.getClubCouponById(orderParamBo.getClubCouponId(), orderParamBo.getUserId());
            if (null == coupon) {
                // 设置手动回滚事务
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ResponseJson.error("优惠券使用异常！", null);
            }
            couponAmount = coupon.getCouponAmount();
        }
        // 美博会临时设置，7260，7261临时套餐商品，不可使用优惠券
        if (productIdList.contains(7260) || productIdList.contains(7261)) {
            couponAmount = 0d;
        }
        mainOrder.setCouponAmount(couponAmount);
        // 订单总额 = 商品费 + 运费
        log.info("总运费" + postageFee.get());
        payTotalFee.set(MathUtil.add(productTotalFee.get(), postageFee.get()).doubleValue());
        log.info("payTotalFee.get()=------=-=-====" + payTotalFee.get());
        // 订单总额 -= 优惠券金额
        payTotalFee.set(MathUtil.sub(payTotalFee.get(), couponAmount).doubleValue());
        // 超级会员优惠
        mainOrder.setSvipFullReduction(svipFullReduction.get());
        // 商品总额
        mainOrder.setProductTotalFee(productTotalFee.get());
        mainOrder.setOrderTotalFee(productTotalFee.get());
        // 订单总额（商品金额+运费）-- 冷链运输费
        mainOrder.setPayTotalFee(payTotalFee.get());
        // 订单状态
        if (3 == orderParamBo.getCartType()) {
            // 协销 状态为 待确认
            mainOrder.setStatus(0);
        } else {
            // 机构用户 状态为 待收待发
            mainOrder.setStatus(11);
            mainOrder.setConfirmTime(curDateStr);
        }
        // 是否完成支付(默认不是，只有余额抵扣才算)
        boolean isPaySuccessFlag = false;
        // 余额支付标识，0不使用，1使用
        if (1 == orderParamBo.getBalancePayFlag()) {
            // 抵扣后用户剩余可用余额
            Double lastAbleUserMoney;
            // 抵扣后账户余额
            Double lastUserMoney = orderParamBo.getUserMoney();
            // 部分抵扣
            if (MathUtil.compare(payTotalFee.get(), orderParamBo.getAbleUserMoney()) > 0) {
                balancePayFee.set(orderParamBo.getAbleUserMoney());
                payableAmount.set(MathUtil.sub(payTotalFee.get(), balancePayFee.get()).doubleValue());
                // 余额抵扣用完
                lastAbleUserMoney = 0d;
                if (3 != orderParamBo.getCartType()) {
                    // 部收款待发货
                    mainOrder.setStatus(21);
                    lastUserMoney = MathUtil.sub(orderParamBo.getUserMoney(), balancePayFee.get()).doubleValue();
                }
                mainOrder.setReceiptStatus(2);
            } else {
                // 全部用余额抵扣， 直接变成支付完成
                balancePayFee.set(payTotalFee.get());
                payableAmount.set(0d);
                lastAbleUserMoney = MathUtil.sub(orderParamBo.getAbleUserMoney(), balancePayFee.get()).doubleValue();
                if (3 != orderParamBo.getCartType()) {
                    // 已收款待发货
                    mainOrder.setStatus(31);
                    lastUserMoney = MathUtil.sub(orderParamBo.getUserMoney(), balancePayFee.get()).doubleValue();
                }
                mainOrder.setReceiptStatus(3);
                mainOrder.setPayFlag(1);
                isPaySuccessFlag = true;
            }
            // 抵扣后更新用户余额
            submitMapper.updateUserMoney(lastUserMoney, lastAbleUserMoney, orderParamBo.getUserId());
            log.info("【提交订单】>>>>>更新用户余额，抵扣后剩余：[userMoney:" + lastUserMoney + "] ,ableUserMoney:" + lastAbleUserMoney);
            // 支付时间
            mainOrder.setPayTime(curDateStr);
        } else {
            log.info("payTotalFee.get()=------=-=-====" + payTotalFee.get());
            //不用余额的情况下真是支付金额赋值为订单总额
            payableAmount.set(payTotalFee.get());
        }
        // 余额支付金额
        mainOrder.setBalancePayFee(balancePayFee.get());
        // 实际支付金额(商品金额+运费-余额抵扣)
        mainOrder.setPayableAmount(payableAmount.get());

        // 是否返佣订单
        Integer rebateFlag = (null == orderParamBo.getRebateFlag() ? 0 : orderParamBo.getRebateFlag());
        mainOrder.setRebateFlag(rebateFlag);
        if (2 == rebateFlag) {
            mainOrder.setRebateFee(orderParamBo.getRebateFee());
        }

        log.info("订单总额" + payTotalFee.get());
        // 判断前端传入orderShouldPayFee订单应付金额,和后台计算应付金额对比
        log.info("【提交订单】>>>>>后台计算payableAmount:" + payableAmount.get() + " ,前端传入orderShouldPayFee:" + orderParamBo.getOrderShouldPayFee());
        int compare = MathUtil.compare(orderParamBo.getOrderShouldPayFee(), payableAmount.get());
        if (compare != 0) {
            // 设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("订单付款金额不正确！", null);
        }
        // 售后条款
        Integer clauseId = orderParamBo.getClauseId();
        String clauseName = null;
        if (null != clauseId && clauseId > 0) {
            clauseName = submitMapper.getClauseNameById(clauseId);
        }
        if (StringUtils.isNotEmpty(clauseName)) {
            mainOrder.setClauseId(clauseId);
            mainOrder.setClauseName(clauseName);
        } else {
            mainOrder.setClauseId(1);
            mainOrder.setClauseName("无条款");
        }
        // 设置组织Id
        mainOrder.setOrganizeId(organizeId);
        if (null == mainOrder.getUserId()) {
            // 设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("用户Id为空！", null);
        }
        CmReportingClub cmReportingClubById = cmReportingClubMapper.getCmReportingClubById(orderParamBo.getClubReportingId());
        if (orderParamBo.getUserIdentity() == 8&&null!=cmReportingClubById.orderStatus()) {
            // 设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("抱歉，该报备中已有人下单，暂无法下单！", null);
        }

        // 根据用户 Id 查询机构 Id
        Integer clubId = submitMapper.getClubId(mainOrder.getUserId());
        mainOrder.setClubId(clubId);
        /**
         * 保存主订单数据，返回订单Id
         */
        submitMapper.insertMainOrder(mainOrder);
        log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增主订单(insert[cm_order])orderId：" + mainOrder.getOrderId());
        //分销人员下单修改报备记录
        if (orderParamBo.getUserIdentity() == 8) {
            cmReportingClubMapper.updateCmReportingClub(new CmReportingClub()
                    .id(orderParamBo.getClubReportingId())
                    .orderId(mainOrder.getOrderId())
                    .orderStatus(0)
                    .splitCode(cmReportingClubMapper.getDistributionSplitCode(cmReportingClubById.distributionId()))
            );
        }
        //推送信息中心-交易物流
        MessageCenter messageCenter = new MessageCenter();
        messageCenter.setShopID(null);
        messageCenter.setClubID(orderParamBo.getClubId());
        messageCenter.setUserType(1);
        messageCenter.setMessageType(1);
        messageCenter.setOrderMessageType(1);
        messageCenter.setOrderID(mainOrder.getOrderId());
        messageCenter.setContent(null);
        messageCenter.setTime(curDateStr);
        orderClubMapper.addMessageCenter(messageCenter);

        /**
         * 保存订单促销,返回订单促销Id
         */
        promotionList.forEach(promotions -> {
            promotions.setOrderId(mainOrder.getOrderId());
            submitMapper.insertOrderPromotions(promotions);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增订单促销活动(insert[cm_promotions_order])id：" + promotions.getId() + "，orderId：" + mainOrder.getOrderId());
        });
        /**
         * 线上单分摊优惠
         */
        if (presentCount.get() > 0) {
            //有赠品不能线上，赠品成本无法分账
            onlinePay.set(false);
        }
        if (onlinePay.get()) {
//            //在店铺满减算完之后重新根据商品的splitCode排shopOrderList
            List<OrderShopPo> shops = submitMapper.findShops(productIdList);
//            //子订单分多子订单，满减分摊到原子订单分开的子订单，无满减重算佣金/应付
//            //重组订单
//            shopOrderList = findSplitShopOrder(orderProductList, shopOrderList, shops, orderParamBo);
            for (OrderShopPo shop : shops) {
                shopOrderList.forEach(s -> {
                    if (s.getShopId().equals(shop.getShopId())) {
                        s.setSplitCode(shop.getSplitCode());
                    }
                });
            }
//            // 分摊优惠前总佣金
//            AtomicDouble oldBrokerage = new AtomicDouble(0);
//            //超级会员价/复购价/阶梯价/促销活动冲突，只能生效一个
//            shopOrderList.forEach(orderShopPo -> {
//                orderShopPo.setOldPrice(orderShopPo.getNeedPayAmount());
//                orderShopPo.setOldBrokerage(orderShopPo.getBrokerage());
//                oldBrokerage.set(MathUtil.add(oldBrokerage.get(), orderShopPo.getOldBrokerage()).doubleValue());
//
//            });
            // 按分摊前的佣金排序
            shopOrderList.sort((o1, o2) -> o2.getOldBrokerage().compareTo(o1.getOldBrokerage()));
            // 前面子订单的分摊优惠总和
            /** todo
             *  当前版本为子订单下单，且优惠仅存在供应商优惠，优惠金额应从供应商成本扣除，暂时去除均摊优惠逻辑
             */
            //无优惠金额
            shopOrderList.forEach(so -> {
//                //不包含优惠的真实支付金额=子订单应付？
//                double realPay = so.getNeedPayAmount();
//                so.setRealPay(realPay);
//                so.setEachDiscount(0d);
                double bro = so.getBrokerage();
//                so.setRealPay(realPay);
//                so.setEachDiscount(0d);
                //成本>实付不能线上
                if (bro < 0) {
                    onlinePay.set(false);
                } else {
                    double WXCharge = MathUtil.mul(so.getRealPay(), 0.0065, 2).doubleValue();
                    double QuickCharge = MathUtil.mul(so.getRealPay(), 0.006, 2).doubleValue();
                    double AliCharge = MathUtil.mul(so.getRealPay(), 0.0025, 2).doubleValue();
                    double B2CCharge = MathUtil.mul(so.getRealPay(), 0.002, 2).doubleValue();
                    //佣金和手续费比较,佣金>手续费则可以走当前支付渠道
                    //支持子订单佣金小于最小手续费时，也能进行线上支付，此时不限制支付方式(佣金大于最大手续费或小于最小手续费支持全渠道付款)
                    String payWays = "";
                    payWays += bro >= WXCharge ? "1" : "";
                    payWays += bro >= AliCharge ? ",2" : "";
                    payWays += bro >= B2CCharge ? ",3" : "";
                    payWays += bro >= 10 ? ",4" : "";
                    payWays += bro >= QuickCharge ? ",5" : "";
                    //小于所有的情况下线下补手续费，开放所有渠道
                    if ("".equals(payWays)) {
                        payWays = "1,2,3,4,5";
                    }
                    so.setOnlinePayWays(payWays);
                }
                log.info("线上重排子订单--------------------------" + so);
            });
        }
        /*
         * 整理 子订单信息cm_shop_order
         */
        AtomicReference<String> shopOrderIds = new AtomicReference<>();
        AtomicInteger shopOrderIndex = new AtomicInteger(1);
        shopOrderList.forEach(shopOrder -> {
            shopOrder.setOrderId(mainOrder.getOrderId());
            shopOrder.setOrderNo(mainOrder.getOrderNo());
            shopOrder.setOrderTime(mainOrder.getOrderTime());
            shopOrder.setOrderSubmitType(mainOrder.getOrderSubmitType());
            // 子订单状态
            if (0 == mainOrder.getStatus()) {
                shopOrder.setShopStatus(0);
            } else {
                shopOrder.setShopStatus(1);
                // 普通订单 订单默认已确认
                shopOrder.setConfirmTime(curDateStr);
            }
            // 生成子订单编号
            StringBuilder shopOrderNo = new StringBuilder();
            shopOrderNo.append(mainOrder.getOrderNo());
            if (shopOrderIndex.get() < 10) {
                shopOrderNo.append("0");
            }
            shopOrderNo.append(shopOrderIndex.get());
            shopOrderIndex.incrementAndGet();
            shopOrder.setShopOrderNo(shopOrderNo.toString());
            // 订单能否拆分 1 为可拆分, 0为不可拆分
            if (shopOrder.getItemCount() > 1) {
                shopOrder.setSplitFlag(1);
            } else {
                shopOrder.setSplitFlag(0);
            }
            // 设置订单促销Id
            promotionList.forEach(promotions -> {
                if (promotions.getPromotionsId().equals(shopOrder.getOrderPromotionsId())) {
                    shopOrder.setOrderPromotionsId(promotions.getId());
                }
            });
            // 设置组织Id
            shopOrder.setOrganizeId(organizeId);
            // 分帐号
            if (0 != organizeId) {
                // 联合丽格
                shopOrder.setRealPay(shopOrder.getNeedPayAmount());
            }
            shopOrder.setClubId(clubId);
            /*
             * 保存子订单
             */
            log.info("shopOrderStatus======" + shopOrder.getShopStatus());
            submitMapper.insertShopOrder(shopOrder);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增子订单(insert[cm_shop_order])shopOrderId：" + shopOrder.getShopOrderId() + "，orderId：" + mainOrder.getOrderId());
            // 收集子订单Id字符串
            if (StringUtils.isEmpty(shopOrderIds.get())) {
                shopOrderIds.set(shopOrder.getShopOrderId().toString());
            } else {
                shopOrderIds.set(shopOrderIds.get() + "," + shopOrder.getShopOrderId());
            }
        });

        /**
         * 保存优惠券使用情况
         * todo
         * 重排子订单后，对促销活动和优惠券的优惠金额对子订单进行减免，
         * 当前版本存在问题，当子订单中商品有多个商户号时，以上根据商户号重新分配子订单会把一个供应商多个商户号的子订单拆分为多个，
         * 此时优惠金额无法再次分配
         */
        if (null != orderParamBo.getClubCouponId() && orderParamBo.getClubCouponId() > 0) {
            /**  修改优惠券使用情况
             *   couponType = 0/3
             *   当前版本仅开放指定商品优惠券和店铺券，优惠券减免金额算作指定商品的供应商/店铺卷供应商对应子订购单的金额减免
             *   totalAmount/needPayAmount/discountAmount应减去优惠券金额
             *   preferential加上优惠券金额
             */
            if (3 != orderParamBo.getCartType()) {
                // 自主订单立即确认，协销订单待确认 ，确认订单时才真正使用优惠券
                orderClubMapper.updateCouponClubStatus(orderParamBo.getClubCouponId(), mainOrder.getOrderId());
            }
            CouponVo finalCoupon = coupon;
            shopOrderList.forEach(s -> {
                /** todo
                 *  shopProductAmount，shouldPayShopAmount - promotionFullReduction
                 *  当前版本付供应商应-促销活动，优惠券金额，因为当前版本所有优惠为商家优惠
                 */
                if (null != s.getPromotionFullReduction() && s.getPromotionFullReduction() > 0) {
                    // 不直接作用付供应商商品费 作用于付供应商
                    // s.setShopProductAmount(s.getShopProductAmount() - s.getPromotionFullReduction());
                    s.setShouldPayShopAmount(s.getShouldPayShopAmount() - s.getPromotionFullReduction());
                    submitMapper.updateShopOrderCoupon(s);
                }
                if (0 == finalCoupon.getCouponType() || 3 == finalCoupon.getCouponType()) {
                    // 当前版本所有指定商品均出自综合供应商，所以limit1
                    Integer shopId = null == finalCoupon.getShopId() ? baseMapper.findShopIdByProduct(finalCoupon.getCouponId()) : finalCoupon.getShopId();
                    s.setCouponAmount(finalCoupon.getCouponAmount());
                    s.setCouponClubId(finalCoupon.getClubCouponId());
                    if (s.getShopId().equals(shopId)) {
                        Double realPay = null == s.getRealPay() ? s.getNeedPayAmount() : s.getRealPay();
                        s.setNeedPayAmount(s.getNeedPayAmount() - finalCoupon.getCouponAmount());
                        s.setTotalAmount(s.getTotalAmount() - finalCoupon.getCouponAmount());
                        s.setRealPay(realPay - finalCoupon.getCouponAmount());
                        // 不直接作用付供应商商品费 作用于付供应商
                        // s.setShopProductAmount(s.getShopProductAmount() - finalCoupon.getCouponAmount());
                        s.setShouldPayShopAmount(s.getShouldPayShopAmount() - finalCoupon.getCouponAmount());
                        submitMapper.updateShopOrderCoupon(s);
                    }
                }
            });
            CouponOrderRecordPo orderRecord = new CouponOrderRecordPo();
            BeanUtils.copyProperties(coupon, orderRecord);
            orderRecord.setOrderId(mainOrder.getOrderId());
            orderRecord.setCreateDate(date);
            // 保存订单优惠券使用记录
            submitMapper.insertCouponOrderRecord(orderRecord);
        }

        /**
         * 订单商品分子订单cm_order_product
         */
        for (OrderProductPo orderProduct : orderProductList) {
            // 设置商品订单Id
            orderProduct.setOrderId(mainOrder.getOrderId());
            orderProduct.setOrderNo(mainOrder.getOrderNo());
            // 设置商品子订单Id
            shopOrderList.forEach(shopOrder -> {
                //能线上的一定有分账号，按分账号分子订单
                if (onlinePay.get()) {
                    //可以线上的情况下，子订单的商户号和商品的一致分到一个子订单
                    if (orderProduct.getShopId().equals(shopOrder.getShopId()) && orderProduct.getSplitCode().equals(shopOrder.getSplitCode())) {
                        orderProduct.setShopOrderId(shopOrder.getShopOrderId());
                        orderProduct.setShopOrderNo(shopOrder.getShopOrderNo());
                    }
                } else {
                    if (orderProduct.getShopId().equals(shopOrder.getShopId())) {
                        orderProduct.setShopOrderId(shopOrder.getShopOrderId());
                        orderProduct.setShopOrderNo(shopOrder.getShopOrderNo());
                    }
                }
            });
            // 设置订单商品促销Id
            promotionList.forEach(promotions -> {
                if (promotions.getPromotionsId().equals(orderProduct.getOrderPromotionsId())) {
                    orderProduct.setOrderPromotionsId(promotions.getId());
                    orderProduct.setActProduct(1);
                }
            });
            orderProduct.setOrganizeId(organizeId);
            /*
             * 保存 订单商品
             */
            submitMapper.insertOrderProduct(orderProduct);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>保存订单商品(insert[cm_order_product])OrderProductId：" + orderProduct.getOrderProductId() + "，orderId：" + mainOrder.getOrderId());
            /*
             * 保存 订单阶梯价格
             */
            if (orderProduct.getLadderPriceFlag() == 1) {
                // 使用阶梯价格的订单商品保存下单时的阶梯价格列表
                List<LadderPriceVo> ladderPrices = orderProduct.getLadderPrices();
                if (null != ladderPrices && ladderPrices.size() > 0) {
                    ladderPrices.forEach(ladderPriceVo -> {
                        ladderPriceVo.setOrderProductId(orderProduct.getOrderProductId());
                        ladderPriceVo.setCreateDate(date);
                        submitMapper.insertLadderPrices(ladderPriceVo);
                        log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>保存订单阶梯价格(insert[order_product_ladder_price])id：" + ladderPriceVo.getId() + "，orderId：" + mainOrder.getOrderId());
                    });
                }
            }
            /*
             * 机构用户 写入价格库
             */
            if (3 != orderParamBo.getCartType() && 1 != orderProduct.getSvipPriceFlag() && 0 == organizeId) {
                // 写入复购价格库
                productService.savePurchasePrice(mainOrder, orderProduct, secondHandOrderFlag, "【提交订单】");
            }
        }

        /**
         * 更新主订单的子订单Id信息,如:1000,1002
         */
        if (StringUtils.isNotEmpty(shopOrderIds.get())) {
            mainOrder.setShopOrderIds(shopOrderIds.get());
            submitMapper.updateShopOrderIds(shopOrderIds.get(), mainOrder.getOrderId());
        } else {
            // 设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("生成子订单异常！", null);
        }
        /**
         * 更新主订单线上支付按钮
         * onlinePayFlag:0可以 1不可以
         */
        Integer onlinePayFlag = onlinePay.get() ? 0 : 1;
        // 组织 是否线上判定
        boolean item = true;
        if (0 != organizeId) {
            // 子订单分帐号
            for (OrderShopPo shopOrder : shopOrderList) {
                if (StringUtils.isEmpty(shopOrder.getSplitCode()) || StringUtils.isBlank(shopOrder.getSplitCode())) {
                    item = false;
                }
            }
            if (item) {
                onlinePayFlag = 0;
            } else {
                onlinePayFlag = 1;
            }
        }
        submitMapper.updateOnlinePayFlag(onlinePayFlag, mainOrder.getOrderId());
        // 更新子订单线上支付
        if (0 != organizeId && item) {
            String onlinePayWays = "1,2,3,4,5";
            submitMapper.updateOnlinePayWays(onlinePayWays, mainOrder.getOrderId());
        }
        /*
         * 保存 订单收货用户信息
         */
        AddressVo address = submitMapper.getAddressDetailById(orderParamBo.getAddressId());
        if (null != address) {
            String userName = baseMapper.getUserNameByUserId(mainOrder.getUserId());
            //保存地址信息
            OrderUserInfoPo userInfo = new OrderUserInfoPo();
            userInfo.setOrderId(mainOrder.getOrderId());
            userInfo.setClubId(mainOrder.getClubId());
            userInfo.setUserId(mainOrder.getUserId());
            userInfo.setName(userName);
            userInfo.setReceiver(address.getReceiver());
            userInfo.setMobile(address.getMobile());
            userInfo.setTownId(address.getTownId());
            userInfo.setProvince(address.getProvince());
            userInfo.setCity(address.getCity());
            userInfo.setTown(address.getTown());
            userInfo.setAddress(address.getAddress());
            // 保存 订单收货用户信息
            submitMapper.insertOrderUserInfo(userInfo);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>保存订单用户地址(insert[bp_order_userinfo])orderId：" + mainOrder.getOrderId());
        } else {
            //设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("订单地址异常！", null);
        }

        /**
         * 保存余额到余额收支记录
         */
        // 余额支付标识，0不使用，1使用
        if (1 == orderParamBo.getBalancePayFlag() && MathUtil.compare(balancePayFee.get(), 0) > 0) {
            // 保存余额抵扣到收款记录（自主订单）
            if (3 != orderParamBo.getCartType()) {
                productService.saveDiscernReceipt(balancePayFee.get(), shopOrderList, "【提交订单】");
            }
        }

        /*
         * 保存发票信息
         */
        if (1 == orderParamBo.getInvoiceType() || 2 == orderParamBo.getInvoiceType()) {
            /*
             * 更新订单增值税发票
             */
            InvoicePo orderInvoice = submitMapper.getOrderInvoice(mainOrder.getOrderId());
            if (null != orderInvoice && null != orderInvoice.getId()) {
                // 更新
                BeanUtils.copyProperties(orderParamBo.getOrderInvoice(), orderInvoice);
                orderInvoice.setOrderId(mainOrder.getOrderId());
                submitMapper.updateOrderInvoice(orderInvoice);
                log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>更新订单发票信息(update[bp_order_invoice])orderId：" + mainOrder.getOrderId());
            } else {
                // 新增
                orderInvoice = new InvoicePo();
                BeanUtils.copyProperties(orderParamBo.getOrderInvoice(), orderInvoice);
                orderInvoice.setOrderId(mainOrder.getOrderId());
                //  保存 订单发票信息
                submitMapper.insertOrderInvoice(orderInvoice);
                log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>保存订单发票信息(insert[bp_order_invoice])orderId：" + mainOrder.getOrderId());
            }
            /*
             * 更新用户增值税发票
             */
            InvoiceVo userInvoice = baseMapper.getUserInvoice(mainOrder.getUserId());
            if (null != userInvoice && null != userInvoice.getId()) {
                // 更新
                BeanUtils.copyProperties(orderInvoice, userInvoice);
                userInvoice.setUserId(mainOrder.getUserId());
                baseMapper.updateUserInvoiceByUserId(userInvoice);
                log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>更新用户增值税发票(update[cm_user_invoiceinfo])userId：" + mainOrder.getUserId());
            } else {
                // 新增
                userInvoice = new InvoiceVo();
                BeanUtils.copyProperties(orderInvoice, userInvoice);
                userInvoice.setUserId(mainOrder.getUserId());
                baseMapper.insertUserInvoiceByUserId(userInvoice);
                log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增用户增值税发票(insert[cm_user_invoiceinfo])userId：" + mainOrder.getUserId());
            }
        }

        /*
         * 保存采美豆使用记录
         */
        if (orderParamBo.getOffsetBeans() > 0) {
            UserBeansHistoryPo beansHistory = new UserBeansHistoryPo();
            beansHistory.setUserId(mainOrder.getUserId());
            beansHistory.setOrderId(mainOrder.getOrderId());
            beansHistory.setBeansType(10);
            beansHistory.setType(2);
            beansHistory.setNum(orderParamBo.getOffsetBeans());
            beansHistory.setPushStatus(0);
            beansHistory.setAddTime(date);
            beansHistory.setDelFlag(0);
            // 更新用户剩余采美豆数量
            int beans = orderParamBo.getUserBeans() - beansHistory.getNum();
            productService.updateUserBeans(beansHistory, beans, "【提交订单】");
        }

        if (0 == organizeId) {
            if (orderParamBo.getOrderMiniType() != 2) {
                boolean bol = false;
                // 机构一级分类
                String firstClubType = baseMapper.firstClubTypeById(orderParamBo.getClubId());
                if (!StringUtils.isNotBlank(firstClubType)) {
                    firstClubType = "0";
                }
                // 获取订单商品械字号类型
                List<String> machineType = baseMapper.machineTypeByID(skuIdList);
                for (String str : machineType) {
                    if ("3".equals(str)) {
                        bol = true;
                    }
                }
                if (!"1".equals(firstClubType) && bol) {
                    // 设置手动回滚事务
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return ResponseJson.error(-6, "订单内存在械字号三类商品，需要拥有医疗执业许可证的医美机构才能购买。建议升级医美机构后再下单，否则会导致订单退款或影响发货。");
                }
            }
        }
        /*
         * 删除购物车
         */
        if (3 == orderParamBo.getCartType()) {
            // 清理当前机构的协销购物车数据
            cartSellerMapper.deleteSellerCartByProductIds(orderParamBo.getServiceProviderId(), orderParamBo.getClubId(), skuIdList);
            log.info("【提交订单】>>>>删除当前机构的 协销 购物车数据！");
        } else if (1 == orderParamBo.getCartType()) {
            // 清理用户购物车
            cartClubMapper.deleteCartByProductIds(orderParamBo.getUserId(), skuIdList);
            log.info("【提交订单】>>>>删除当前机构 用户 购物车数据！");
        }
        log.info("**********************微信模板消息推送*****************************");
        List<Integer> productIds = orderClubMapper.getProductOrder(mainOrder.getOrderId());
        if (0 == organizeId && 0 == mainOrder.getSecondHandOrderFlag() && 0 == mainOrder.getRebateFlag() && (null != productIds && !productIds.contains(6060)) && mainOrder.getOrderType() != 0) {
            log.info("*******获取公众名" + orderClubMapper.getOrderIds(mainOrder.getOrderId()).toString());
            String name = "0";
            try {
                String accessToken = weChatService.getAccessToken();
                List<String> openidList = orderClubMapper.getOpenidListByPermission(orderParamBo.getUnionId());
                orderClubMapper.AddUnionId(orderParamBo.getUnionId(), mainOrder.getOrderId());
                openidList.removeIf(Objects::isNull);
                String i = orderClubMapper.getOrderIds(mainOrder.getOrderId()).toString();
                name = StringUtils.strip(i, "[]");
                log.info("订单商品>>>>>" + name + "" + mainOrder.getOrderId());
                String associateTitle = "订单已成功提交，等待付款！";
                Double money = mainOrder.getPayTotalFee();
                String orderno = mainOrder.getOrderNo();
                String remarkText = "下单时间：" + mainOrder.getOrderTime() + "点击详情进行付款";
                // 跳转到【小程序付款-选择支付方式页面】
                String pagePath = "pages/user/order/order-details?orderId=" + mainOrder.getOrderId();
                for (String openid : openidList) {
                    // sendTemplateMsg(openid, 标题, 金额, 收款日期, 备注, 跳转链接
                    log.error("获取openid>>>>>" + openidList);
                    weChatService.sendTemplateMsgs(accessToken, openid, associateTitle, name, orderno, money, remarkText, pagePath);
                }
            } catch (Exception e) {
                log.error("【订单下单通知】获取微信公众号access_token异常！", e);
            }
        }

        log.info("******************** 提交订单逻辑处理 end *******************");

        /*
         * 下单推送(自主订单才推送)
         */
        if (0 == organizeId) {
            if (orderClubMapper.getProductOrder(mainOrder.getOrderId()).contains(6060)) {
                if (StringUtils.isNotBlank(orderParamBo.getBindMobile()) && 0 == mainOrder.getRebateFlag() && 1 == mainOrder.getOrderType()) {
                    String shortLink = remoteCallService.getShortLink(8, 3, domain + "/user/order/detail.html?orderId=" + mainOrder.getOrderId());
                    String name = orderProductList.get(0).getName();
                    if (name.length() > 10) {
                        name = name.substring(0, 10);
                    }
                    String content = "您已成功下单“" + name + "...”等" + mainOrder.getProductCount() + "件商品，订单编号：" + mainOrder.getOrderNo() + "，订单等待支付，支付完成后采美将尽快安排发货。" + "您可关注采美公众号或者访问采美微信小程序和网站查看并支付订单。平台公众号：微信搜索“采美365网”; 微信小程序：微信搜索“采美采购商城”；网址：www.caimei365.com/t/" + shortLink;
                    boolean sendSms = remoteCallService.getSendSms(1, 3, orderParamBo.getBindMobile(), content);
                    if (!sendSms) {
                        log.info("下单推送失败，orderId>>>>" + mainOrder.getOrderId());
                    }
                }
            }
        }
        /*
         * 构造返回参数
         */
        Map<String, Object> info = new HashMap<>();
        info.put("orderId", mainOrder.getOrderId());
        info.put("orderNo", mainOrder.getOrderNo());
        info.put("orderMark", "#" + mainOrder.getOrderId() + "#");
        //应付订单金额
        info.put("payTotalFee", mainOrder.getPayTotalFee());
        //线上线下按钮onlinePayFlag:0线上，1线下
        info.put("onlinePayFlag", onlinePayFlag);
        //真实需要付款金额
        info.put("payableAmount", mainOrder.getPayableAmount());
        if (isPaySuccessFlag) {
            // 1提交成功（支付完成）
            info.put("code", 1);
            info.put("msg", "提交成功且已支付完成!");
        } else {
            //2提交成功(未支付)
            info.put("code", 2);
            info.put("msg", "提交成功但未支付!");
        }
        // 统计销量
        remoteCallService.productSaleUpdate(salesInfo.toString());
        // 返回结果
        return ResponseJson.success("操作成功!", info);
    }

    private List<OrderShopPo> findSplitShopOrder(List<OrderProductPo> orderProductList, List<OrderShopPo> shopOrderList, List<OrderShopPo> shops, OrderParamBo orderParamBo) {
        //统计拆分的shop
        //List<OrderShopPo> collect = shops.stream().filter(s -> shops.stream().filter(so -> so.getShopId().equals(s.getShopId())).count() > 1).collect(Collectors.toList());
        //对拆分的shop重新计算子订单佣金，应付，不计入店铺满减
        ArrayList<OrderShopPo> orderShopPos = new ArrayList<>();
        for (OrderShopPo shop : shops) {
            // 供应商名称
            String shopName = baseMapper.getShopNameById(shop.getShopId());
            /*
             * 初始化子订单
             */
            List<OrderShopPo> oldShop = shopOrderList.stream().filter(s -> s.getShopId().equals(shop.getShopId())).collect(Collectors.toList());
            OrderShopPo shopOrder = new OrderShopPo();
            shopOrder.setIsColdChina(oldShop.get(0).getIsColdChina());
            shopOrder.setSplitCode(shop.getSplitCode());
            shopOrder.setShopId(shop.getShopId());
            shopOrder.setShopName(shopName);
            shopOrder.setNote(oldShop.get(0).getNote());
            // 用户Id
            shopOrder.setUserId(oldShop.get(0).getUserId());
            shopOrder.setClubId(oldShop.get(0).getClubId());
            if (null != oldShop.get(0).getOrderPromotionsId()) {
                shopOrder.setOrderPromotionsId(oldShop.get(0).getOrderPromotionsId());
            }
            if (3 == orderParamBo.getCartType()) {
                // 协销订单
                shopOrder.setSpId(orderParamBo.getBuyUserId());
            }
            // 商品总数量
            AtomicInteger shopProductCount = new AtomicInteger(0);
            // 佣金 =  应付采美
            AtomicDouble brokerage = new AtomicDouble(0);
            // 需要支付金额
            AtomicDouble needPayAmount = new AtomicDouble(0);
            // 商品总金额
            AtomicDouble shopAmount = new AtomicDouble(0);
            // 付供应商 商品费
            AtomicDouble shopProductAmount = new AtomicDouble(0);
            // 付供应商税费
            AtomicDouble shopTaxFee = new AtomicDouble(0);
            // 商品费
            AtomicDouble shopProductFee = new AtomicDouble(0);
            // 该供应商下该分帐号的商品
            List<OrderProductPo> shopProducts = orderProductList.stream().filter(po -> po.getSplitCode().equals(shop.getSplitCode()) && po.getShopId().equals(shop.getShopId())).collect(Collectors.toList());
            // 遍历所有商品
            for (OrderProductPo product : shopProducts) {
                /*
                 * 子订单金额计算
                 */
                // 供应商商品数量
                shopProductCount.set(shopProductCount.get() + product.getNum());
                // 佣金 = 应付采美
                brokerage.set(MathUtil.add(brokerage.get(), product.getCmFee()).doubleValue());
                // 需要支付金额 shouldPayFee +运费
                needPayAmount.set(MathUtil.add(needPayAmount.get(), product.getRealProductFee()).doubleValue());
                // 统计 总金额 包括税费
                shopProductFee.set(MathUtil.add(shopProductFee.get(), product.getRealProductFee()).doubleValue());

                // 商品总金额
                shopAmount.set(MathUtil.add(shopAmount.get(), product.getTotalAmount()).doubleValue());
                // 付供应商 商品费
                shopProductAmount.set(MathUtil.add(shopProductAmount.get(), product.getShopProductAmount()).doubleValue());
                // 付供应商税费
                shopTaxFee.set(MathUtil.add(shopTaxFee.get(), product.getShouldPayTotalTax()).doubleValue());
            }
            // 需要支付金额 需支付金额 + 运费
            needPayAmount.set(MathUtil.add(needPayAmount.get(), oldShop.get(0).getShopPostFee()).doubleValue());
            // 总金额 + 运费
            shopProductFee.set(MathUtil.add(shopProductFee.get(), oldShop.get(0).getShopPostFee()).doubleValue());
            // 佣金 采美应收
            shopOrder.setBrokerage(brokerage.get());
            // 可退款金额 = 余额抵扣金额
            shopOrder.setCanRefundAmount(needPayAmount.get());
            // 购买商品数
            shopOrder.setItemCount(shopProductCount.get());
            // 总价(含税) = totalFee
            shopOrder.setTotalAmount(shopProductFee.get());
            // 总金额 = 订单商品totalAmount
            shopOrder.setProductAmount(shopAmount.get());
            // 需要支付金额 shouldPayFee +运费
            shopOrder.setNeedPayAmount(needPayAmount.get());
            // 付供应商 商品费=成本价*(购买数量  + 赠品数量)
            shopOrder.setShopProductAmount(shopProductAmount.get());
            // 付给供应商税费
            shopOrder.setShopTaxFee(shopTaxFee.get());
            // 付供应商 = 商品费 + 运费
            // 暂时不加 税费
            shopOrder.setShouldPayShopAmount(MathUtil.add(shopProductAmount.get(), oldShop.get(0).getShopPostFee()).doubleValue());
            // 付给供应商运费
            shopOrder.setShopPostFee(oldShop.get(0).getShopPostFee());
            // 运费类型
            shopOrder.setShopPostFlag(oldShop.get(0).getShopPostFlag());
            // 已付供应商金额
            shopOrder.setPayedShopAmount(0d);
            // (付款供应商)付款状态：1待付款、2部分付款、3已付款
            shopOrder.setPayStatus(1);
            if (3 == orderParamBo.getCartType()) {
                // 协销订单
                shopOrder.setOrderType(0);
            } else {
                // 普通订单
                shopOrder.setOrderType(1);
            }
            // 余额抵扣金额
            shopOrder.setAccountAmount(oldShop.get(0).getAccountAmount());
            // 是否使用余额抵扣
            shopOrder.setUseBalanceFlag(oldShop.get(0).getUseBalanceFlag());
            orderShopPos.add(shopOrder);
        }
        return orderShopPos;
    }

    /**
     * 生成充值商品订单
     *
     * @param rechargeDto {
     *                    serviceProviderId 协销id
     *                    clubId            机构Id
     *                    name              商品名称
     *                    price             价格
     *                    note              备注
     *                    }
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseJson<Map<String, Object>> generateRechargeOrder(RechargeDto rechargeDto) {
        // 获取机构用户Id
        Integer clubUserId = baseMapper.getUserIdByClubId(rechargeDto.getClubId());
        if (null == clubUserId || clubUserId == 0) {
            return ResponseJson.error("机构用户信息异常！", null);
        }
        // 获取协销用户Id
        Integer spUserId = submitMapper.getServiceProviderUserId(rechargeDto.getServiceProviderId());
        if (null == spUserId || spUserId == 0) {
            return ResponseJson.error("协销用户异常！", null);
        }
        log.info("******************** 提交充值订单逻辑处理 start *******************");
        /*
         * 初始化主订单
         */
        OrderPo mainOrder = new OrderPo();
        mainOrder.setOrganizeId(0);
        //协销订金订单默认不可见
        mainOrder.setOrderSeen(2);
        mainOrder.setSpId(rechargeDto.getServiceProviderId());
        // 订单来源
        mainOrder.setOrderSource(6);
        // 订单号
        String orderNo = CodeUtil.generateOrderNo(6);
        mainOrder.setOrderNo(orderNo);
        // 用户Id
        mainOrder.setClubId(rechargeDto.getClubId());
        mainOrder.setUserId(clubUserId);
        mainOrder.setBuyUserId(spUserId);
        // 订单提交时间
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        mainOrder.setOrderTime(curDateStr);
        mainOrder.setUpdateDate(curDateStr);
        // 订单状态 0 有效  其它无效
        mainOrder.setDelFlag(0);
        mainOrder.setOrderType(0);
        mainOrder.setOrderSubmitType(3);
        mainOrder.setConfirmFlag(0);
        mainOrder.setPayFlag(0);
        mainOrder.setOnlinePayFlag(1);
        // 订单提交时间
        mainOrder.setOrderTime(curDateStr);
        // 订单不可以拆分
        mainOrder.setSplitFlag(0);
        // 发票类型
        mainOrder.setInvoiceFlag(0);
        mainOrder.setReceiptStatus(1);
        mainOrder.setPayStatus(1);
        mainOrder.setZeroCostFlag(0);
        mainOrder.setSendOutStatus(1);
        mainOrder.setRefundType(0);
        // 是否包含活动商品(受订单未支付自动关闭时间影响)  0 否 1 是
        mainOrder.setHasActProduct(0);
        // 是否确认付款供应商
        mainOrder.setAffirmPaymentFlag(0);
        // 设置是否是二手订单
        mainOrder.setSecondHandOrderFlag(0);
        mainOrder.setPromotionFullReduction(0d);
        // 商品总数量
        mainOrder.setProductCount(1);
        // 赠品数量
        mainOrder.setPresentCount(0);
        //促销赠品数量
        mainOrder.setPromotionalGiftsCount(0);
        // 0包邮 -1到付 1 有运费
        mainOrder.setPostageFlag(0);
        mainOrder.setPostage(0d);
        // 商品总额
        mainOrder.setProductTotalFee(rechargeDto.getPrice());
        mainOrder.setOrderTotalFee(rechargeDto.getPrice());
        mainOrder.setPayTotalFee(rechargeDto.getPrice());
        mainOrder.setCouponAmount(0d);
        // 订单状态
        mainOrder.setStatus(0);
        // 余额支付金额
        mainOrder.setBalancePayFee(0d);
        // 实际支付金额(商品金额+运费-余额抵扣)
        mainOrder.setPayableAmount(rechargeDto.getPrice());
        // 售后条款
        mainOrder.setClauseId(1);
        mainOrder.setClauseName("无条款");
        // 是否返佣订单
        mainOrder.setRebateFlag(0);
        /*
         * 保存主订单数据，返回订单Id
         */
        submitMapper.insertMainOrder(mainOrder);
        log.info("【提交充值订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增主订单(insert[cm_order])orderId：" + mainOrder.getOrderId());
        // 获取数据库商品信息
        OrderProductPo rechargeProduct = submitMapper.getRechargeProductDetails(6060);
        // 供应商名称
        String shopName = baseMapper.getShopNameById(rechargeProduct.getShopId());
        /*
         * 初始化子订单
         */
        OrderShopPo rechargeShopOrder = new OrderShopPo();
        // 子订单编号
        rechargeShopOrder.setShopOrderNo(mainOrder.getOrderNo() + "01");
        rechargeShopOrder.setOrderNo(mainOrder.getOrderNo());
        rechargeShopOrder.setOrderId(mainOrder.getOrderId());
        rechargeShopOrder.setShopId(rechargeProduct.getShopId());
        rechargeShopOrder.setShopName(shopName);
        // 用户Id
        rechargeShopOrder.setClubId(mainOrder.getClubId());
        rechargeShopOrder.setUserId(mainOrder.getUserId());
        // 协销订单
        rechargeShopOrder.setSpId(rechargeDto.getServiceProviderId());
        rechargeShopOrder.setOrderSubmitType(mainOrder.getOrderSubmitType());
        rechargeShopOrder.setItemCount(1);
        rechargeShopOrder.setProductAmount(rechargeDto.getPrice());
        rechargeShopOrder.setTotalAmount(rechargeDto.getPrice());
        rechargeShopOrder.setNeedPayAmount(rechargeDto.getPrice());
        rechargeShopOrder.setOrderTime(mainOrder.getOrderTime());
        rechargeShopOrder.setCanRefundAmount(0d);
        rechargeShopOrder.setShopPostFee(0d);
        rechargeShopOrder.setSplitFlag(0);
        rechargeShopOrder.setOrderType(0);
        /*
         * 保存子订单
         */
        submitMapper.insertShopOrder(rechargeShopOrder);
        log.info("【提交充值订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增子订单(insert[cm_shop_order])shopOrderId：" + rechargeShopOrder.getShopOrderId() + "，orderId：" + mainOrder.getOrderId());
        /*
         * 更新主订单的子订单Id信息,如:1000,1002
         */
        if (null != rechargeShopOrder.getShopOrderId()) {
            mainOrder.setShopOrderIds(rechargeShopOrder.getShopOrderId().toString());
            submitMapper.updateShopOrderIds(rechargeShopOrder.getShopOrderId().toString(), mainOrder.getOrderId());
        } else {
            // 设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("生成子订单异常！", null);
        }
        /*
         * 订单商品
         */
        rechargeProduct.setOrderNo(mainOrder.getOrderNo());
        rechargeProduct.setOrderId(mainOrder.getOrderId());
        rechargeProduct.setShopOrderId(rechargeShopOrder.getShopOrderId());
        rechargeProduct.setShopOrderNo(rechargeShopOrder.getShopOrderNo());
        rechargeProduct.setNum(1);
        rechargeProduct.setPresentNum(0);
        rechargeProduct.setPrice(rechargeDto.getPrice());
        rechargeProduct.setNormalPrice(rechargeDto.getPrice());
        rechargeProduct.setTotalAmount(rechargeDto.getPrice());
        rechargeProduct.setTotalFee(rechargeDto.getPrice());
        rechargeProduct.setShouldPayFee(rechargeDto.getPrice());
        rechargeProduct.setDiscount(100d);
        rechargeProduct.setDiscountPrice(rechargeDto.getPrice());
        rechargeProduct.setTaxRate(100d);
        rechargeProduct.setAddedValueTax(0d);
        rechargeProduct.setTotalAddedValueTax(0d);
        rechargeProduct.setShopFee(0d);
        rechargeProduct.setOtherFee(0d);
        rechargeProduct.setCmFee(rechargeDto.getPrice());
        rechargeProduct.setSingleShopFee(0d);
        rechargeProduct.setSingleOtherFee(0d);
        rechargeProduct.setSingleCmFee(rechargeDto.getPrice());
        rechargeProduct.setNotOutStore(0);
        rechargeProduct.setBuyAgainFlag(0);
        /*
         * 保存 订单商品
         */
        submitMapper.insertOrderProduct(rechargeProduct);
        log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>保存订单商品(insert[cm_order_product])OrderProductId：" + rechargeProduct.getOrderProductId() + "，orderId：" + mainOrder.getOrderId());

        log.info("******************** 提交充值订单逻辑处理 end *******************");

        /*
         * 构造返回参数
         */
        Map<String, Object> info = new HashMap<>();
        info.put("orderId", String.valueOf(mainOrder.getOrderId()));
        info.put("orderNo", String.valueOf(mainOrder.getOrderNo()));
        info.put("orderMark", "#" + mainOrder.getOrderId() + "#");
        //应付订单金额
        info.put("payTotalFee", String.valueOf(mainOrder.getPayTotalFee()));
        //真实需要付款金额
        info.put("payableAmount", String.valueOf(mainOrder.getPayableAmount()));
        return ResponseJson.success(info);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseJson updateOrder(List<OrderProductVo> orderProducts) {
        /**
         * 仅未付款可修改订单
         * 当前版本仅修改订单商品价格
         * 余额抵扣修改时，未确认修改抵扣记录抵扣金额，已确认则根据差价修改抵扣记录，并返还余额/或再抵扣
         */

        ShopOrderVo shopOrder = shipMapper.getShopOrder(orderProducts.get(0).getShopOrderId());
        Integer identity = baseMapper.getIdentityByUserId(shopOrder.getUserId());

        // 商品总数量
        AtomicInteger shopProductCount = new AtomicInteger(0);
        // 佣金 =  应付采美
        AtomicDouble brokerage = new AtomicDouble(0);
        // 需要支付金额
        AtomicDouble needPayAmount = new AtomicDouble(0);
        // 商品总金额
        AtomicDouble shopAmount = new AtomicDouble(0);
        // 付供应商 商品费
        AtomicDouble shopProductAmount = new AtomicDouble(0);
        // 付供应商税费
        AtomicDouble shopTaxFee = new AtomicDouble(0);
        // 商品费
        AtomicDouble shopProductFee = new AtomicDouble(0);
        // 子订单应付供应商
        AtomicReference<Double> shouldPayShopAmount = new AtomicReference<>(0d);
        // 订单商品列表
        List<OrderProductPo> orderProductList = new ArrayList<>();

        for (OrderProductVo orderProduct : orderProducts) {
            Integer skuId = orderProduct.getSkuId();
            Integer productNum = orderProduct.getNum();
            Integer presentNum = orderProduct.getPresentNum();
            Integer giftType = orderProduct.getGiftType();
            if (null == skuId) {
                return ResponseJson.error("订单商品数据异常！", null);
            }
            if (null == productNum || productNum == 0) {
                return ResponseJson.error("商品购买数量异常！", null);
            }
            // 获取数据库商品信息
            OrderProductPo dbProduct = null;
            if (0 == orderProduct.getOrganizeId()) {
                if (identity == 8) {
                    //分销人员商品价格
                    dbProduct = submitMapper.getDistributionProductDetails(skuId);
                } else {
                    dbProduct = submitMapper.getProductDetails(skuId);
                }
            } else {
                dbProduct = submitMapper.getProductOrganizeDetails(skuId);
            }
            OrderProductPo product = new OrderProductPo();
            BeanUtils.copyProperties(dbProduct, product);
            product.setOrderProductId(orderProduct.getOrderProductId());
            product.setPrice(orderProduct.getPrice());

            if (null == product) {
                return ResponseJson.error("订单商品不存在！", null);
            }

            // 是否添加税费，不含税商品 开票需添加税费
            boolean taxFlag = false;
            if (0 == orderProduct.getOrganizeId()) {
                taxFlag = (Integer.valueOf(0).equals(product.getIncludedTax()) && (Integer.valueOf(1).equals(product.getInvoiceType()) || Integer.valueOf(2).equals(product.getInvoiceType())));
            }
            Double shopTax = 0d;

            // 是否是促销赠品
            if (giftType == 2) {
                // 促销赠品数+1
                product.setPrice(0d);
                product.setDiscountPrice(0d);
                product.setDiscount(0d);
                product.setTotalAmount(0d);
                product.setTotalFee(0d);
                product.setTaxRate(0d);
                product.setAddedValueTax(0d);
                product.setTotalAddedValueTax(0d);
            } else {
                // 不是赠品，获取商品购买价格(超级会员优惠商品价格>>>活动价格>>>阶梯价格>>>复购价格库>>>商品原始价)
                Double productPrice = product.getPrice();
                Double discountPrice = product.getPrice();
                // 商品税费
                Double productTax = 0d;
                Double discountTax = 0d;
                /**
                 * 修改订单取消所有优惠，按修改金额为准
                 */
                if (MathUtil.compare(discountPrice, BigDecimal.ZERO) == 0) {
                    return ResponseJson.error("商品购买价格不能为0！", null);
                }
                // 不含税可开票商品计算税费
                if (null == product.getTaxRate() || product.getTaxRate() <= 0) {
                    product.setTaxRate(0d);
                }
                if (taxFlag) {
                    productTax = MathUtil.div(MathUtil.mul(productPrice, product.getTaxRate()), 100, 2).doubleValue();
                    discountTax = MathUtil.div(MathUtil.mul(discountPrice, product.getTaxRate()), 100, 2).doubleValue();
                } else if (!Integer.valueOf(1).equals(product.getIncludedTax())) {
                    // 不含税不可开票商品和未知商品，税率置为0
                    product.setTaxRate(0d);
                }
                // 商品价格
                product.setPrice(productPrice);
                // 商品税后价格
                Double productTaxPrice = MathUtil.add(productPrice, productTax).doubleValue();
                // 折后税后单价
                Double discountTaxPrice = MathUtil.add(discountPrice, discountTax).doubleValue();
                // 折扣率 = 折后单价/机构价
                Double discountRate = MathUtil.mul(MathUtil.div(discountTaxPrice, productTaxPrice), 100).doubleValue();
                // 单个商品的金额
                Double productAmount = MathUtil.mul(productTaxPrice, productNum).doubleValue();
                // 单个商品的折后金额
                Double productFee = MathUtil.mul(discountTaxPrice, productNum).doubleValue();
                // 不包含单品满减的商品金额
                Double realProductFee = productFee;
                // 单个商品总税费
                Double taxFee = MathUtil.mul(discountTax, productNum).doubleValue();
                // 折后单价
                product.setDiscountPrice(discountPrice);
                // 折扣率 = 折后单价/机构价
                product.setDiscount(discountRate);
                // 单个商品的金额
                product.setTotalAmount(productAmount);
                // 单个商品的折后金额
                product.setTotalFee(productFee);
                product.setShouldPayFee(productFee);
                product.setRealProductFee(realProductFee);
                // 税费
                product.setAddedValueTax(discountTax);
                product.setTotalAddedValueTax(taxFee);
            }

            // 成本价
            Double costPrice = null;
            // 判断是否选中固定成本价
            if (null != product.getCostPrice() && 1 == product.getCostCheckFlag() && product.getCostPrice() >= 0d) {
                // 通过售价*比例得到成本价
                double costTax = 0d;
                if (taxFlag) {
                    costTax = MathUtil.div(MathUtil.mul(product.getCostPrice(), product.getTaxRate()), 100, 2).doubleValue();
                }
                costPrice = MathUtil.add(product.getCostPrice(), costTax).doubleValue();
            }
            // 判断是否选中比例成本价
            if (null != product.getCostProportional() && 2 == product.getCostCheckFlag() && product.getCostProportional() > 0d) {
                // 通过售价*比例得到成本价
                double discountTax = 0d;
                if (taxFlag) {
                    discountTax = MathUtil.div(MathUtil.mul(product.getDiscountPrice(), product.getTaxRate()), 100, 2).doubleValue();
                }
                costPrice = MathUtil.div(MathUtil.mul(MathUtil.add(product.getDiscountPrice(), discountTax), product.getCostProportional()), 100, 2).doubleValue();
            }
            if (null == costPrice) {
                return ResponseJson.error("订单商品成本异常！", null);
            }
            product.setCostPrice(costPrice);
            // 付供应商税费
            if (null == product.getShopTaxRate() || product.getShopTaxRate() <= 0) {
                product.setShopTaxRate(product.getTaxRate());
            }
            if (taxFlag) {
                shopTax = MathUtil.div(MathUtil.mul(MathUtil.div(costPrice, MathUtil.div(MathUtil.add(product.getTaxRate(), 100), 100)), product.getShopTaxRate()), 100, 2).doubleValue();
            } else if (!Integer.valueOf(1).equals(product.getIncludedTax())) {
                // 不含税不可开票商品和未知商品，税率置为0
                product.setShopTaxRate(0d);
            }
            // 单个付供应商税费
            product.setSingleShouldPayTotalTax(shopTax);
            // 单个商品付供应商总税费
            product.setShouldPayTotalTax(MathUtil.mul(shopTax, productNum).doubleValue());
            // 付供应商 商品费=成本价*(购买数量  + 赠品数量)
            Double costAmount = null;
            if (0 == orderProduct.getOrganizeId()) {
                costAmount = MathUtil.mul(costPrice, MathUtil.add(productNum, presentNum)).doubleValue();
            } else {
                costAmount = MathUtil.mul(product.getPrice(), MathUtil.add(productNum, presentNum)).doubleValue();
            }
            product.setShopProductAmount(costAmount);

            //应付供应商(单)=成本价+供应商税费(单)
            Double singleShopFee = MathUtil.add(product.getCostPrice(), shopTax).doubleValue();
            product.setSingleShopFee(singleShopFee);
            // 应付供应商(总)=应付供应商(单) * 商品数量
            Double shopFee = MathUtil.mul(singleShopFee, productNum).doubleValue();
            product.setShopFee(shopFee);
            // 付第三方，默认0
            product.setOtherFee(0d);
            product.setSingleOtherFee(0d);
            //应付采美(单)=单价+机构税费(单)-(成本(单)+供应商税费(单))
            Double singleCmFee = MathUtil.sub(MathUtil.add(BigDecimal.valueOf(product.getPrice()), product.getAddedValueTax()), MathUtil.add(product.getCostPrice(), shopTax)).doubleValue();
            product.setSingleCmFee(singleCmFee);
            // 应付采美(总)=应付采美(单)*商品数量
            Double cmFee = MathUtil.mul(singleCmFee, productNum).doubleValue();
            product.setCmFee(cmFee);
            // 默认未支付
            product.setPayStatus(0);
            // 默认非再次购买商品
            product.setBuyAgainFlag(0);
            // 未出库数量
            product.setNotOutStore(productNum);
            /*
             * 子订单金额计算
             */

            // 佣金 = 应付采美
            brokerage.set(MathUtil.add(brokerage.get(), product.getCmFee()).doubleValue());
            // 需要支付金额 shouldPayFee +运费
            needPayAmount.set(MathUtil.add(needPayAmount.get(), product.getShouldPayFee()).doubleValue());
            // 统计 总金额 包括税费
            shopProductFee.set(MathUtil.add(shopProductFee.get(), product.getTotalFee()).doubleValue());
            // 商品总金额
            shopAmount.set(MathUtil.add(shopAmount.get(), product.getTotalAmount()).doubleValue());
            // 付供应商 商品费
            log.info("shopProductAmount.get()" + shopProductAmount.get());
            log.info("product.getShopProductAmount()" + product.getShopProductAmount());
            shopProductAmount.set(MathUtil.add(shopProductAmount.get(), product.getShopProductAmount()).doubleValue());
            // 付供应商税费
            shopTaxFee.set(MathUtil.add(shopTaxFee.get(), product.getShouldPayTotalTax()).doubleValue());

            // 成本 | 佣金比例  商城sku配置 成本 | 佣金比例
            CmOrganizeSkuPo organizeSkuInfo = null;
            if (identity == 8) {
                organizeSkuInfo = submitMapper.getDistributionSkuInfo(skuId, orderProduct.getOrganizeId());
            } else {
                organizeSkuInfo = submitMapper.getOrganizeSkuInfo(skuId, orderProduct.getOrganizeId());
            }
            if (StringUtils.isNotEmpty(organizeSkuInfo.getCostCheckFlag()) && "1".equals(organizeSkuInfo.getCostCheckFlag())) {
                // 固定成本
                if (taxFlag) {
                    organizeSkuInfo.setOrganizeCostPrice(MathUtil.add(organizeSkuInfo.getOrganizeCostPrice(), MathUtil.div(MathUtil.mul(organizeSkuInfo.getOrganizeCostPrice(), product.getTaxRate()), 100)).doubleValue());
                    organizeSkuInfo.setCmCostPrice(MathUtil.add(organizeSkuInfo.getCmCostPrice(), MathUtil.div(MathUtil.mul(organizeSkuInfo.getCmCostPrice(), product.getTaxRate()), 100)).doubleValue());
                }
                product.setOrganizeCostPrice(organizeSkuInfo.getOrganizeCostPrice());
                product.setCmCostPrice(organizeSkuInfo.getCmCostPrice());
            }
            if (StringUtils.isNotEmpty(organizeSkuInfo.getCostCheckFlag()) && "2".equals(organizeSkuInfo.getCostCheckFlag())) {
                // 比例成本
                product.setOrganizeCostPrice(MathUtil.div(MathUtil.mul(product.getPrice(), organizeSkuInfo.getOrganizePercent()), 100).doubleValue());
                product.setCmCostPrice(MathUtil.div(MathUtil.mul(product.getPrice(), organizeSkuInfo.getCmPercent()), 100).doubleValue());

                product.setCmPercent(organizeSkuInfo.getCmPercent());
                product.setOrganizePercent(organizeSkuInfo.getOrganizePercent());
                product.setShopPercent(organizeSkuInfo.getShopPercent());
            }
            shouldPayShopAmount.updateAndGet(v -> v + product.getShopFee());
            // 加入订单商品列表
            orderProductList.add(product);
        }

        // 佣金 采美应收
        shopOrder.setBrokerage(brokerage.get());
        // 可退款金额 = 余额抵扣金额
        shopOrder.setCanRefundAmount(needPayAmount.get());
        // 购买商品数
        shopOrder.setItemCount(shopProductCount.get());
        // 总价(含税) = totalFee
        shopOrder.setTotalAmount(MathUtil.add(shopProductFee.get(), shopOrder.getShopPostFee()).doubleValue());
        // 总金额 = 订单商品totalAmount
        shopOrder.setProductAmount(shopAmount.get());
        // 需要支付金额 shouldPayFee +运费
        shopOrder.setNeedPayAmount(MathUtil.add(shopProductFee.get(), shopOrder.getShopPostFee()).doubleValue());
        // 付供应商 商品费=成本价*(购买数量  + 赠品数量)
        shopOrder.setShopProductAmount(shopProductAmount.get());
        // 付给供应商税费
        shopOrder.setShopTaxFee(shopTaxFee.get());
        // 付供应商 = (商品费 + 运费 + 税费)* 供应商比例
        shopOrder.setShouldPayShopAmount(shouldPayShopAmount.get());
        // 均摊优惠后子订单待支付金额
        shopOrder.setRealNeedPay(shopOrder.getNeedPayAmount());

        orderProductList.forEach(o -> submitMapper.updateOrderProduct(o));
        /**
         *  由于余额抵扣过的子订单可以修改，有收款非余额抵扣的仅需改变子订单相关应收/应付金额
         *  如果有余额抵扣，应计算余额抵扣金额，如果子订单应收总金额减少且余额抵扣金额大于应收，
         *  则退还余额给用户，同时计算子订单的收款状态修改
         */
        List<BalanceRecordPo> records = baseMapper.findShopOrderBalanceRecord(shopOrder.getShopOrderId());
        if (null != records && records.size() > 0) {
            double total = records.stream().mapToDouble(BalanceRecordPo::getAmount).sum();
            /**
             * 总抵扣金额 < 子订单应收,改变子订单状态，子订单部分收，抵扣金额>子订单应收，退余额，子订单已收
             */
            double change = MathUtil.sub(total, shopOrder.getNeedPayAmount()).doubleValue();
            if (change < 0) {
                shopOrder.setReceiptStatus(2);
            } else {
                shopOrder.setReceiptStatus(3);
                shopOrder.setAccountAmount(total);
                if (change > 0) {
                    // 修改user
                    submitMapper.updateUserMoneyByChange(shopOrder.getUserId(), change);
                    // 生成多收退余额记录
                    BalanceRecordPo balanceRecordPo = new BalanceRecordPo();
                    balanceRecordPo.setShopOrderId(shopOrder.getShopOrderId());
                    balanceRecordPo.setUserId(shopOrder.getUserId());
                    balanceRecordPo.setType(1);
                    balanceRecordPo.setBalanceType(2);
                    balanceRecordPo.setAddDate(new Date());
                    balanceRecordPo.setAmount(change);
                    balanceRecordPo.setOrderId(shopOrder.getOrderId());
                    balanceRecordPo.setRemark("修改子订单，总金额降低，退还抵扣余额");
                    balanceRecordPo.setDelFlag(0);
                    baseMapper.insertBalanceRecord(balanceRecordPo);
                    shopOrder.setReceiptAmount(shopOrder.getReceiptAmount() - change);
                }
            }
        }
        submitMapper.updateShopOrder(shopOrder);
        return ResponseJson.success();
    }


}

